// Poductivity SDK version: 0.11.0 - Built March 11, 2021 13:05:46
'use strict';

const index = require('./index-4949b76c.js');

class InvalidEpisodeParam extends Error {
  constructor(message) {
    super(message);
    this.name = 'InvalidEpisodeParam';
  }
}

function validateEpisodeParams(params) {
  if (!params) {
    throw new InvalidEpisodeParam('No parameters supplied');
  }
  if ('id' in params) {
    if (typeof params.id !== 'string' || params.id === '') {
      throw new InvalidEpisodeParam('Invalid episode ID - must be a non-empty string');
    }
  }
  else if ('feedUrl' in params || 'episodeGuid' in params) {
    if (!('feedUrl' in params) ||
      typeof params.feedUrl !== 'string' ||
      params.feedUrl === '') {
      throw new InvalidEpisodeParam('Invalid feed URL - must be a non-empty string');
    }
    if (!('episodeGuid' in params) ||
      typeof params.episodeGuid !== 'string' ||
      params.episodeGuid === '') {
      throw new InvalidEpisodeParam('Invalid episode GUID - must be a non-empty string');
    }
  }
  else {
    throw new InvalidEpisodeParam('Episode parameters not recognised');
  }
}
function sleep(milliseconds = 500) {
  return new Promise((resolve) => setTimeout(resolve, milliseconds));
}
function uuid() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
    // tslint:disable-next-line:no-bitwise
    const r = (Math.random() * 16) | 0;
    // tslint:disable-next-line:no-bitwise
    const v = c === 'x' ? r : (r & 0x3) | 0x8;
    return v.toString(16);
  });
}
function addQueryParam(url, param, value) {
  const prefix = url.indexOf('?') > -1 ? '&' : '?';
  return `${url}${prefix}${param}` + (value ? `=${value}` : '');
}
function secsToHr(secs) {
  const iSecs = (typeof secs === 'number' ? secs : parseFloat(secs)) || 0;
  const isNegative = iSecs < 0;
  const res = new Date(Math.abs(iSecs) * 1000).toISOString().substr(11, 8);
  return isNegative ? `-${res}` : res;
}
function getExtension(url) {
  return String(url).split(/[#?]/)[0].split('.').pop().trim();
}
function guessImageMimeFromPath(url) {
  switch (getExtension(url)) {
    case 'jpg':
    case 'jpeg':
      return 'image/jpeg';
    case 'png':
      return 'image/png';
    default:
      return '';
  }
}
function classList(classes) {
  return Object.entries(classes)
    .filter((entry) => entry[1])
    .map((entry) => entry[0])
    .join(' ');
}
function isHTMLAudioElement(el) {
  if (typeof HTMLAudioElement !== 'undefined') {
    return el instanceof window.HTMLAudioElement;
  }
  else {
    return !!el && 'play' in el && typeof el.play === 'function';
  }
}
/**
 * Get the local timezone offset in minutes
 */
function getTimezoneOffset() {
  return new Date().getTimezoneOffset();
}

class Logger {
  constructor(context, debug) {
    this.context = context;
    this.debug = typeof debug === 'boolean' ? debug : index.Build.isDev;
  }
  log(...params) {
    if (this.debug) {
      console.log(this.context, ...params);
    }
  }
  warn(...params) {
    console.warn(this.context, ...params);
  }
  error(...params) {
    console.error(this.context, ...params);
  }
  setDebug(debug) {
    this.debug = debug;
  }
}

exports.Logger = Logger;
exports.addQueryParam = addQueryParam;
exports.classList = classList;
exports.getTimezoneOffset = getTimezoneOffset;
exports.guessImageMimeFromPath = guessImageMimeFromPath;
exports.isHTMLAudioElement = isHTMLAudioElement;
exports.secsToHr = secsToHr;
exports.sleep = sleep;
exports.uuid = uuid;
exports.validateEpisodeParams = validateEpisodeParams;
