import { Component, h, Prop, Event, Listen } from '@stencil/core';
export class PdInputRange {
  constructor() {
    /**
     * The minimum value
     */
    this.min = 0;
    /**
     * The maximum value
     */
    this.max = 100;
    /**
     * The step between values
     */
    this.step = 1;
    /**
     * Disable the input
     */
    this.disabled = false;
  }
  get style() {
    return {
      '--min': String(this.min),
      // Never allow the max to be 0 so there's always some bar rendered
      '--max': String(this.max || 1),
      '--val': String(this.value),
    };
  }
  changed(e) {
    const value = parseFloat(e.target.value) || 0;
    this.inputRangeChanged.emit({ value });
    this.inputRangeInteracting.emit({ isSeeking: false });
  }
  startedInteracting() {
    this.inputRangeInteracting.emit({ isSeeking: true });
  }
  render() {
    return (h("input", { type: "range", class: "pd-input-range-control", min: this.min, max: this.max, step: 0.01, value: this.value, style: this.style, disabled: this.disabled }));
  }
  static get is() { return "poductivity-input-range"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-input-range.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-input-range.css"]
  }; }
  static get properties() { return {
    "value": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The input value"
      },
      "attribute": "value",
      "reflect": false
    },
    "min": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The minimum value"
      },
      "attribute": "min",
      "reflect": false,
      "defaultValue": "0"
    },
    "max": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The maximum value"
      },
      "attribute": "max",
      "reflect": false,
      "defaultValue": "100"
    },
    "step": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The step between values"
      },
      "attribute": "step",
      "reflect": false,
      "defaultValue": "1"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disable the input"
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "inputRangeChanged",
      "name": "inputRangeChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input changes"
      },
      "complexType": {
        "original": "InputRangeChangeEventData",
        "resolved": "InputRangeChangeEventData",
        "references": {
          "InputRangeChangeEventData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }, {
      "method": "inputRangeInteracting",
      "name": "inputRangeInteracting",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the user begins or ends seeking"
      },
      "complexType": {
        "original": "InputRangeInteractingEventData",
        "resolved": "InputRangeInteractingEventData",
        "references": {
          "InputRangeInteractingEventData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }]; }
  static get listeners() { return [{
      "name": "input",
      "method": "changed",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "touchStart",
      "method": "startedInteracting",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "mouseDown",
      "method": "startedInteracting",
      "target": undefined,
      "capture": false,
      "passive": true
    }]; }
}
