import { Component, Event, h, Prop, State } from '@stencil/core';
import { classList } from '../../utils/utils';
export class PdOptOutControl {
  constructor() {
    /**
     * Make the dropdown appear above the trigger
     */
    this.up = false;
    /**
     * Make the dropdown's right edge align to the trigger
     */
    this.right = false;
    this.isOpen = false;
  }
  get dropdownClasses() {
    return classList({
      'pd-dropdown-menu': true,
      up: this.up,
      right: this.right,
    });
  }
  changed(optOut) {
    this.optOutChanged.emit({ optOut });
    this.isOpen = false;
  }
  open() {
    this.isOpen = true;
  }
  close() {
    this.isOpen = false;
  }
  getOptions() {
    const options = [];
    if (this.optOut === '') {
      options.push(h("button", { class: "pd-dropdown-item", onClick: this.changed.bind(this, 'show') }, "Opt out of analytics for show"), h("button", { class: "pd-dropdown-item", onClick: this.changed.bind(this, 'listener') }, "Opt out of analytics globally"));
    }
    else {
      options.push(h("button", { class: "pd-dropdown-item", onClick: this.changed.bind(this, '') },
        "Disable ",
        this.optOut === 'show' ? 'for show' : ''));
    }
    return options;
  }
  render() {
    return (h("div", { class: "pd-dropdown-container", "aria-expanded": this.isOpen ? 'true' : 'false', "aria-haspopup": "true" },
      h("button", { class: "pd-button-link pd-opt-out-reveal pd-dropdown-toggle", title: "Privacy", onFocus: this.open, onBlur: this.close }, "Privacy"),
      h("div", { class: this.dropdownClasses }, this.getOptions())));
  }
  static get is() { return "poductivity-opt-out-control"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-opt-out-control.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-opt-out-control.css"]
  }; }
  static get properties() { return {
    "optOut": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "OptOutType",
        "resolved": "\"\" | \"listener\" | \"show\"",
        "references": {
          "OptOutType": {
            "location": "import",
            "path": "../../slot.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The current opt out status"
      },
      "attribute": "opt-out",
      "reflect": false
    },
    "up": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Make the dropdown appear above the trigger"
      },
      "attribute": "up",
      "reflect": false,
      "defaultValue": "false"
    },
    "right": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Make the dropdown's right edge align to the trigger"
      },
      "attribute": "right",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "isOpen": {}
  }; }
  static get events() { return [{
      "method": "optOutChanged",
      "name": "optOutChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the user changes their opt in status"
      },
      "complexType": {
        "original": "OptOutEventData",
        "resolved": "OptOutEventData",
        "references": {
          "OptOutEventData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }]; }
}
