import { Component, h, Prop, Event } from '@stencil/core';
import icon0_5 from './assets/0_5x.svg';
import icon1_0 from './assets/1_0x.svg';
import icon1_5 from './assets/1_5x.svg';
import icon2_0 from './assets/2_0x.svg';
import icon2_5 from './assets/2_5x.svg';
import icon3_0 from './assets/3_0x.svg';
export class PdPlaybackRateControl {
  constructor() {
    this.toggleRate = this.toggleRate.bind(this);
  }
  changed(playbackRate) {
    this.playbackRateChange.emit({ playbackRate });
  }
  toggleRate() {
    const rate = this.playbackRate < 3 ? this.playbackRate + 0.5 : 0.5;
    this.changed(rate);
  }
  /* istanbul ignore next */
  getIcon() {
    switch (this.playbackRate) {
      case 0.5:
        return icon0_5;
      case 1:
        return icon1_0;
      case 1.5:
        return icon1_5;
      case 2:
        return icon2_0;
      case 2.5:
        return icon2_5;
      case 3:
        return icon3_0;
      default:
        return String(this.playbackRate);
    }
  }
  render() {
    return (h("button", { class: "btn-playback-rate", title: "Toggle playback rate", onClick: this.toggleRate, innerHTML: this.getIcon() }));
  }
  static get is() { return "poductivity-playback-rate-control"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-playback-rate-control.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-playback-rate-control.css"]
  }; }
  static get properties() { return {
    "playbackRate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The current rate"
      },
      "attribute": "playback-rate",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "playbackRateChange",
      "name": "playbackRateChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the user changes their opt in status"
      },
      "complexType": {
        "original": "PlaybackRateEventData",
        "resolved": "PlaybackRateEventData",
        "references": {
          "PlaybackRateEventData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }]; }
}
