import { Logger } from './Logger';
import { request } from './httpUtils';
export default class ClientService {
  constructor(userAgent) {
    this.userAgent = '';
    this.currentShowId = null;
    this.currentEpisodeId = null;
    this.responseCache = [];
    this.playCache = [];
    this.userAgent = userAgent;
    this.logger = new Logger('ClientService');
  }
  setUserAgent(userAgent) {
    this.userAgent = userAgent;
  }
  get cacheDirty() {
    return this.playCache.length > 0 || this.responseCache.length > 0;
  }
  async getEpisodeData(params, deviceIdentifier) {
    var _a, _b;
    await this.flushResponseCache(deviceIdentifier);
    let data;
    let settings = {
      userAgent: this.userAgent,
    };
    if ('id' in params) {
      data = await request('GET', `v1/episodes/${params.id}/slots`, { deviceIdentifier }, {}, settings);
    }
    else {
      data = await request('GET', 'v1/slots/by-feed', Object.assign(Object.assign({}, params), { deviceIdentifier }), {}, settings);
    }
    this.currentEpisodeId = ((_a = data.episode) === null || _a === void 0 ? void 0 : _a.id) || null;
    this.currentShowId = ((_b = data.episode) === null || _b === void 0 ? void 0 : _b.showId) || null;
    return data;
  }
  async recordPlays(slotPlays) {
    this.playCache.push(...slotPlays);
    this.logger.log('Recorded slot plays', this.playCache);
  }
  async recordResponse(response) {
    // It's important new responses are added to the start of the array so this updates the correct response
    const existing = this.responseCache.findIndex((r) => r.actionId && r.actionId === response.actionId);
    if (existing !== -1) {
      this.responseCache.splice(existing, 1);
    }
    this.responseCache.unshift(response);
    this.logger.log('Recorded response', this.responseCache);
  }
  async flushResponseCache(deviceIdentifier) {
    if (this.cacheDirty) {
      const data = {
        device: {
          deviceIdentifier,
          userAgent: this.userAgent,
        },
        responses: this.responseCache,
        plays: this.playCache,
      };
      this.logger.log('Flushing response cache', data);
      const path = `v1/shows/${this.currentShowId}/episodes/${this.currentEpisodeId}/responses`;
      //https://volument.com/blog/sendbeacon-is-broken
      // if (navigator && 'sendBeacon' in navigator) {
      //   const fd = createFormData(data);
      //   navigator.sendBeacon(`${API_SERVER}/${path}`, fd);
      // } else {
      await request('POST', path, {}, data, {
        userAgent: this.userAgent,
        rawResponse: true,
      });
      // }
      this.responseCache = [];
      this.playCache = [];
      return true;
    }
    return false;
  }
}
