class FontLoader {
  constructor() {
    this.loaded = [];
  }
  isLoaded(font) {
    return (this.loaded.findIndex((comp) => comp.family === font.family &&
      comp.weight === font.weight &&
      comp.style === font.style) > -1);
  }
  load(font) {
    if (!this.isLoaded(font)) {
      const style = this.getStyleElement(font);
      document.head.append(style);
      this.markLoaded(font);
    }
  }
  getStyleElement(font) {
    const el = document.createElement('style');
    el.innerHTML = this.getFontFaceRule(font);
    return el;
  }
  getFontFaceRule(font) {
    return `
    @font-face {
      font-family: '${font.family}';
      font-style: '${font.style}';
      font-weight: ${font.weight};
      font-display: swap;
      src: url('${font.woff2Url}') format('woff2');
      ${font.unicodeRange ? `unicode-range: ${font.unicodeRange};` : ''}
    }
    `;
  }
  markLoaded(font) {
    if (!this.isLoaded(font)) {
      this.loaded.push(font);
    }
  }
}
export default new FontLoader();
