import { request } from './httpUtils';
import { Logger } from './Logger';
import { BehaviorSubject, from } from 'rxjs';
import { distinctUntilChanged, map } from 'rxjs/operators';
class OptOutService {
  constructor() {
    this.cache = {};
    this.logger = new Logger('OptOutService');
  }
  async saveOptOut(val, showId, deviceIdentifier, userAgent) {
    await request(val === '' ? 'DELETE' : 'POST', 'v1/listeners/opt-out', {}, {
      showId,
      deviceIdentifier,
      type: val,
    }, {
      userAgent,
    });
    this.logger.log('Saved opt out', val);
    const cacheKey = this.getCacheKey(deviceIdentifier, showId);
    if (cacheKey in this.cache) {
      this.cache[cacheKey].next(val);
    }
    return true;
  }
  getOptOut(deviceIdentifier, showId, userAgent) {
    const key = this.getCacheKey(deviceIdentifier, showId);
    if (!(key in this.cache)) {
      this.cache[key] = new BehaviorSubject('');
      this.fetchOptOut(deviceIdentifier, showId, userAgent).subscribe((optOut) => this.cache[key].next(optOut));
    }
    return this.cache[key].asObservable().pipe(distinctUntilChanged());
  }
  fetchOptOut(deviceIdentifier, showId, userAgent) {
    return from(request('GET', 'v1/listeners/opt-out/check', {
      showId,
      deviceIdentifier,
    }, {}, {
      userAgent,
    })).pipe(map((res) => {
      var _a;
      if (res.optOut) {
        return ((_a = res.optOut) === null || _a === void 0 ? void 0 : _a.showId) ? 'show' : 'listener';
      }
      return '';
    }));
  }
  getCacheKey(deviceIdentifier, showId) {
    return btoa(`${deviceIdentifier}.${showId}`);
  }
}
export default new OptOutService();
