import { checkMessageOrigin } from './httpUtils';
import { Logger } from './Logger';
import { addQueryParam } from './utils';
export class PopupController {
  constructor(instanceId) {
    this.resolveI = null;
    this.rejectI = null;
    this.instanceId = instanceId;
    this.logger = new Logger('Popup');
    this.onMessage = this.onMessage.bind(this);
    this.attachListeners();
  }
  async show(url) {
    this.hide(false);
    this.targetWindow = window.open(addQueryParam(url, 'instanceId', this.instanceId), 'PoductivityAction', 'height=500,width=500,scrollbars');
    if (this.targetWindow) {
      this.targetWindow.focus();
      return new Promise((resolve, reject) => {
        this.resolveI = resolve;
        this.rejectI = reject;
      });
    }
    else {
      window.alert('The action window was blocked by your pop-up blocker.');
      return new Promise((_resolve, reject) => {
        reject();
      });
    }
  }
  hide(message) {
    if (this.targetWindow) {
      this.targetWindow.close();
      if (this.resolveI && this.rejectI) {
        message ? this.resolveI(message) : this.rejectI(message);
        this.rejectI = null;
        this.resolveI = null;
      }
    }
  }
  attachListeners() {
    window.addEventListener('message', this.onMessage);
  }
  destroy() {
    window.removeEventListener('message', this.onMessage);
  }
  onMessage(e) {
    var _a;
    if (checkMessageOrigin(e.origin) &&
      ((_a = e.data) === null || _a === void 0 ? void 0 : _a.instanceId) === this.instanceId) {
      this.logger.log('Message received from popup', e.data);
      this.hide(e.data.success || false);
    }
  }
}
