import { API_SERVER, LISTENERS_SERVER } from './config';
export function checkMessageOrigin(origin) {
  return (origin && String(origin).toLowerCase() === LISTENERS_SERVER.toLowerCase());
}
export function createFormData(data) {
  const fd = new FormData();
  Object.keys(data).forEach((key) => fd.append(key, data[key]));
  return fd;
}
export async function request(type, path, queryParams = {}, data = {}, settings) {
  const fullUrl = new URL(`${API_SERVER}/${path}`);
  if (queryParams) {
    fullUrl.search = new URLSearchParams(queryParams).toString();
  }
  const response = await fetch(fullUrl.toString(), {
    method: type,
    body: ['GET', 'HEAD'].indexOf(type) === -1 && data
      ? JSON.stringify(data)
      : null,
    headers: {
      'Content-Type': 'application/json',
      'User-Agent': settings.userAgent,
    },
  });
  return settings.rawResponse ? response : response.json();
}
export function tryScheme(uri, timeout = 3000) {
  return new Promise((resolve, reject) => {
    window.location.href = uri;
    let timer = setTimeout(reject, timeout);
    window.addEventListener('visibilitychange', () => {
      clearTimeout(timer);
      resolve();
    }, { once: true });
  });
}
