import { InvalidEpisodeParam } from '../errors/invalidEpisodeParam';
export function validateEpisodeParams(params) {
  if (!params) {
    throw new InvalidEpisodeParam('No parameters supplied');
  }
  if ('id' in params) {
    if (typeof params.id !== 'string' || params.id === '') {
      throw new InvalidEpisodeParam('Invalid episode ID - must be a non-empty string');
    }
  }
  else if ('feedUrl' in params || 'episodeGuid' in params) {
    if (!('feedUrl' in params) ||
      typeof params.feedUrl !== 'string' ||
      params.feedUrl === '') {
      throw new InvalidEpisodeParam('Invalid feed URL - must be a non-empty string');
    }
    if (!('episodeGuid' in params) ||
      typeof params.episodeGuid !== 'string' ||
      params.episodeGuid === '') {
      throw new InvalidEpisodeParam('Invalid episode GUID - must be a non-empty string');
    }
  }
  else {
    throw new InvalidEpisodeParam('Episode parameters not recognised');
  }
}
export function sleep(milliseconds = 500) {
  return new Promise((resolve) => setTimeout(resolve, milliseconds));
}
export function uuid() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
    // tslint:disable-next-line:no-bitwise
    const r = (Math.random() * 16) | 0;
    // tslint:disable-next-line:no-bitwise
    const v = c === 'x' ? r : (r & 0x3) | 0x8;
    return v.toString(16);
  });
}
export function addQueryParam(url, param, value) {
  const prefix = url.indexOf('?') > -1 ? '&' : '?';
  return `${url}${prefix}${param}` + (value ? `=${value}` : '');
}
export function secsToHr(secs) {
  const iSecs = (typeof secs === 'number' ? secs : parseFloat(secs)) || 0;
  const isNegative = iSecs < 0;
  const res = new Date(Math.abs(iSecs) * 1000).toISOString().substr(11, 8);
  return isNegative ? `-${res}` : res;
}
export function getExtension(url) {
  return String(url).split(/[#?]/)[0].split('.').pop().trim();
}
export function guessImageMimeFromPath(url) {
  switch (getExtension(url)) {
    case 'jpg':
    case 'jpeg':
      return 'image/jpeg';
    case 'png':
      return 'image/png';
    default:
      return '';
  }
}
export function classList(classes) {
  return Object.entries(classes)
    .filter((entry) => entry[1])
    .map((entry) => entry[0])
    .join(' ');
}
export function isHTMLAudioElement(el) {
  if (typeof HTMLAudioElement !== 'undefined') {
    return el instanceof window.HTMLAudioElement;
  }
  else {
    return !!el && 'play' in el && typeof el.play === 'function';
  }
}
/**
 * Get the local timezone offset in minutes
 */
export function getTimezoneOffset() {
  return new Date().getTimezoneOffset();
}
export function deepCopy(inObject) {
  let outObject, value, key;
  if (typeof inObject !== 'object' || inObject === null) {
    return inObject;
  }
  outObject = Array.isArray(inObject) ? [] : {};
  for (key in inObject) {
    value = inObject[key];
    outObject[key] = deepCopy(value);
  }
  return outObject;
}
// https://github.com/JDMcKinstry/debounce
export function debounce(cb, args = null, delay = 250, obj) {
  var tmr;
  return function () {
    void 0 == obj && (obj = this);
    tmr && clearTimeout(tmr);
    args =
      args && 'object' == typeof args
        ? Array.prototype.slice.call(args)
        : void 0 != args
          ? [args]
          : arguments;
    (0 > delay || 'number' != typeof delay) && (delay = 250);
    tmr = setTimeout(function () {
      cb.apply(obj, args);
    }, delay);
  };
}
