// Poductivity SDK version: 0.11.0 - Built March 11, 2021 13:05:46
import { B as Build, r as registerInstance, c as createEvent, h, e as readTask, H as Host } from './index-d5e03b38.js';
import { L as Logger, a as sleep, b as addQueryParam, u as uuid, c as classList, v as validateEpisodeParams, i as isHTMLAudioElement, d as getTimezoneOffset } from './Logger-d43ff94a.js';

const API_SERVER =  'https://api.poductivity.com';
const LISTENERS_SERVER =  'https://listeners.poductivity.com';

function checkMessageOrigin(origin) {
  return (origin && String(origin).toLowerCase() === LISTENERS_SERVER.toLowerCase());
}
async function request(type, path, queryParams = {}, data = {}, settings) {
  const fullUrl = new URL(`${API_SERVER}/${path}`);
  if (queryParams) {
    fullUrl.search = new URLSearchParams(queryParams).toString();
  }
  const response = await fetch(fullUrl.toString(), {
    method: type,
    body: ['GET', 'HEAD'].indexOf(type) === -1 && data
      ? JSON.stringify(data)
      : null,
    headers: {
      'Content-Type': 'application/json',
      'User-Agent': settings.userAgent,
    },
  });
  return settings.rawResponse ? response : response.json();
}
function tryScheme(uri, timeout = 3000) {
  return new Promise((resolve, reject) => {
    window.location.href = uri;
    let timer = setTimeout(reject, timeout);
    window.addEventListener('visibilitychange', () => {
      clearTimeout(timer);
      resolve();
    }, { once: true });
  });
}

class DeviceService {
  constructor(debug) {
    this.debug = typeof debug === 'boolean' ? debug : Build.isDev;
    this.logger = new Logger('DeviceService', debug);
    this.logger.log('Created device class instance');
  }
  /**
   * listenerAccessToken will only be set if this is running
   * on a trusted domain - otherwise will be undefined
   */
  getDeviceInfo(maxTime = 10000) {
    return new Promise(async (resolve, reject) => {
      let exists = this.getCache();
      if (exists && !exists.working) {
        return resolve({
          deviceIdentifier: exists.deviceIdentifier,
          listenerAccessToken: exists.listenerAccessToken,
        });
      }
      if (exists && exists.working) {
        do {
          //TODO: can we do this without polling?
          await sleep(10);
          exists = this.getCache();
        } while (exists.working);
        if (exists.failed) {
          return reject('Already failed to get device identifier');
        }
        return resolve({
          deviceIdentifier: exists.deviceIdentifier,
          listenerAccessToken: exists.listenerAccessToken,
        });
      }
      this.logger.log('Initialising identifier frame');
      const idFrame = this.createIdentifierFrame();
      this.setCache({
        deviceIdentifier: '',
        working: true,
        failed: false,
      });
      const timeout = setTimeout(() => {
        this.setCache({
          deviceIdentifier: '',
          working: false,
          failed: true,
        });
        reject('Unable to get device identifier');
      }, maxTime);
      window.addEventListener('message', (e) => {
        var _a;
        if (checkMessageOrigin(e.origin) && ((_a = e.data) === null || _a === void 0 ? void 0 : _a.deviceIdentifier)) {
          clearTimeout(timeout);
          this.logger.log('Receiving local storage device info', e);
          resolve({
            deviceIdentifier: e.data.deviceIdentifier,
            listenerAccessToken: e.data.listenerAccessToken,
          });
          this.setCache({
            deviceIdentifier: e.data.deviceIdentifier,
            working: false,
            failed: false,
          });
          // document.body.removeChild(iframe);
        }
      });
      document.body.appendChild(idFrame);
    });
  }
  getCache() {
    return this.cache;
  }
  setCache(item) {
    this.cache = item;
  }
  getFrameSrc() {
    return `${LISTENERS_SERVER}/v1/devices/id-frame${this.debug ? '?debug' : ''}`;
  }
  createIdentifierFrame() {
    const iframe = document.createElement('iframe');
    iframe.src = this.getFrameSrc();
    iframe.tabIndex = -1;
    iframe.style.display = 'block';
    iframe.style.visibility = 'hidden';
    iframe.style.width = '0';
    iframe.style.height = '0';
    iframe.style.border = '0';
    iframe.setAttribute('aria-hidden', 'true');
    return iframe;
  }
}
const DeviceService$1 = new DeviceService();

class ClientService {
  constructor(userAgent) {
    this.userAgent = '';
    this.currentShowId = null;
    this.currentEpisodeId = null;
    this.responseCache = [];
    this.playCache = [];
    this.userAgent = userAgent;
    this.logger = new Logger('ClientService');
  }
  setUserAgent(userAgent) {
    this.userAgent = userAgent;
  }
  get cacheDirty() {
    return this.playCache.length > 0 || this.responseCache.length > 0;
  }
  async getEpisodeData(params, deviceIdentifier) {
    var _a, _b;
    await this.flushResponseCache(deviceIdentifier);
    let data;
    let settings = {
      userAgent: this.userAgent,
    };
    if ('id' in params) {
      data = await request('GET', `v1/episodes/${params.id}/slots`, { deviceIdentifier }, {}, settings);
    }
    else {
      data = await request('GET', 'v1/slots/by-feed', Object.assign(Object.assign({}, params), { deviceIdentifier }), {}, settings);
    }
    this.currentEpisodeId = ((_a = data.episode) === null || _a === void 0 ? void 0 : _a.id) || null;
    this.currentShowId = ((_b = data.episode) === null || _b === void 0 ? void 0 : _b.showId) || null;
    return data;
  }
  async recordPlays(slotPlays) {
    this.playCache.push(...slotPlays);
    this.logger.log('Recorded slot plays', this.playCache);
  }
  async recordResponse(response) {
    // It's important new responses are added to the start of the array so this updates the correct response
    const existing = this.responseCache.findIndex((r) => r.actionId && r.actionId === response.actionId);
    if (existing !== -1) {
      this.responseCache.splice(existing, 1);
    }
    this.responseCache.unshift(response);
    this.logger.log('Recorded response', this.responseCache);
  }
  async flushResponseCache(deviceIdentifier) {
    if (this.cacheDirty) {
      const data = {
        device: {
          deviceIdentifier,
          userAgent: this.userAgent,
        },
        responses: this.responseCache,
        plays: this.playCache,
      };
      this.logger.log('Flushing response cache', data);
      const path = `v1/shows/${this.currentShowId}/episodes/${this.currentEpisodeId}/responses`;
      //https://volument.com/blog/sendbeacon-is-broken
      // if (navigator && 'sendBeacon' in navigator) {
      //   const fd = createFormData(data);
      //   navigator.sendBeacon(`${API_SERVER}/${path}`, fd);
      // } else {
      await request('POST', path, {}, data, {
        userAgent: this.userAgent,
        rawResponse: true,
      });
      // }
      this.responseCache = [];
      this.playCache = [];
      return true;
    }
    return false;
  }
}

function isFunction(x) {
    return typeof x === 'function';
}

let _enable_super_gross_mode_that_will_cause_bad_things = false;
const config = {
    Promise: undefined,
    set useDeprecatedSynchronousErrorHandling(value) {
        if (value) {
            const error = new Error();
            console.warn('DEPRECATED! RxJS was set to use deprecated synchronous error handling behavior by code at: \n' + error.stack);
        }
        else if (_enable_super_gross_mode_that_will_cause_bad_things) {
            console.log('RxJS: Back to a better error behavior. Thank you. <3');
        }
        _enable_super_gross_mode_that_will_cause_bad_things = value;
    },
    get useDeprecatedSynchronousErrorHandling() {
        return _enable_super_gross_mode_that_will_cause_bad_things;
    },
};

function hostReportError(err) {
    setTimeout(() => { throw err; }, 0);
}

const empty = {
    closed: true,
    next(value) { },
    error(err) {
        if (config.useDeprecatedSynchronousErrorHandling) {
            throw err;
        }
        else {
            hostReportError(err);
        }
    },
    complete() { }
};

const isArray = (() => Array.isArray || ((x) => x && typeof x.length === 'number'))();

function isObject(x) {
    return x !== null && typeof x === 'object';
}

const UnsubscriptionErrorImpl = (() => {
    function UnsubscriptionErrorImpl(errors) {
        Error.call(this);
        this.message = errors ?
            `${errors.length} errors occurred during unsubscription:
${errors.map((err, i) => `${i + 1}) ${err.toString()}`).join('\n  ')}` : '';
        this.name = 'UnsubscriptionError';
        this.errors = errors;
        return this;
    }
    UnsubscriptionErrorImpl.prototype = Object.create(Error.prototype);
    return UnsubscriptionErrorImpl;
})();
const UnsubscriptionError = UnsubscriptionErrorImpl;

class Subscription {
    constructor(unsubscribe) {
        this.closed = false;
        this._parentOrParents = null;
        this._subscriptions = null;
        if (unsubscribe) {
            this._ctorUnsubscribe = true;
            this._unsubscribe = unsubscribe;
        }
    }
    unsubscribe() {
        let errors;
        if (this.closed) {
            return;
        }
        let { _parentOrParents, _ctorUnsubscribe, _unsubscribe, _subscriptions } = this;
        this.closed = true;
        this._parentOrParents = null;
        this._subscriptions = null;
        if (_parentOrParents instanceof Subscription) {
            _parentOrParents.remove(this);
        }
        else if (_parentOrParents !== null) {
            for (let index = 0; index < _parentOrParents.length; ++index) {
                const parent = _parentOrParents[index];
                parent.remove(this);
            }
        }
        if (isFunction(_unsubscribe)) {
            if (_ctorUnsubscribe) {
                this._unsubscribe = undefined;
            }
            try {
                _unsubscribe.call(this);
            }
            catch (e) {
                errors = e instanceof UnsubscriptionError ? flattenUnsubscriptionErrors(e.errors) : [e];
            }
        }
        if (isArray(_subscriptions)) {
            let index = -1;
            let len = _subscriptions.length;
            while (++index < len) {
                const sub = _subscriptions[index];
                if (isObject(sub)) {
                    try {
                        sub.unsubscribe();
                    }
                    catch (e) {
                        errors = errors || [];
                        if (e instanceof UnsubscriptionError) {
                            errors = errors.concat(flattenUnsubscriptionErrors(e.errors));
                        }
                        else {
                            errors.push(e);
                        }
                    }
                }
            }
        }
        if (errors) {
            throw new UnsubscriptionError(errors);
        }
    }
    add(teardown) {
        let subscription = teardown;
        if (!teardown) {
            return Subscription.EMPTY;
        }
        switch (typeof teardown) {
            case 'function':
                subscription = new Subscription(teardown);
            case 'object':
                if (subscription === this || subscription.closed || typeof subscription.unsubscribe !== 'function') {
                    return subscription;
                }
                else if (this.closed) {
                    subscription.unsubscribe();
                    return subscription;
                }
                else if (!(subscription instanceof Subscription)) {
                    const tmp = subscription;
                    subscription = new Subscription();
                    subscription._subscriptions = [tmp];
                }
                break;
            default: {
                throw new Error('unrecognized teardown ' + teardown + ' added to Subscription.');
            }
        }
        let { _parentOrParents } = subscription;
        if (_parentOrParents === null) {
            subscription._parentOrParents = this;
        }
        else if (_parentOrParents instanceof Subscription) {
            if (_parentOrParents === this) {
                return subscription;
            }
            subscription._parentOrParents = [_parentOrParents, this];
        }
        else if (_parentOrParents.indexOf(this) === -1) {
            _parentOrParents.push(this);
        }
        else {
            return subscription;
        }
        const subscriptions = this._subscriptions;
        if (subscriptions === null) {
            this._subscriptions = [subscription];
        }
        else {
            subscriptions.push(subscription);
        }
        return subscription;
    }
    remove(subscription) {
        const subscriptions = this._subscriptions;
        if (subscriptions) {
            const subscriptionIndex = subscriptions.indexOf(subscription);
            if (subscriptionIndex !== -1) {
                subscriptions.splice(subscriptionIndex, 1);
            }
        }
    }
}
Subscription.EMPTY = (function (empty) {
    empty.closed = true;
    return empty;
}(new Subscription()));
function flattenUnsubscriptionErrors(errors) {
    return errors.reduce((errs, err) => errs.concat((err instanceof UnsubscriptionError) ? err.errors : err), []);
}

const rxSubscriber = (() => typeof Symbol === 'function'
    ? Symbol('rxSubscriber')
    : '@@rxSubscriber_' + Math.random())();

class Subscriber extends Subscription {
    constructor(destinationOrNext, error, complete) {
        super();
        this.syncErrorValue = null;
        this.syncErrorThrown = false;
        this.syncErrorThrowable = false;
        this.isStopped = false;
        switch (arguments.length) {
            case 0:
                this.destination = empty;
                break;
            case 1:
                if (!destinationOrNext) {
                    this.destination = empty;
                    break;
                }
                if (typeof destinationOrNext === 'object') {
                    if (destinationOrNext instanceof Subscriber) {
                        this.syncErrorThrowable = destinationOrNext.syncErrorThrowable;
                        this.destination = destinationOrNext;
                        destinationOrNext.add(this);
                    }
                    else {
                        this.syncErrorThrowable = true;
                        this.destination = new SafeSubscriber(this, destinationOrNext);
                    }
                    break;
                }
            default:
                this.syncErrorThrowable = true;
                this.destination = new SafeSubscriber(this, destinationOrNext, error, complete);
                break;
        }
    }
    [rxSubscriber]() { return this; }
    static create(next, error, complete) {
        const subscriber = new Subscriber(next, error, complete);
        subscriber.syncErrorThrowable = false;
        return subscriber;
    }
    next(value) {
        if (!this.isStopped) {
            this._next(value);
        }
    }
    error(err) {
        if (!this.isStopped) {
            this.isStopped = true;
            this._error(err);
        }
    }
    complete() {
        if (!this.isStopped) {
            this.isStopped = true;
            this._complete();
        }
    }
    unsubscribe() {
        if (this.closed) {
            return;
        }
        this.isStopped = true;
        super.unsubscribe();
    }
    _next(value) {
        this.destination.next(value);
    }
    _error(err) {
        this.destination.error(err);
        this.unsubscribe();
    }
    _complete() {
        this.destination.complete();
        this.unsubscribe();
    }
    _unsubscribeAndRecycle() {
        const { _parentOrParents } = this;
        this._parentOrParents = null;
        this.unsubscribe();
        this.closed = false;
        this.isStopped = false;
        this._parentOrParents = _parentOrParents;
        return this;
    }
}
class SafeSubscriber extends Subscriber {
    constructor(_parentSubscriber, observerOrNext, error, complete) {
        super();
        this._parentSubscriber = _parentSubscriber;
        let next;
        let context = this;
        if (isFunction(observerOrNext)) {
            next = observerOrNext;
        }
        else if (observerOrNext) {
            next = observerOrNext.next;
            error = observerOrNext.error;
            complete = observerOrNext.complete;
            if (observerOrNext !== empty) {
                context = Object.create(observerOrNext);
                if (isFunction(context.unsubscribe)) {
                    this.add(context.unsubscribe.bind(context));
                }
                context.unsubscribe = this.unsubscribe.bind(this);
            }
        }
        this._context = context;
        this._next = next;
        this._error = error;
        this._complete = complete;
    }
    next(value) {
        if (!this.isStopped && this._next) {
            const { _parentSubscriber } = this;
            if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                this.__tryOrUnsub(this._next, value);
            }
            else if (this.__tryOrSetError(_parentSubscriber, this._next, value)) {
                this.unsubscribe();
            }
        }
    }
    error(err) {
        if (!this.isStopped) {
            const { _parentSubscriber } = this;
            const { useDeprecatedSynchronousErrorHandling } = config;
            if (this._error) {
                if (!useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                    this.__tryOrUnsub(this._error, err);
                    this.unsubscribe();
                }
                else {
                    this.__tryOrSetError(_parentSubscriber, this._error, err);
                    this.unsubscribe();
                }
            }
            else if (!_parentSubscriber.syncErrorThrowable) {
                this.unsubscribe();
                if (useDeprecatedSynchronousErrorHandling) {
                    throw err;
                }
                hostReportError(err);
            }
            else {
                if (useDeprecatedSynchronousErrorHandling) {
                    _parentSubscriber.syncErrorValue = err;
                    _parentSubscriber.syncErrorThrown = true;
                }
                else {
                    hostReportError(err);
                }
                this.unsubscribe();
            }
        }
    }
    complete() {
        if (!this.isStopped) {
            const { _parentSubscriber } = this;
            if (this._complete) {
                const wrappedComplete = () => this._complete.call(this._context);
                if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                    this.__tryOrUnsub(wrappedComplete);
                    this.unsubscribe();
                }
                else {
                    this.__tryOrSetError(_parentSubscriber, wrappedComplete);
                    this.unsubscribe();
                }
            }
            else {
                this.unsubscribe();
            }
        }
    }
    __tryOrUnsub(fn, value) {
        try {
            fn.call(this._context, value);
        }
        catch (err) {
            this.unsubscribe();
            if (config.useDeprecatedSynchronousErrorHandling) {
                throw err;
            }
            else {
                hostReportError(err);
            }
        }
    }
    __tryOrSetError(parent, fn, value) {
        if (!config.useDeprecatedSynchronousErrorHandling) {
            throw new Error('bad call');
        }
        try {
            fn.call(this._context, value);
        }
        catch (err) {
            if (config.useDeprecatedSynchronousErrorHandling) {
                parent.syncErrorValue = err;
                parent.syncErrorThrown = true;
                return true;
            }
            else {
                hostReportError(err);
                return true;
            }
        }
        return false;
    }
    _unsubscribe() {
        const { _parentSubscriber } = this;
        this._context = null;
        this._parentSubscriber = null;
        _parentSubscriber.unsubscribe();
    }
}

function canReportError(observer) {
    while (observer) {
        const { closed, destination, isStopped } = observer;
        if (closed || isStopped) {
            return false;
        }
        else if (destination && destination instanceof Subscriber) {
            observer = destination;
        }
        else {
            observer = null;
        }
    }
    return true;
}

function toSubscriber(nextOrObserver, error, complete) {
    if (nextOrObserver) {
        if (nextOrObserver instanceof Subscriber) {
            return nextOrObserver;
        }
        if (nextOrObserver[rxSubscriber]) {
            return nextOrObserver[rxSubscriber]();
        }
    }
    if (!nextOrObserver && !error && !complete) {
        return new Subscriber(empty);
    }
    return new Subscriber(nextOrObserver, error, complete);
}

const observable = (() => typeof Symbol === 'function' && Symbol.observable || '@@observable')();

function identity(x) {
    return x;
}

function pipeFromArray(fns) {
    if (fns.length === 0) {
        return identity;
    }
    if (fns.length === 1) {
        return fns[0];
    }
    return function piped(input) {
        return fns.reduce((prev, fn) => fn(prev), input);
    };
}

class Observable {
    constructor(subscribe) {
        this._isScalar = false;
        if (subscribe) {
            this._subscribe = subscribe;
        }
    }
    lift(operator) {
        const observable = new Observable();
        observable.source = this;
        observable.operator = operator;
        return observable;
    }
    subscribe(observerOrNext, error, complete) {
        const { operator } = this;
        const sink = toSubscriber(observerOrNext, error, complete);
        if (operator) {
            sink.add(operator.call(sink, this.source));
        }
        else {
            sink.add(this.source || (config.useDeprecatedSynchronousErrorHandling && !sink.syncErrorThrowable) ?
                this._subscribe(sink) :
                this._trySubscribe(sink));
        }
        if (config.useDeprecatedSynchronousErrorHandling) {
            if (sink.syncErrorThrowable) {
                sink.syncErrorThrowable = false;
                if (sink.syncErrorThrown) {
                    throw sink.syncErrorValue;
                }
            }
        }
        return sink;
    }
    _trySubscribe(sink) {
        try {
            return this._subscribe(sink);
        }
        catch (err) {
            if (config.useDeprecatedSynchronousErrorHandling) {
                sink.syncErrorThrown = true;
                sink.syncErrorValue = err;
            }
            if (canReportError(sink)) {
                sink.error(err);
            }
            else {
                console.warn(err);
            }
        }
    }
    forEach(next, promiseCtor) {
        promiseCtor = getPromiseCtor(promiseCtor);
        return new promiseCtor((resolve, reject) => {
            let subscription;
            subscription = this.subscribe((value) => {
                try {
                    next(value);
                }
                catch (err) {
                    reject(err);
                    if (subscription) {
                        subscription.unsubscribe();
                    }
                }
            }, reject, resolve);
        });
    }
    _subscribe(subscriber) {
        const { source } = this;
        return source && source.subscribe(subscriber);
    }
    [observable]() {
        return this;
    }
    pipe(...operations) {
        if (operations.length === 0) {
            return this;
        }
        return pipeFromArray(operations)(this);
    }
    toPromise(promiseCtor) {
        promiseCtor = getPromiseCtor(promiseCtor);
        return new promiseCtor((resolve, reject) => {
            let value;
            this.subscribe((x) => value = x, (err) => reject(err), () => resolve(value));
        });
    }
}
Observable.create = (subscribe) => {
    return new Observable(subscribe);
};
function getPromiseCtor(promiseCtor) {
    if (!promiseCtor) {
        promiseCtor =  Promise;
    }
    if (!promiseCtor) {
        throw new Error('no Promise impl found');
    }
    return promiseCtor;
}

const ObjectUnsubscribedErrorImpl = (() => {
    function ObjectUnsubscribedErrorImpl() {
        Error.call(this);
        this.message = 'object unsubscribed';
        this.name = 'ObjectUnsubscribedError';
        return this;
    }
    ObjectUnsubscribedErrorImpl.prototype = Object.create(Error.prototype);
    return ObjectUnsubscribedErrorImpl;
})();
const ObjectUnsubscribedError = ObjectUnsubscribedErrorImpl;

class SubjectSubscription extends Subscription {
    constructor(subject, subscriber) {
        super();
        this.subject = subject;
        this.subscriber = subscriber;
        this.closed = false;
    }
    unsubscribe() {
        if (this.closed) {
            return;
        }
        this.closed = true;
        const subject = this.subject;
        const observers = subject.observers;
        this.subject = null;
        if (!observers || observers.length === 0 || subject.isStopped || subject.closed) {
            return;
        }
        const subscriberIndex = observers.indexOf(this.subscriber);
        if (subscriberIndex !== -1) {
            observers.splice(subscriberIndex, 1);
        }
    }
}

class SubjectSubscriber extends Subscriber {
    constructor(destination) {
        super(destination);
        this.destination = destination;
    }
}
class Subject extends Observable {
    constructor() {
        super();
        this.observers = [];
        this.closed = false;
        this.isStopped = false;
        this.hasError = false;
        this.thrownError = null;
    }
    [rxSubscriber]() {
        return new SubjectSubscriber(this);
    }
    lift(operator) {
        const subject = new AnonymousSubject(this, this);
        subject.operator = operator;
        return subject;
    }
    next(value) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        if (!this.isStopped) {
            const { observers } = this;
            const len = observers.length;
            const copy = observers.slice();
            for (let i = 0; i < len; i++) {
                copy[i].next(value);
            }
        }
    }
    error(err) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        this.hasError = true;
        this.thrownError = err;
        this.isStopped = true;
        const { observers } = this;
        const len = observers.length;
        const copy = observers.slice();
        for (let i = 0; i < len; i++) {
            copy[i].error(err);
        }
        this.observers.length = 0;
    }
    complete() {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        this.isStopped = true;
        const { observers } = this;
        const len = observers.length;
        const copy = observers.slice();
        for (let i = 0; i < len; i++) {
            copy[i].complete();
        }
        this.observers.length = 0;
    }
    unsubscribe() {
        this.isStopped = true;
        this.closed = true;
        this.observers = null;
    }
    _trySubscribe(subscriber) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        else {
            return super._trySubscribe(subscriber);
        }
    }
    _subscribe(subscriber) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        else if (this.hasError) {
            subscriber.error(this.thrownError);
            return Subscription.EMPTY;
        }
        else if (this.isStopped) {
            subscriber.complete();
            return Subscription.EMPTY;
        }
        else {
            this.observers.push(subscriber);
            return new SubjectSubscription(this, subscriber);
        }
    }
    asObservable() {
        const observable = new Observable();
        observable.source = this;
        return observable;
    }
}
Subject.create = (destination, source) => {
    return new AnonymousSubject(destination, source);
};
class AnonymousSubject extends Subject {
    constructor(destination, source) {
        super();
        this.destination = destination;
        this.source = source;
    }
    next(value) {
        const { destination } = this;
        if (destination && destination.next) {
            destination.next(value);
        }
    }
    error(err) {
        const { destination } = this;
        if (destination && destination.error) {
            this.destination.error(err);
        }
    }
    complete() {
        const { destination } = this;
        if (destination && destination.complete) {
            this.destination.complete();
        }
    }
    _subscribe(subscriber) {
        const { source } = this;
        if (source) {
            return this.source.subscribe(subscriber);
        }
        else {
            return Subscription.EMPTY;
        }
    }
}

class BehaviorSubject extends Subject {
    constructor(_value) {
        super();
        this._value = _value;
    }
    get value() {
        return this.getValue();
    }
    _subscribe(subscriber) {
        const subscription = super._subscribe(subscriber);
        if (subscription && !subscription.closed) {
            subscriber.next(this._value);
        }
        return subscription;
    }
    getValue() {
        if (this.hasError) {
            throw this.thrownError;
        }
        else if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        else {
            return this._value;
        }
    }
    next(value) {
        super.next(this._value = value);
    }
}

const subscribeToArray = (array) => (subscriber) => {
    for (let i = 0, len = array.length; i < len && !subscriber.closed; i++) {
        subscriber.next(array[i]);
    }
    subscriber.complete();
};

function scheduleArray(input, scheduler) {
    return new Observable(subscriber => {
        const sub = new Subscription();
        let i = 0;
        sub.add(scheduler.schedule(function () {
            if (i === input.length) {
                subscriber.complete();
                return;
            }
            subscriber.next(input[i++]);
            if (!subscriber.closed) {
                sub.add(this.schedule());
            }
        }));
        return sub;
    });
}

function map(project, thisArg) {
    return function mapOperation(source) {
        if (typeof project !== 'function') {
            throw new TypeError('argument is not a function. Are you looking for `mapTo()`?');
        }
        return source.lift(new MapOperator(project, thisArg));
    };
}
class MapOperator {
    constructor(project, thisArg) {
        this.project = project;
        this.thisArg = thisArg;
    }
    call(subscriber, source) {
        return source.subscribe(new MapSubscriber(subscriber, this.project, this.thisArg));
    }
}
class MapSubscriber extends Subscriber {
    constructor(destination, project, thisArg) {
        super(destination);
        this.project = project;
        this.count = 0;
        this.thisArg = thisArg || this;
    }
    _next(value) {
        let result;
        try {
            result = this.project.call(this.thisArg, value, this.count++);
        }
        catch (err) {
            this.destination.error(err);
            return;
        }
        this.destination.next(result);
    }
}

const subscribeToPromise = (promise) => (subscriber) => {
    promise.then((value) => {
        if (!subscriber.closed) {
            subscriber.next(value);
            subscriber.complete();
        }
    }, (err) => subscriber.error(err))
        .then(null, hostReportError);
    return subscriber;
};

function getSymbolIterator() {
    if (typeof Symbol !== 'function' || !Symbol.iterator) {
        return '@@iterator';
    }
    return Symbol.iterator;
}
const iterator = getSymbolIterator();

const subscribeToIterable = (iterable) => (subscriber) => {
    const iterator$1 = iterable[iterator]();
    do {
        let item;
        try {
            item = iterator$1.next();
        }
        catch (err) {
            subscriber.error(err);
            return subscriber;
        }
        if (item.done) {
            subscriber.complete();
            break;
        }
        subscriber.next(item.value);
        if (subscriber.closed) {
            break;
        }
    } while (true);
    if (typeof iterator$1.return === 'function') {
        subscriber.add(() => {
            if (iterator$1.return) {
                iterator$1.return();
            }
        });
    }
    return subscriber;
};

const subscribeToObservable = (obj) => (subscriber) => {
    const obs = obj[observable]();
    if (typeof obs.subscribe !== 'function') {
        throw new TypeError('Provided object does not correctly implement Symbol.observable');
    }
    else {
        return obs.subscribe(subscriber);
    }
};

const isArrayLike = ((x) => x && typeof x.length === 'number' && typeof x !== 'function');

function isPromise(value) {
    return !!value && typeof value.subscribe !== 'function' && typeof value.then === 'function';
}

const subscribeTo = (result) => {
    if (!!result && typeof result[observable] === 'function') {
        return subscribeToObservable(result);
    }
    else if (isArrayLike(result)) {
        return subscribeToArray(result);
    }
    else if (isPromise(result)) {
        return subscribeToPromise(result);
    }
    else if (!!result && typeof result[iterator] === 'function') {
        return subscribeToIterable(result);
    }
    else {
        const value = isObject(result) ? 'an invalid object' : `'${result}'`;
        const msg = `You provided ${value} where a stream was expected.`
            + ' You can provide an Observable, Promise, Array, or Iterable.';
        throw new TypeError(msg);
    }
};

function scheduleObservable(input, scheduler) {
    return new Observable(subscriber => {
        const sub = new Subscription();
        sub.add(scheduler.schedule(() => {
            const observable$1 = input[observable]();
            sub.add(observable$1.subscribe({
                next(value) { sub.add(scheduler.schedule(() => subscriber.next(value))); },
                error(err) { sub.add(scheduler.schedule(() => subscriber.error(err))); },
                complete() { sub.add(scheduler.schedule(() => subscriber.complete())); },
            }));
        }));
        return sub;
    });
}

function schedulePromise(input, scheduler) {
    return new Observable(subscriber => {
        const sub = new Subscription();
        sub.add(scheduler.schedule(() => input.then(value => {
            sub.add(scheduler.schedule(() => {
                subscriber.next(value);
                sub.add(scheduler.schedule(() => subscriber.complete()));
            }));
        }, err => {
            sub.add(scheduler.schedule(() => subscriber.error(err)));
        })));
        return sub;
    });
}

function scheduleIterable(input, scheduler) {
    if (!input) {
        throw new Error('Iterable cannot be null');
    }
    return new Observable(subscriber => {
        const sub = new Subscription();
        let iterator$1;
        sub.add(() => {
            if (iterator$1 && typeof iterator$1.return === 'function') {
                iterator$1.return();
            }
        });
        sub.add(scheduler.schedule(() => {
            iterator$1 = input[iterator]();
            sub.add(scheduler.schedule(function () {
                if (subscriber.closed) {
                    return;
                }
                let value;
                let done;
                try {
                    const result = iterator$1.next();
                    value = result.value;
                    done = result.done;
                }
                catch (err) {
                    subscriber.error(err);
                    return;
                }
                if (done) {
                    subscriber.complete();
                }
                else {
                    subscriber.next(value);
                    this.schedule();
                }
            }));
        }));
        return sub;
    });
}

function isInteropObservable(input) {
    return input && typeof input[observable] === 'function';
}

function isIterable(input) {
    return input && typeof input[iterator] === 'function';
}

function scheduled(input, scheduler) {
    if (input != null) {
        if (isInteropObservable(input)) {
            return scheduleObservable(input, scheduler);
        }
        else if (isPromise(input)) {
            return schedulePromise(input, scheduler);
        }
        else if (isArrayLike(input)) {
            return scheduleArray(input, scheduler);
        }
        else if (isIterable(input) || typeof input === 'string') {
            return scheduleIterable(input, scheduler);
        }
    }
    throw new TypeError((input !== null && typeof input || input) + ' is not observable');
}

function from(input, scheduler) {
    if (!scheduler) {
        if (input instanceof Observable) {
            return input;
        }
        return new Observable(subscribeTo(input));
    }
    else {
        return scheduled(input, scheduler);
    }
}

function distinctUntilChanged(compare, keySelector) {
    return (source) => source.lift(new DistinctUntilChangedOperator(compare, keySelector));
}
class DistinctUntilChangedOperator {
    constructor(compare, keySelector) {
        this.compare = compare;
        this.keySelector = keySelector;
    }
    call(subscriber, source) {
        return source.subscribe(new DistinctUntilChangedSubscriber(subscriber, this.compare, this.keySelector));
    }
}
class DistinctUntilChangedSubscriber extends Subscriber {
    constructor(destination, compare, keySelector) {
        super(destination);
        this.keySelector = keySelector;
        this.hasKey = false;
        if (typeof compare === 'function') {
            this.compare = compare;
        }
    }
    compare(x, y) {
        return x === y;
    }
    _next(value) {
        let key;
        try {
            const { keySelector } = this;
            key = keySelector ? keySelector(value) : value;
        }
        catch (err) {
            return this.destination.error(err);
        }
        let result = false;
        if (this.hasKey) {
            try {
                const { compare } = this;
                result = compare(this.key, key);
            }
            catch (err) {
                return this.destination.error(err);
            }
        }
        else {
            this.hasKey = true;
        }
        if (!result) {
            this.key = key;
            this.destination.next(value);
        }
    }
}

class OptOutService {
  constructor() {
    this.cache = {};
    this.logger = new Logger('OptOutService');
  }
  async saveOptOut(val, showId, deviceIdentifier, userAgent) {
    await request(val === '' ? 'DELETE' : 'POST', 'v1/listeners/opt-out', {}, {
      showId,
      deviceIdentifier,
      type: val,
    }, {
      userAgent,
    });
    this.logger.log('Saved opt out', val);
    const cacheKey = this.getCacheKey(deviceIdentifier, showId);
    if (cacheKey in this.cache) {
      this.cache[cacheKey].next(val);
    }
    return true;
  }
  getOptOut(deviceIdentifier, showId, userAgent) {
    const key = this.getCacheKey(deviceIdentifier, showId);
    if (!(key in this.cache)) {
      this.cache[key] = new BehaviorSubject('');
      this.fetchOptOut(deviceIdentifier, showId, userAgent).subscribe((optOut) => this.cache[key].next(optOut));
    }
    return this.cache[key].asObservable().pipe(distinctUntilChanged());
  }
  fetchOptOut(deviceIdentifier, showId, userAgent) {
    return from(request('GET', 'v1/listeners/opt-out/check', {
      showId,
      deviceIdentifier,
    }, {}, {
      userAgent,
    })).pipe(map((res) => {
      var _a;
      if (res.optOut) {
        return ((_a = res.optOut) === null || _a === void 0 ? void 0 : _a.showId) ? 'show' : 'listener';
      }
      return '';
    }));
  }
  getCacheKey(deviceIdentifier, showId) {
    return btoa(`${deviceIdentifier}.${showId}`);
  }
}
const OptOutService$1 = new OptOutService();

class PopupController {
  constructor(instanceId) {
    this.resolveI = null;
    this.rejectI = null;
    this.instanceId = instanceId;
    this.logger = new Logger('Popup');
    this.onMessage = this.onMessage.bind(this);
    this.attachListeners();
  }
  async show(url) {
    this.hide(false);
    this.targetWindow = window.open(addQueryParam(url, 'instanceId', this.instanceId), 'PoductivityAction', 'height=500,width=500,scrollbars');
    if (this.targetWindow) {
      this.targetWindow.focus();
      return new Promise((resolve, reject) => {
        this.resolveI = resolve;
        this.rejectI = reject;
      });
    }
    else {
      window.alert('The action window was blocked by your pop-up blocker.');
      return new Promise((_resolve, reject) => {
        reject();
      });
    }
  }
  hide(message) {
    if (this.targetWindow) {
      this.targetWindow.close();
      if (this.resolveI && this.rejectI) {
        message ? this.resolveI(message) : this.rejectI(message);
        this.rejectI = null;
        this.resolveI = null;
      }
    }
  }
  attachListeners() {
    window.addEventListener('message', this.onMessage);
  }
  destroy() {
    window.removeEventListener('message', this.onMessage);
  }
  onMessage(e) {
    var _a;
    if (checkMessageOrigin(e.origin) &&
      ((_a = e.data) === null || _a === void 0 ? void 0 : _a.instanceId) === this.instanceId) {
      this.logger.log('Message received from popup', e.data);
      this.hide(e.data.success || false);
    }
  }
}

class ActionHandler {
  constructor(client) {
    this.client = client;
    this.popup = new PopupController(uuid());
  }
  async mediaPause() {
    this.client.controlAudio('pause');
    return true;
  }
  async mediaResume() {
    this.client.controlAudio('resume');
    return true;
  }
  async mediaSeekAbsolute(action) {
    var _a;
    this.client.controlAudio('seekAbsolute', ((_a = action.meta) === null || _a === void 0 ? void 0 : _a.timestamp) || 0);
    return true;
  }
  async mediaSeekRelative(action) {
    var _a;
    this.client.controlAudio('seekRelative', ((_a = action.meta) === null || _a === void 0 ? void 0 : _a.seconds) || 0);
    return true;
  }
  async modalFrame(action) {
    var _a;
    return await this.client.modalElement.show((_a = action.meta) === null || _a === void 0 ? void 0 : _a.frameSrc);
  }
  async modalPopup(action) {
    var _a;
    return await this.popup.show((_a = action.meta) === null || _a === void 0 ? void 0 : _a.frameSrc);
  }
  async optionGroupGoTo(action) {
    this.client.dynamicOptionGroup =
      this.client.optionGroups.find((og) => { var _a; return og.id === ((_a = action.meta) === null || _a === void 0 ? void 0 : _a.optionGroupId); }) || null;
    return true;
  }
  async optionGroupHide(action, _option, optionGroup) {
    var _a;
    return !!this.client.editOptionGroup(optionGroup.id, {
      hide: true,
      hideMessage: ((_a = action.meta) === null || _a === void 0 ? void 0 : _a.textContent) || '',
    });
  }
  async optionHide(_action, option, optionGroup) {
    return !!this.client.editOption(optionGroup.id, option.id, {
      hide: true,
    });
  }
  async externalUrl(action) {
    var _a;
    if (typeof ((_a = action.meta) === null || _a === void 0 ? void 0 : _a.url) === 'string') {
      this.client.openExternalUrl.emit({ url: action.meta.url });
      if (window === null || window === void 0 ? void 0 : window.open) {
        window.open(action.meta.url);
      }
      return true;
    }
    return false;
  }
  async feedSubscribe(_action) {
    var _a;
    this.client.subscribeToFeed.emit({});
    if (!this.client.noSubscribe) {
      await ((_a = this.client.subscribeElement) === null || _a === void 0 ? void 0 : _a.attemptOpenDefaultApp());
    }
    return true;
  }
}

const pdClientCss = ".pd-button{background:var(--poductivity-accent-alt, #61dcd2);outline:none;font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";color:var(--pd-text-color, #ffffff);font-size:0.9em;padding:4px 8px;display:inline-block;border-radius:3px;text-align:center;text-decoration:none;vertical-align:middle;cursor:pointer;border:none}.pd-button.pd-button-small{font-size:0.8em;padding:2px 4px}.pd-button.pd-button-large{padding:8px 12px}.pd-button-link{background:none;padding:0;border:none;display:inline-block;width:auto;color:var(--poductivity-accent, #112bd1);cursor:pointer;font-size:inherit}:host{display:block;color:var(--pd-text-color, #ffffff);font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";height:100%;min-height:40px;position:relative;line-height:1.1}:host,:host *{box-sizing:border-box}:host .poductivity-wrapper{background:transparent;height:100%;overflow-x:auto;display:flex;align-items:center;justify-content:center;flex-direction:column;text-align:center;transition:background-color 0.2s linear}:host .poductivity-wrapper.active{background:rgba(17, 43, 209, 0.9)}:host poductivity-messages{flex:0 0 auto}:host .pd-option-group{border-bottom:3px solid var(--pd-text-color, #ffffff)}:host .pd-option-group:last-of-type{border-bottom:none}:host .pd-option-group .pd-option-group-text{font-size:0.9em;margin:8px 8px 16px}:host .pd-option-group .pd-option{margin:0 8px 12px;color:var(--poductivity-accent, #112bd1)}:host .pd-option-group.pd-option-group-multiple{flex:1 1 100%;width:100%;display:flex;flex-direction:column;align-items:stretch;justify-content:center}:host .pd-option-group.pd-option-group-multiple .pd-option-group-text{margin:8px}:host .pd-option-group.pd-option-group-multiple .pd-option-group-options{display:flex;flex-direction:column;overflow:hidden;flex:1}:host .pd-option-group.pd-option-group-multiple .pd-option{background:transparent;flex:1 1 100%;transition:all 0.4s linear;color:#ffffff;border-radius:0;margin:0}:host .pd-option-group.pd-option-group-multiple .pd-option:last-child{border-bottom:none}:host .pd-option-group.pd-option-group-multiple .pd-option:hover,:host .pd-option-group.pd-option-group-multiple .pd-option:focus,:host .pd-option-group.pd-option-group-multiple .pd-option:active{background:rgba(255, 255, 255, 0.4)}:host .pd-option-group-null{width:100%;height:100%}:host .pd-debug{position:absolute;top:0;right:0;font-size:10px;text-align:right;opacity:0.2;pointer-events:none}:host poductivity-opt-out-control{position:absolute;bottom:4px;right:4px;z-index:1}";

const PdClient = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.loadingEpisodeData = createEvent(this, "loadingEpisodeData", 7);
    this.loadedEpisodeData = createEvent(this, "loadedEpisodeData", 7);
    this.loadingEpisodeDataFailed = createEvent(this, "loadingEpisodeDataFailed", 7);
    this.performActionEvent = createEvent(this, "performAction", 7);
    this.audioControlEvent = createEvent(this, "audioControl", 7);
    this.openExternalUrl = createEvent(this, "openExternalUrl", 7);
    this.subscribeToFeed = createEvent(this, "subscribeToFeed", 7);
    var _a;
    /**
     * If not using the audioElement prop, pass the current playback rate here
     */
    this.playbackRate = 1;
    /**
     * The User Agent your app uses.
     * Only necessary when integrating into an app with a custom user agent.
     * Will use the current browser's user agent if available.
     */
    this.userAgent = ((_a = window === null || window === void 0 ? void 0 : window.navigator) === null || _a === void 0 ? void 0 : _a.userAgent) || 'Poductivity-Client';
    /**
     * Optionally supply a <poductivity-opt-out-control> element positioned
     * anywhere in the DOM. Will show inside this element otherwise
     */
    this.optOutControl = null;
    /**
     * Use to disable the standard Poductivity subscribe behaviour if you wish
     * to capture and handle the subscribeToFeed event yourself
     */
    this.noSubscribe = false;
    /**
     * Optionally supply a <poductivity-messages> element positioned
     * anywhere in the DOM. Will show in this element otherwise
     */
    this.messagesElement = null;
    /**
     * Enable debug mode. Shows a status panel in the client and adds
     * extra console logging. For internal use only
     */
    this.debug = false;
    /**
     * Pass in a data object to use instead of contacting the API server
     * For internal/debug use only
     */
    this.data = null;
    this.episode = null;
    this.slots = [];
    this.optionGroups = [];
    this.optOut = '';
    this.loading = false;
    this.loaded = false;
    this.timestampI = 0;
    this.isPlayingI = false;
    this.internalDeviceIdentifier = '';
    this.dynamicOptionGroup = null;
    this.lastClose = 0;
    this.contiguousPlaybackStart = 0;
    this.playbackStarted = false;
    this.enablePrivacyControl = false;
    //rollup-plugin-version-injector will replace this
    this.version = '0.11.0';
    this.clientService = new ClientService(this.userAgent);
    this.optOutService = OptOutService$1;
    this.logger = new Logger('PdClient', this.debug);
    this.actionHandler = new ActionHandler(this);
    this.lastTimestampUpdateAt = Date.now();
    this.audioElPlaying = this.audioElPlaying.bind(this);
    this.audioElPaused = this.audioElPaused.bind(this);
    this.audioElTimeUpdate = this.audioElTimeUpdate.bind(this);
    this.changeOptOut = this.changeOptOut.bind(this);
  }
  componentWillLoad() {
    this.updateAudioElementListeners(this.audioElement);
    this.updateExternalOptOutControlListeners(this.optOutControl);
    this.messagesElementChanged(this.messagesElement);
  }
  componentDidRender() {
    if (this.messagesElement) {
      this.messagesElement.optionGroups = this.currentOptionGroups();
    }
  }
  disconnectedCallback() {
    this.close();
  }
  /**
   * Set the current episode by feed URL and episode GUID.
   */
  async setEpisodeByFeed(feedUrl, episodeGuid) {
    return this.loadAll({
      feedUrl,
      episodeGuid,
    });
  }
  /**
   * Set the current episode by Poductivity Episode ID
   */
  async setEpisodeById(episodeId) {
    return this.loadAll({ id: episodeId });
  }
  /**
   * If integrating into an app, call this before destroying
   * the webview to allow the client to flush it's caches.
   * If integrating into the web, the client handles this itself
   * using the Browser Visibility API
   */
  async close() {
    const now = Date.now();
    const ignore = this.lastClose > now - 200;
    this.logger.log('Close method called', ignore);
    if (!ignore) {
      this.lastClose = now;
      await this.recordPlaybackPortion(this.contiguousPlaybackStart, this.timestampI);
      await this.clientService.flushResponseCache(this.internalDeviceIdentifier);
    }
  }
  /**
   * Is the client ready?
   */
  async isReady() {
    return this.hasLoadedData && !!this.internalDeviceIdentifier;
  }
  get hasLoadedData() {
    return !!this.data || this.loaded;
  }
  get subscribeData() {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    return {
      feedUrl: ((_b = (_a = this.episode) === null || _a === void 0 ? void 0 : _a.show) === null || _b === void 0 ? void 0 : _b.feedUrl) || '',
      feedProtocol: ((_d = (_c = this.episode) === null || _c === void 0 ? void 0 : _c.show) === null || _d === void 0 ? void 0 : _d.feedProtocol) || '',
      applePodcastsId: (_f = (_e = this.episode) === null || _e === void 0 ? void 0 : _e.show) === null || _f === void 0 ? void 0 : _f.applePodcastsId,
      spotifyId: (_h = (_g = this.episode) === null || _g === void 0 ? void 0 : _g.show) === null || _h === void 0 ? void 0 : _h.spotifyId,
    };
  }
  get playbackRateI() {
    var _a;
    return ((_a = this.audioElement) === null || _a === void 0 ? void 0 : _a.playbackRate) || this.playbackRate;
  }
  get wrapperClasses() {
    return classList({
      'poductivity-wrapper': true,
      active: this.currentActionableOptionGroups().length ||
        (!this.messagesElement && this.currentOptionGroups().length),
    });
  }
  async staticDataChanged(data) {
    await this.setDeviceIdentifier(this.deviceIdentifier);
    this.slots = data.slots;
    this.optionGroups = data.optionGroups;
    this.episode = data.episode;
  }
  async loadAll(params) {
    validateEpisodeParams(params);
    try {
      this.loading = true;
      await this.setDeviceIdentifier(this.deviceIdentifier);
      await this.loadData(params);
      if (this.episode && this.episode.id) {
        this.subscribeToOptOuts();
        this.loaded = true;
      }
    }
    catch (e) {
      this.logger.error('Unable to load episode', e);
      this.loadingEpisodeDataFailed.emit(e);
    }
    this.loading = false;
  }
  async loadData(params) {
    if (!this.data) {
      this.slots = [];
      this.optionGroups = [];
      this.dynamicOptionGroup = null;
      this.loadingEpisodeData.emit();
      const data = await this.clientService.getEpisodeData(params, this.internalDeviceIdentifier);
      this.slots = data.slots || [];
      this.optionGroups = data.optionGroups || [];
      this.episode = data.episode || null;
      this.loadedEpisodeData.emit(data);
    }
  }
  subscribeToOptOuts() {
    var _a;
    if (this.enablePrivacyControl) {
      (_a = this.privacySubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
      this.privacySubscription = this.optOutService
        .getOptOut(this.internalDeviceIdentifier, this.episode.showId, this.userAgent)
        .subscribe((optOut) => (this.optOut = optOut));
    }
  }
  optOutChanged(val) {
    if (this.optOutControl) {
      this.optOutControl.optOut = val;
    }
  }
  onPageHide() {
    this.close();
  }
  onVisibilityChange() {
    if (document.visibilityState === 'hidden') {
      this.close();
    }
  }
  async changeOptOut(event) {
    var _a;
    const { detail } = event;
    this.logger.log('Opt out changed', detail);
    await this.optOutService.saveOptOut(detail.optOut, (_a = this === null || this === void 0 ? void 0 : this.episode) === null || _a === void 0 ? void 0 : _a.showId, this.internalDeviceIdentifier, this.userAgent);
  }
  debugChanged(debug) {
    this.logger.setDebug(!!debug);
  }
  userAgentChanged(userAgent) {
    this.clientService.setUserAgent(userAgent);
  }
  async setDeviceIdentifier(suppliedDeviceIdentifier) {
    if (suppliedDeviceIdentifier) {
      this.internalDeviceIdentifier = suppliedDeviceIdentifier;
    }
    else {
      const { deviceIdentifier } = await DeviceService$1.getDeviceInfo();
      this.internalDeviceIdentifier = deviceIdentifier;
    }
  }
  updateAudioElementListeners(newEl, oldEl) {
    if (isHTMLAudioElement(oldEl)) {
      this.logger.log('Detaching audio element listeners', oldEl);
      oldEl.removeEventListener('timeupdate', this.audioElTimeUpdate);
      oldEl.removeEventListener('play', this.audioElPlaying);
      oldEl.removeEventListener('pause', this.audioElPaused);
    }
    if (isHTMLAudioElement(newEl)) {
      this.logger.log('Attaching audio element listeners', newEl);
      newEl.addEventListener('timeupdate', this.audioElTimeUpdate);
      newEl.addEventListener('play', this.audioElPlaying);
      newEl.addEventListener('pause', this.audioElPaused);
    }
  }
  updateExternalOptOutControlListeners(newEl, oldEl) {
    if (oldEl) {
      this.logger.log('Detaching opt out control listeners');
      oldEl.removeEventListener('optOutChanged', this.changeOptOut);
    }
    if (newEl) {
      this.logger.log('Attaching opt out control listeners');
      newEl.addEventListener('optOutChanged', this.changeOptOut);
      newEl.optOut = this.optOut;
    }
  }
  audioElPlaying() {
    this.isPlayingI = true;
  }
  audioElPaused() {
    this.isPlayingI = false;
  }
  isPlayingPropChanged() {
    if (typeof this.isPlaying !== 'boolean') {
      this.logger.warn('Poductivity: Invalid isPlaying prop');
    }
    else {
      this.isPlayingI = this.isPlaying;
    }
  }
  internalIsPlayingChanged(isPlaying) {
    this.logger.log('Playing state changed', isPlaying);
    if (!this.playbackStarted) {
      this.contiguousPlaybackStart = this.timestampI;
      this.playbackStarted = true;
    }
    if (!isPlaying) {
      this.recordPlaybackPortion(this.contiguousPlaybackStart, this.timestampI);
    }
  }
  audioElTimeUpdate(event) {
    this.timestampI = event.target.currentTime || 0;
  }
  timestampPropChanged() {
    if (typeof this.timestamp !== 'number') {
      this.logger.warn('Poductivity: Invalid timestamp prop');
    }
    else {
      this.timestampI = this.timestamp;
    }
  }
  internalTimestampChanged(newTimestamp, oldTimestamp) {
    const timeStampDiff = (newTimestamp - oldTimestamp) * 1000;
    const timeDiff = Date.now() - this.lastTimestampUpdateAt;
    //apply fuzz factor of 1% + 50ms
    const seeked = timeStampDiff < 0 ||
      timeStampDiff > timeDiff * this.playbackRateI * 1.01 + 50;
    if (seeked) {
      this.logger.log('Seek detected', oldTimestamp, newTimestamp, (timeStampDiff - timeDiff) / 1000);
      this.recordPlaybackPortion(this.contiguousPlaybackStart, oldTimestamp);
      this.contiguousPlaybackStart = newTimestamp;
    }
    this.lastTimestampUpdateAt = Date.now();
  }
  messagesElementChanged(messagesElement) {
    if (messagesElement) {
      this.logger.log('messages element changed, setting option groups', messagesElement);
      messagesElement.optionGroups = this.currentOptionGroups();
    }
  }
  currentSlots(timestamp) {
    const ts = typeof timestamp === 'number' ? timestamp : this.timestampI;
    return this.slots.filter((s) => s.start <= ts && s.end > ts);
  }
  currentOptionGroups(timestamp) {
    const ts = typeof timestamp === 'number' ? timestamp : this.timestampI;
    const slotIds = this.currentSlots(ts).map((s) => s.id);
    const ogs = this.optionGroups.filter((og) => slotIds.indexOf(og.slotId) !== -1);
    if (this.dynamicOptionGroup) {
      ogs.unshift(this.dynamicOptionGroup);
    }
    return ogs;
  }
  currentActionableOptionGroups(timestamp) {
    const ts = typeof timestamp === 'number' ? timestamp : this.timestampI;
    return this.currentOptionGroups(ts).filter((og) => !og.hide && !(og.oneTime && og.previouslyActed));
  }
  editOptionGroup(optionGroupId, changes) {
    const optionGroups = [...this.optionGroups];
    const index = optionGroups.findIndex((og) => og.id === optionGroupId);
    if (index === -1) {
      return null;
    }
    optionGroups[index] = Object.assign(Object.assign({}, optionGroups[index]), changes);
    this.optionGroups = optionGroups;
    return optionGroups[index];
  }
  editOption(optionGroupId, optionId, changes) {
    const optionGroups = [...this.optionGroups];
    const optionGroupIndex = optionGroups.findIndex((og) => og.id === optionGroupId);
    if (optionGroupIndex === -1) {
      return null;
    }
    const options = optionGroups[optionGroupIndex].options;
    const optionIndex = options.findIndex((o) => o.id === optionId);
    if (optionIndex === -1) {
      return null;
    }
    options[optionIndex] = Object.assign(Object.assign({}, options[optionIndex]), changes);
    optionGroups[optionGroupIndex] = Object.assign(Object.assign({}, optionGroups[optionGroupIndex]), { options: options });
    this.optionGroups = optionGroups;
    return options[optionIndex];
  }
  async recordPlaybackPortion(start, end) {
    // Ignore delta < 1 second
    if (end - start < 1) {
      return;
    }
    const coveredSlots = this.slots.filter((s) => s.start >= start - 0.5 && s.end <= end + 0.5);
    const plays = coveredSlots.map((s) => ({
      slotId: s.id,
      occurredAt: Math.round(Date.now() - (end - s.end) * 100),
      localTimezoneOffset: getTimezoneOffset(),
      playbackRate: this.playbackRateI,
    }));
    this.logger.log('Portion played', coveredSlots, start, end);
    if (plays.length) {
      await this.clientService.recordPlays(plays);
    }
  }
  async recordAction(option, actionIndex) {
    var _a, _b, _c;
    if (!this.episode) {
      throw 'Episode not loaded';
    }
    // actionIndex === -1 refers to the first action being started,
    // so all step counts have to be shifted by 1
    const response = {
      optionId: option === null || option === void 0 ? void 0 : option.id,
      actionId: (_a = option === null || option === void 0 ? void 0 : option.clientActions[actionIndex !== -1 ? actionIndex : 0]) === null || _a === void 0 ? void 0 : _a.actionId,
      timestamp: ((_b = this.audioElement) === null || _b === void 0 ? void 0 : _b.currentTime) || 0,
      step: actionIndex + 2,
      totalSteps: ((_c = option === null || option === void 0 ? void 0 : option.clientActions) === null || _c === void 0 ? void 0 : _c.length) + 1,
      occurredAt: new Date().getTime(),
      localTimezoneOffset: getTimezoneOffset(),
    };
    await this.clientService.recordResponse(response);
  }
  async perform(option, optionGroup) {
    this.logger.log('Option initiated', option);
    this.performActionEvent.emit({
      option,
      optionGroup,
    });
    try {
      await this.recordAction(option, -1);
      for (let index = 0; index < option.clientActions.length; index++) {
        const action = option.clientActions[index];
        if (!(action.type in this.actionHandler)) {
          throw 'Action type not supported by this client version';
        }
        this.logger.log('Starting action', action);
        await this.actionHandler[action.type](action, option, optionGroup);
        await this.recordAction(option, index);
        this.logger.log('Action completed', action);
      }
      this.editOptionGroup(optionGroup.id, {
        previouslyActed: true,
      });
      this.logger.log('All actions performed for option', option);
    }
    catch (e) {
      this.logger.log('Action not completed', e);
    }
    this.clientService.flushResponseCache(this.internalDeviceIdentifier);
  }
  controlAudio(action, val) {
    this.audioControlEvent.emit({
      action,
      val,
    });
    if (isHTMLAudioElement(this.audioElement)) {
      switch (action) {
        case 'seekAbsolute':
          this.audioElement.currentTime = val || 0;
          break;
        case 'seekRelative':
          this.audioElement.currentTime += val || 0;
          break;
        case 'pause':
          this.audioElement.pause();
          break;
        case 'resume':
          this.audioElement.play();
          break;
      }
    }
  }
  getDebugPanel() {
    var _a;
    return (this.debug && (h("pre", { class: "pd-debug" }, h("div", null, "SDK Version: ", this.version), h("div", null, "Device Identifier: ", this.internalDeviceIdentifier), h("div", null, "Client ready: ", this.isReady() ? 'true' : 'false'), h("div", null, "AudioElement ready:", ((_a = this.audioElement) === null || _a === void 0 ? void 0 : _a.isConnected) ? 'true' : 'false'), this.enablePrivacyControl && (h("div", null, "Opt out: ", this.optOut || 'false')), h("div", null, "API Server: ", API_SERVER), h("div", null, "Current slots: ", this.currentSlots().length), h("div", null, "Current option groups: ", this.currentOptionGroups().length), h("div", null, "Current time: ", this.timestampI))));
  }
  getCurrentOptionGroupsUI() {
    if (this.currentOptionGroups().length === 0) {
      return (h("div", { class: "pd-option-group-null", onClick: () => this.recordAction() }, "\u00A0"));
    }
    else {
      return this.currentOptionGroups().map((optionGroup) => {
        const classes = classList({
          'pd-option-group': true,
          'pd-option-group-multiple': optionGroup.options.length > 1,
        });
        return (h("div", { key: optionGroup.id, class: classes }, !this.messagesElement && (h("poductivity-messages", { optionGroups: optionGroup })), this.getOptionGroupButtons(optionGroup)));
      });
    }
  }
  getOptionGroupButtons(optionGroup) {
    return (!optionGroup.hide &&
      !(optionGroup.oneTime && optionGroup.previouslyActed) && (h("div", { class: "pd-option-group-options" }, (optionGroup.options || []).map((option) => {
      return (h("button", { key: option.id, class: "pd-button pd-button-large pd-option", onClick: () => this.perform(option, optionGroup) }, option.title));
    }))));
  }
  render() {
    var _a;
    return (h("div", { class: this.wrapperClasses }, this.getDebugPanel(), this.isReady() && this.getCurrentOptionGroupsUI(), h("poductivity-modal", { ref: (el) => (this.modalElement = el), debug: this.debug }), !this.noSubscribe && ((_a = this.episode) === null || _a === void 0 ? void 0 : _a.show) && (h("poductivity-subscribe", { ref: (el) => (this.subscribeElement = el), showData: this.subscribeData, userAgent: this.userAgent, debug: this.debug })), !this.optOutControl && this.enablePrivacyControl && (h("poductivity-opt-out-control", { optOut: this.optOut, up: true, right: true }))));
  }
  static get watchers() { return {
    "data": ["staticDataChanged"],
    "optOut": ["optOutChanged"],
    "debug": ["debugChanged"],
    "userAgent": ["userAgentChanged"],
    "deviceIdentifier": ["setDeviceIdentifier"],
    "audioElement": ["updateAudioElementListeners"],
    "optOutControl": ["updateExternalOptOutControlListeners"],
    "isPlaying": ["isPlayingPropChanged"],
    "isPlayingI": ["internalIsPlayingChanged"],
    "timestamp": ["timestampPropChanged"],
    "timestampI": ["internalTimestampChanged"],
    "messagesElement": ["messagesElementChanged"]
  }; }
};
PdClient.style = pdClientCss;

/**
 * A collection of shims that provide minimal functionality of the ES6 collections.
 *
 * These implementations are not meant to be used outside of the ResizeObserver
 * modules as they cover only a limited range of use cases.
 */
/* eslint-disable require-jsdoc, valid-jsdoc */
var MapShim = (function () {
    if (typeof Map !== 'undefined') {
        return Map;
    }
    /**
     * Returns index in provided array that matches the specified key.
     *
     * @param {Array<Array>} arr
     * @param {*} key
     * @returns {number}
     */
    function getIndex(arr, key) {
        var result = -1;
        arr.some(function (entry, index) {
            if (entry[0] === key) {
                result = index;
                return true;
            }
            return false;
        });
        return result;
    }
    return /** @class */ (function () {
        function class_1() {
            this.__entries__ = [];
        }
        Object.defineProperty(class_1.prototype, "size", {
            /**
             * @returns {boolean}
             */
            get: function () {
                return this.__entries__.length;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * @param {*} key
         * @returns {*}
         */
        class_1.prototype.get = function (key) {
            var index = getIndex(this.__entries__, key);
            var entry = this.__entries__[index];
            return entry && entry[1];
        };
        /**
         * @param {*} key
         * @param {*} value
         * @returns {void}
         */
        class_1.prototype.set = function (key, value) {
            var index = getIndex(this.__entries__, key);
            if (~index) {
                this.__entries__[index][1] = value;
            }
            else {
                this.__entries__.push([key, value]);
            }
        };
        /**
         * @param {*} key
         * @returns {void}
         */
        class_1.prototype.delete = function (key) {
            var entries = this.__entries__;
            var index = getIndex(entries, key);
            if (~index) {
                entries.splice(index, 1);
            }
        };
        /**
         * @param {*} key
         * @returns {void}
         */
        class_1.prototype.has = function (key) {
            return !!~getIndex(this.__entries__, key);
        };
        /**
         * @returns {void}
         */
        class_1.prototype.clear = function () {
            this.__entries__.splice(0);
        };
        /**
         * @param {Function} callback
         * @param {*} [ctx=null]
         * @returns {void}
         */
        class_1.prototype.forEach = function (callback, ctx) {
            if (ctx === void 0) { ctx = null; }
            for (var _i = 0, _a = this.__entries__; _i < _a.length; _i++) {
                var entry = _a[_i];
                callback.call(ctx, entry[1], entry[0]);
            }
        };
        return class_1;
    }());
})();

/**
 * Detects whether window and document objects are available in current environment.
 */
var isBrowser = typeof window !== 'undefined' && typeof document !== 'undefined' && window.document === document;

// Returns global object of a current environment.
var global$1 = (function () {
    if (typeof global !== 'undefined' && global.Math === Math) {
        return global;
    }
    if (typeof self !== 'undefined' && self.Math === Math) {
        return self;
    }
    if (typeof window !== 'undefined' && window.Math === Math) {
        return window;
    }
    // eslint-disable-next-line no-new-func
    return Function('return this')();
})();

/**
 * A shim for the requestAnimationFrame which falls back to the setTimeout if
 * first one is not supported.
 *
 * @returns {number} Requests' identifier.
 */
var requestAnimationFrame$1 = (function () {
    if (typeof requestAnimationFrame === 'function') {
        // It's required to use a bounded function because IE sometimes throws
        // an "Invalid calling object" error if rAF is invoked without the global
        // object on the left hand side.
        return requestAnimationFrame.bind(global$1);
    }
    return function (callback) { return setTimeout(function () { return callback(Date.now()); }, 1000 / 60); };
})();

// Defines minimum timeout before adding a trailing call.
var trailingTimeout = 2;
/**
 * Creates a wrapper function which ensures that provided callback will be
 * invoked only once during the specified delay period.
 *
 * @param {Function} callback - Function to be invoked after the delay period.
 * @param {number} delay - Delay after which to invoke callback.
 * @returns {Function}
 */
function throttle (callback, delay) {
    var leadingCall = false, trailingCall = false, lastCallTime = 0;
    /**
     * Invokes the original callback function and schedules new invocation if
     * the "proxy" was called during current request.
     *
     * @returns {void}
     */
    function resolvePending() {
        if (leadingCall) {
            leadingCall = false;
            callback();
        }
        if (trailingCall) {
            proxy();
        }
    }
    /**
     * Callback invoked after the specified delay. It will further postpone
     * invocation of the original function delegating it to the
     * requestAnimationFrame.
     *
     * @returns {void}
     */
    function timeoutCallback() {
        requestAnimationFrame$1(resolvePending);
    }
    /**
     * Schedules invocation of the original function.
     *
     * @returns {void}
     */
    function proxy() {
        var timeStamp = Date.now();
        if (leadingCall) {
            // Reject immediately following calls.
            if (timeStamp - lastCallTime < trailingTimeout) {
                return;
            }
            // Schedule new call to be in invoked when the pending one is resolved.
            // This is important for "transitions" which never actually start
            // immediately so there is a chance that we might miss one if change
            // happens amids the pending invocation.
            trailingCall = true;
        }
        else {
            leadingCall = true;
            trailingCall = false;
            setTimeout(timeoutCallback, delay);
        }
        lastCallTime = timeStamp;
    }
    return proxy;
}

// Minimum delay before invoking the update of observers.
var REFRESH_DELAY = 20;
// A list of substrings of CSS properties used to find transition events that
// might affect dimensions of observed elements.
var transitionKeys = ['top', 'right', 'bottom', 'left', 'width', 'height', 'size', 'weight'];
// Check if MutationObserver is available.
var mutationObserverSupported = typeof MutationObserver !== 'undefined';
/**
 * Singleton controller class which handles updates of ResizeObserver instances.
 */
var ResizeObserverController = /** @class */ (function () {
    /**
     * Creates a new instance of ResizeObserverController.
     *
     * @private
     */
    function ResizeObserverController() {
        /**
         * Indicates whether DOM listeners have been added.
         *
         * @private {boolean}
         */
        this.connected_ = false;
        /**
         * Tells that controller has subscribed for Mutation Events.
         *
         * @private {boolean}
         */
        this.mutationEventsAdded_ = false;
        /**
         * Keeps reference to the instance of MutationObserver.
         *
         * @private {MutationObserver}
         */
        this.mutationsObserver_ = null;
        /**
         * A list of connected observers.
         *
         * @private {Array<ResizeObserverSPI>}
         */
        this.observers_ = [];
        this.onTransitionEnd_ = this.onTransitionEnd_.bind(this);
        this.refresh = throttle(this.refresh.bind(this), REFRESH_DELAY);
    }
    /**
     * Adds observer to observers list.
     *
     * @param {ResizeObserverSPI} observer - Observer to be added.
     * @returns {void}
     */
    ResizeObserverController.prototype.addObserver = function (observer) {
        if (!~this.observers_.indexOf(observer)) {
            this.observers_.push(observer);
        }
        // Add listeners if they haven't been added yet.
        if (!this.connected_) {
            this.connect_();
        }
    };
    /**
     * Removes observer from observers list.
     *
     * @param {ResizeObserverSPI} observer - Observer to be removed.
     * @returns {void}
     */
    ResizeObserverController.prototype.removeObserver = function (observer) {
        var observers = this.observers_;
        var index = observers.indexOf(observer);
        // Remove observer if it's present in registry.
        if (~index) {
            observers.splice(index, 1);
        }
        // Remove listeners if controller has no connected observers.
        if (!observers.length && this.connected_) {
            this.disconnect_();
        }
    };
    /**
     * Invokes the update of observers. It will continue running updates insofar
     * it detects changes.
     *
     * @returns {void}
     */
    ResizeObserverController.prototype.refresh = function () {
        var changesDetected = this.updateObservers_();
        // Continue running updates if changes have been detected as there might
        // be future ones caused by CSS transitions.
        if (changesDetected) {
            this.refresh();
        }
    };
    /**
     * Updates every observer from observers list and notifies them of queued
     * entries.
     *
     * @private
     * @returns {boolean} Returns "true" if any observer has detected changes in
     *      dimensions of it's elements.
     */
    ResizeObserverController.prototype.updateObservers_ = function () {
        // Collect observers that have active observations.
        var activeObservers = this.observers_.filter(function (observer) {
            return observer.gatherActive(), observer.hasActive();
        });
        // Deliver notifications in a separate cycle in order to avoid any
        // collisions between observers, e.g. when multiple instances of
        // ResizeObserver are tracking the same element and the callback of one
        // of them changes content dimensions of the observed target. Sometimes
        // this may result in notifications being blocked for the rest of observers.
        activeObservers.forEach(function (observer) { return observer.broadcastActive(); });
        return activeObservers.length > 0;
    };
    /**
     * Initializes DOM listeners.
     *
     * @private
     * @returns {void}
     */
    ResizeObserverController.prototype.connect_ = function () {
        // Do nothing if running in a non-browser environment or if listeners
        // have been already added.
        if (!isBrowser || this.connected_) {
            return;
        }
        // Subscription to the "Transitionend" event is used as a workaround for
        // delayed transitions. This way it's possible to capture at least the
        // final state of an element.
        document.addEventListener('transitionend', this.onTransitionEnd_);
        window.addEventListener('resize', this.refresh);
        if (mutationObserverSupported) {
            this.mutationsObserver_ = new MutationObserver(this.refresh);
            this.mutationsObserver_.observe(document, {
                attributes: true,
                childList: true,
                characterData: true,
                subtree: true
            });
        }
        else {
            document.addEventListener('DOMSubtreeModified', this.refresh);
            this.mutationEventsAdded_ = true;
        }
        this.connected_ = true;
    };
    /**
     * Removes DOM listeners.
     *
     * @private
     * @returns {void}
     */
    ResizeObserverController.prototype.disconnect_ = function () {
        // Do nothing if running in a non-browser environment or if listeners
        // have been already removed.
        if (!isBrowser || !this.connected_) {
            return;
        }
        document.removeEventListener('transitionend', this.onTransitionEnd_);
        window.removeEventListener('resize', this.refresh);
        if (this.mutationsObserver_) {
            this.mutationsObserver_.disconnect();
        }
        if (this.mutationEventsAdded_) {
            document.removeEventListener('DOMSubtreeModified', this.refresh);
        }
        this.mutationsObserver_ = null;
        this.mutationEventsAdded_ = false;
        this.connected_ = false;
    };
    /**
     * "Transitionend" event handler.
     *
     * @private
     * @param {TransitionEvent} event
     * @returns {void}
     */
    ResizeObserverController.prototype.onTransitionEnd_ = function (_a) {
        var _b = _a.propertyName, propertyName = _b === void 0 ? '' : _b;
        // Detect whether transition may affect dimensions of an element.
        var isReflowProperty = transitionKeys.some(function (key) {
            return !!~propertyName.indexOf(key);
        });
        if (isReflowProperty) {
            this.refresh();
        }
    };
    /**
     * Returns instance of the ResizeObserverController.
     *
     * @returns {ResizeObserverController}
     */
    ResizeObserverController.getInstance = function () {
        if (!this.instance_) {
            this.instance_ = new ResizeObserverController();
        }
        return this.instance_;
    };
    /**
     * Holds reference to the controller's instance.
     *
     * @private {ResizeObserverController}
     */
    ResizeObserverController.instance_ = null;
    return ResizeObserverController;
}());

/**
 * Defines non-writable/enumerable properties of the provided target object.
 *
 * @param {Object} target - Object for which to define properties.
 * @param {Object} props - Properties to be defined.
 * @returns {Object} Target object.
 */
var defineConfigurable = (function (target, props) {
    for (var _i = 0, _a = Object.keys(props); _i < _a.length; _i++) {
        var key = _a[_i];
        Object.defineProperty(target, key, {
            value: props[key],
            enumerable: false,
            writable: false,
            configurable: true
        });
    }
    return target;
});

/**
 * Returns the global object associated with provided element.
 *
 * @param {Object} target
 * @returns {Object}
 */
var getWindowOf = (function (target) {
    // Assume that the element is an instance of Node, which means that it
    // has the "ownerDocument" property from which we can retrieve a
    // corresponding global object.
    var ownerGlobal = target && target.ownerDocument && target.ownerDocument.defaultView;
    // Return the local global object if it's not possible extract one from
    // provided element.
    return ownerGlobal || global$1;
});

// Placeholder of an empty content rectangle.
var emptyRect = createRectInit(0, 0, 0, 0);
/**
 * Converts provided string to a number.
 *
 * @param {number|string} value
 * @returns {number}
 */
function toFloat(value) {
    return parseFloat(value) || 0;
}
/**
 * Extracts borders size from provided styles.
 *
 * @param {CSSStyleDeclaration} styles
 * @param {...string} positions - Borders positions (top, right, ...)
 * @returns {number}
 */
function getBordersSize(styles) {
    var positions = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        positions[_i - 1] = arguments[_i];
    }
    return positions.reduce(function (size, position) {
        var value = styles['border-' + position + '-width'];
        return size + toFloat(value);
    }, 0);
}
/**
 * Extracts paddings sizes from provided styles.
 *
 * @param {CSSStyleDeclaration} styles
 * @returns {Object} Paddings box.
 */
function getPaddings(styles) {
    var positions = ['top', 'right', 'bottom', 'left'];
    var paddings = {};
    for (var _i = 0, positions_1 = positions; _i < positions_1.length; _i++) {
        var position = positions_1[_i];
        var value = styles['padding-' + position];
        paddings[position] = toFloat(value);
    }
    return paddings;
}
/**
 * Calculates content rectangle of provided SVG element.
 *
 * @param {SVGGraphicsElement} target - Element content rectangle of which needs
 *      to be calculated.
 * @returns {DOMRectInit}
 */
function getSVGContentRect(target) {
    var bbox = target.getBBox();
    return createRectInit(0, 0, bbox.width, bbox.height);
}
/**
 * Calculates content rectangle of provided HTMLElement.
 *
 * @param {HTMLElement} target - Element for which to calculate the content rectangle.
 * @returns {DOMRectInit}
 */
function getHTMLElementContentRect(target) {
    // Client width & height properties can't be
    // used exclusively as they provide rounded values.
    var clientWidth = target.clientWidth, clientHeight = target.clientHeight;
    // By this condition we can catch all non-replaced inline, hidden and
    // detached elements. Though elements with width & height properties less
    // than 0.5 will be discarded as well.
    //
    // Without it we would need to implement separate methods for each of
    // those cases and it's not possible to perform a precise and performance
    // effective test for hidden elements. E.g. even jQuery's ':visible' filter
    // gives wrong results for elements with width & height less than 0.5.
    if (!clientWidth && !clientHeight) {
        return emptyRect;
    }
    var styles = getWindowOf(target).getComputedStyle(target);
    var paddings = getPaddings(styles);
    var horizPad = paddings.left + paddings.right;
    var vertPad = paddings.top + paddings.bottom;
    // Computed styles of width & height are being used because they are the
    // only dimensions available to JS that contain non-rounded values. It could
    // be possible to utilize the getBoundingClientRect if only it's data wasn't
    // affected by CSS transformations let alone paddings, borders and scroll bars.
    var width = toFloat(styles.width), height = toFloat(styles.height);
    // Width & height include paddings and borders when the 'border-box' box
    // model is applied (except for IE).
    if (styles.boxSizing === 'border-box') {
        // Following conditions are required to handle Internet Explorer which
        // doesn't include paddings and borders to computed CSS dimensions.
        //
        // We can say that if CSS dimensions + paddings are equal to the "client"
        // properties then it's either IE, and thus we don't need to subtract
        // anything, or an element merely doesn't have paddings/borders styles.
        if (Math.round(width + horizPad) !== clientWidth) {
            width -= getBordersSize(styles, 'left', 'right') + horizPad;
        }
        if (Math.round(height + vertPad) !== clientHeight) {
            height -= getBordersSize(styles, 'top', 'bottom') + vertPad;
        }
    }
    // Following steps can't be applied to the document's root element as its
    // client[Width/Height] properties represent viewport area of the window.
    // Besides, it's as well not necessary as the <html> itself neither has
    // rendered scroll bars nor it can be clipped.
    if (!isDocumentElement(target)) {
        // In some browsers (only in Firefox, actually) CSS width & height
        // include scroll bars size which can be removed at this step as scroll
        // bars are the only difference between rounded dimensions + paddings
        // and "client" properties, though that is not always true in Chrome.
        var vertScrollbar = Math.round(width + horizPad) - clientWidth;
        var horizScrollbar = Math.round(height + vertPad) - clientHeight;
        // Chrome has a rather weird rounding of "client" properties.
        // E.g. for an element with content width of 314.2px it sometimes gives
        // the client width of 315px and for the width of 314.7px it may give
        // 314px. And it doesn't happen all the time. So just ignore this delta
        // as a non-relevant.
        if (Math.abs(vertScrollbar) !== 1) {
            width -= vertScrollbar;
        }
        if (Math.abs(horizScrollbar) !== 1) {
            height -= horizScrollbar;
        }
    }
    return createRectInit(paddings.left, paddings.top, width, height);
}
/**
 * Checks whether provided element is an instance of the SVGGraphicsElement.
 *
 * @param {Element} target - Element to be checked.
 * @returns {boolean}
 */
var isSVGGraphicsElement = (function () {
    // Some browsers, namely IE and Edge, don't have the SVGGraphicsElement
    // interface.
    if (typeof SVGGraphicsElement !== 'undefined') {
        return function (target) { return target instanceof getWindowOf(target).SVGGraphicsElement; };
    }
    // If it's so, then check that element is at least an instance of the
    // SVGElement and that it has the "getBBox" method.
    // eslint-disable-next-line no-extra-parens
    return function (target) { return (target instanceof getWindowOf(target).SVGElement &&
        typeof target.getBBox === 'function'); };
})();
/**
 * Checks whether provided element is a document element (<html>).
 *
 * @param {Element} target - Element to be checked.
 * @returns {boolean}
 */
function isDocumentElement(target) {
    return target === getWindowOf(target).document.documentElement;
}
/**
 * Calculates an appropriate content rectangle for provided html or svg element.
 *
 * @param {Element} target - Element content rectangle of which needs to be calculated.
 * @returns {DOMRectInit}
 */
function getContentRect(target) {
    if (!isBrowser) {
        return emptyRect;
    }
    if (isSVGGraphicsElement(target)) {
        return getSVGContentRect(target);
    }
    return getHTMLElementContentRect(target);
}
/**
 * Creates rectangle with an interface of the DOMRectReadOnly.
 * Spec: https://drafts.fxtf.org/geometry/#domrectreadonly
 *
 * @param {DOMRectInit} rectInit - Object with rectangle's x/y coordinates and dimensions.
 * @returns {DOMRectReadOnly}
 */
function createReadOnlyRect(_a) {
    var x = _a.x, y = _a.y, width = _a.width, height = _a.height;
    // If DOMRectReadOnly is available use it as a prototype for the rectangle.
    var Constr = typeof DOMRectReadOnly !== 'undefined' ? DOMRectReadOnly : Object;
    var rect = Object.create(Constr.prototype);
    // Rectangle's properties are not writable and non-enumerable.
    defineConfigurable(rect, {
        x: x, y: y, width: width, height: height,
        top: y,
        right: x + width,
        bottom: height + y,
        left: x
    });
    return rect;
}
/**
 * Creates DOMRectInit object based on the provided dimensions and the x/y coordinates.
 * Spec: https://drafts.fxtf.org/geometry/#dictdef-domrectinit
 *
 * @param {number} x - X coordinate.
 * @param {number} y - Y coordinate.
 * @param {number} width - Rectangle's width.
 * @param {number} height - Rectangle's height.
 * @returns {DOMRectInit}
 */
function createRectInit(x, y, width, height) {
    return { x: x, y: y, width: width, height: height };
}

/**
 * Class that is responsible for computations of the content rectangle of
 * provided DOM element and for keeping track of it's changes.
 */
var ResizeObservation = /** @class */ (function () {
    /**
     * Creates an instance of ResizeObservation.
     *
     * @param {Element} target - Element to be observed.
     */
    function ResizeObservation(target) {
        /**
         * Broadcasted width of content rectangle.
         *
         * @type {number}
         */
        this.broadcastWidth = 0;
        /**
         * Broadcasted height of content rectangle.
         *
         * @type {number}
         */
        this.broadcastHeight = 0;
        /**
         * Reference to the last observed content rectangle.
         *
         * @private {DOMRectInit}
         */
        this.contentRect_ = createRectInit(0, 0, 0, 0);
        this.target = target;
    }
    /**
     * Updates content rectangle and tells whether it's width or height properties
     * have changed since the last broadcast.
     *
     * @returns {boolean}
     */
    ResizeObservation.prototype.isActive = function () {
        var rect = getContentRect(this.target);
        this.contentRect_ = rect;
        return (rect.width !== this.broadcastWidth ||
            rect.height !== this.broadcastHeight);
    };
    /**
     * Updates 'broadcastWidth' and 'broadcastHeight' properties with a data
     * from the corresponding properties of the last observed content rectangle.
     *
     * @returns {DOMRectInit} Last observed content rectangle.
     */
    ResizeObservation.prototype.broadcastRect = function () {
        var rect = this.contentRect_;
        this.broadcastWidth = rect.width;
        this.broadcastHeight = rect.height;
        return rect;
    };
    return ResizeObservation;
}());

var ResizeObserverEntry = /** @class */ (function () {
    /**
     * Creates an instance of ResizeObserverEntry.
     *
     * @param {Element} target - Element that is being observed.
     * @param {DOMRectInit} rectInit - Data of the element's content rectangle.
     */
    function ResizeObserverEntry(target, rectInit) {
        var contentRect = createReadOnlyRect(rectInit);
        // According to the specification following properties are not writable
        // and are also not enumerable in the native implementation.
        //
        // Property accessors are not being used as they'd require to define a
        // private WeakMap storage which may cause memory leaks in browsers that
        // don't support this type of collections.
        defineConfigurable(this, { target: target, contentRect: contentRect });
    }
    return ResizeObserverEntry;
}());

var ResizeObserverSPI = /** @class */ (function () {
    /**
     * Creates a new instance of ResizeObserver.
     *
     * @param {ResizeObserverCallback} callback - Callback function that is invoked
     *      when one of the observed elements changes it's content dimensions.
     * @param {ResizeObserverController} controller - Controller instance which
     *      is responsible for the updates of observer.
     * @param {ResizeObserver} callbackCtx - Reference to the public
     *      ResizeObserver instance which will be passed to callback function.
     */
    function ResizeObserverSPI(callback, controller, callbackCtx) {
        /**
         * Collection of resize observations that have detected changes in dimensions
         * of elements.
         *
         * @private {Array<ResizeObservation>}
         */
        this.activeObservations_ = [];
        /**
         * Registry of the ResizeObservation instances.
         *
         * @private {Map<Element, ResizeObservation>}
         */
        this.observations_ = new MapShim();
        if (typeof callback !== 'function') {
            throw new TypeError('The callback provided as parameter 1 is not a function.');
        }
        this.callback_ = callback;
        this.controller_ = controller;
        this.callbackCtx_ = callbackCtx;
    }
    /**
     * Starts observing provided element.
     *
     * @param {Element} target - Element to be observed.
     * @returns {void}
     */
    ResizeObserverSPI.prototype.observe = function (target) {
        if (!arguments.length) {
            throw new TypeError('1 argument required, but only 0 present.');
        }
        // Do nothing if current environment doesn't have the Element interface.
        if (typeof Element === 'undefined' || !(Element instanceof Object)) {
            return;
        }
        if (!(target instanceof getWindowOf(target).Element)) {
            throw new TypeError('parameter 1 is not of type "Element".');
        }
        var observations = this.observations_;
        // Do nothing if element is already being observed.
        if (observations.has(target)) {
            return;
        }
        observations.set(target, new ResizeObservation(target));
        this.controller_.addObserver(this);
        // Force the update of observations.
        this.controller_.refresh();
    };
    /**
     * Stops observing provided element.
     *
     * @param {Element} target - Element to stop observing.
     * @returns {void}
     */
    ResizeObserverSPI.prototype.unobserve = function (target) {
        if (!arguments.length) {
            throw new TypeError('1 argument required, but only 0 present.');
        }
        // Do nothing if current environment doesn't have the Element interface.
        if (typeof Element === 'undefined' || !(Element instanceof Object)) {
            return;
        }
        if (!(target instanceof getWindowOf(target).Element)) {
            throw new TypeError('parameter 1 is not of type "Element".');
        }
        var observations = this.observations_;
        // Do nothing if element is not being observed.
        if (!observations.has(target)) {
            return;
        }
        observations.delete(target);
        if (!observations.size) {
            this.controller_.removeObserver(this);
        }
    };
    /**
     * Stops observing all elements.
     *
     * @returns {void}
     */
    ResizeObserverSPI.prototype.disconnect = function () {
        this.clearActive();
        this.observations_.clear();
        this.controller_.removeObserver(this);
    };
    /**
     * Collects observation instances the associated element of which has changed
     * it's content rectangle.
     *
     * @returns {void}
     */
    ResizeObserverSPI.prototype.gatherActive = function () {
        var _this = this;
        this.clearActive();
        this.observations_.forEach(function (observation) {
            if (observation.isActive()) {
                _this.activeObservations_.push(observation);
            }
        });
    };
    /**
     * Invokes initial callback function with a list of ResizeObserverEntry
     * instances collected from active resize observations.
     *
     * @returns {void}
     */
    ResizeObserverSPI.prototype.broadcastActive = function () {
        // Do nothing if observer doesn't have active observations.
        if (!this.hasActive()) {
            return;
        }
        var ctx = this.callbackCtx_;
        // Create ResizeObserverEntry instance for every active observation.
        var entries = this.activeObservations_.map(function (observation) {
            return new ResizeObserverEntry(observation.target, observation.broadcastRect());
        });
        this.callback_.call(ctx, entries, ctx);
        this.clearActive();
    };
    /**
     * Clears the collection of active observations.
     *
     * @returns {void}
     */
    ResizeObserverSPI.prototype.clearActive = function () {
        this.activeObservations_.splice(0);
    };
    /**
     * Tells whether observer has active observations.
     *
     * @returns {boolean}
     */
    ResizeObserverSPI.prototype.hasActive = function () {
        return this.activeObservations_.length > 0;
    };
    return ResizeObserverSPI;
}());

// Registry of internal observers. If WeakMap is not available use current shim
// for the Map collection as it has all required methods and because WeakMap
// can't be fully polyfilled anyway.
var observers = typeof WeakMap !== 'undefined' ? new WeakMap() : new MapShim();
/**
 * ResizeObserver API. Encapsulates the ResizeObserver SPI implementation
 * exposing only those methods and properties that are defined in the spec.
 */
var ResizeObserver = /** @class */ (function () {
    /**
     * Creates a new instance of ResizeObserver.
     *
     * @param {ResizeObserverCallback} callback - Callback that is invoked when
     *      dimensions of the observed elements change.
     */
    function ResizeObserver(callback) {
        if (!(this instanceof ResizeObserver)) {
            throw new TypeError('Cannot call a class as a function.');
        }
        if (!arguments.length) {
            throw new TypeError('1 argument required, but only 0 present.');
        }
        var controller = ResizeObserverController.getInstance();
        var observer = new ResizeObserverSPI(callback, controller, this);
        observers.set(this, observer);
    }
    return ResizeObserver;
}());
// Expose public methods of ResizeObserver.
[
    'observe',
    'unobserve',
    'disconnect'
].forEach(function (method) {
    ResizeObserver.prototype[method] = function () {
        var _a;
        return (_a = observers.get(this))[method].apply(_a, arguments);
    };
});

var index = (function () {
    // Export existing implementation if available.
    if (typeof global$1.ResizeObserver !== 'undefined') {
        return global$1.ResizeObserver;
    }
    return ResizeObserver;
})();

const pdMarqueeCss = "poductivity-marquee{overflow:hidden;display:block}.pd-marquee-content{white-space:nowrap}";

const PdMarquee = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * How many px should move per second?
     */
    this.speed = 40;
    /**
     * Enable the animation
     */
    this.active = false;
    /**
     * A unique ID for the component. Will be auto generated if not supplied
     */
    this.uid = '';
    this.overflow = 0;
    this.fallbackId = String(Math.floor(Math.random() * 1000000));
  }
  componentDidLoad() {
    this.resizeObserver = new index(() => {
      this.setOverflow();
    });
    this.resizeObserver.observe(this.contentEl);
  }
  disconnectedCallback() {
    this.resizeObserver.disconnect();
  }
  componentWillUpdate() {
    this.setOverflow();
  }
  /**
   * Force a recalculation of the overflow
   */
  async setOverflow() {
    readTask(() => {
      this.overflow =
        (this.contentEl.scrollWidth || 0) - (this.contentEl.offsetWidth || 0);
    });
  }
  get contentClasses() {
    return classList({
      'pd-marquee-content': true,
      active: this.active,
    });
  }
  get contentStyle() {
    return this.overflow
      ? {
        animationName: this.animationName,
        animationDelay: '500ms',
        animationDirection: 'alternate',
        animationDuration: `${this.duration}ms`,
        animationIterationCount: 'infinite',
        animationTimingFunction: 'ease-in-out',
        animationPlayState: this.active ? 'running' : 'paused',
      }
      : {};
  }
  get internalId() {
    return this.uid || this.fallbackId;
  }
  get animationName() {
    return `pd-marquee-${this.internalId}`;
  }
  get duration() {
    const dur = (this.overflow / this.speed) * 1000;
    return dur > 1000 ? dur : 1000;
  }
  render() {
    return (h(Host, null, h("style", null, this.overflow
      ? `
          @keyframes ${this.animationName} {
            from { transform: translateX(0) }
            to { transform: translateX(-${this.overflow}px) }
          }
        `
      : ''), h("div", { ref: (el) => (this.contentEl = el), class: this.contentClasses, style: this.contentStyle }, h("slot", null))));
  }
};
PdMarquee.style = pdMarqueeCss;

const pdMessagesCss = ".pd-option-group-text{margin:8px 0;min-height:1em;font-weight:700}";

const PdMessages = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * The current opt out status.
     * This is set automatically by the client - DO NOT USE
     */
    this.optionGroups = [];
  }
  get hasMultipleOptionGroups() {
    return Array.isArray(this.optionGroups);
  }
  get filteredOptionGroups() {
    return (this.hasMultipleOptionGroups
      ? this.optionGroups
      : [this.optionGroups]);
  }
  getOptionGroupTitle(optionGroup) {
    return (!optionGroup.hide &&
      optionGroup.title && (h("poductivity-marquee", { class: "pd-option-group-title pd-option-group-text", active: true }, optionGroup.title)));
  }
  getOptionGroupHideMessage(optionGroup) {
    return (optionGroup.hide &&
      optionGroup.hideMessage && (h("div", { class: "pd-option-group-text pd-option-group-hidden-message" }, optionGroup.hideMessage)));
  }
  getOptionGroupOneTimeActedMessage(optionGroup) {
    return (!(optionGroup.hide && optionGroup.hideMessage) &&
      optionGroup.oneTime &&
      optionGroup.previouslyActed &&
      optionGroup.oneTimeActedMessage && (h("div", { class: "pd-option-group-text pd-option-group-hidden-message" }, optionGroup.oneTimeActedMessage)));
  }
  render() {
    return (h(Host, null, this.filteredOptionGroups.map((optionGroup) => {
      return (h("div", { class: "pd-messages-group" }, this.getOptionGroupTitle(optionGroup), this.getOptionGroupHideMessage(optionGroup), this.getOptionGroupOneTimeActedMessage(optionGroup)));
    })));
  }
};
PdMessages.style = pdMessagesCss;

function createCommonjsModule(fn, basedir, module) {
	return module = {
		path: basedir,
		exports: {},
		require: function (path, base) {
			return commonjsRequire();
		}
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var iframeResizer = createCommonjsModule(function (module) {
(function (undefined$1) {
  if (typeof window === 'undefined') return // don't run for server side render

  var count = 0,
    logEnabled = false,
    hiddenCheckEnabled = false,
    msgHeader = 'message',
    msgHeaderLen = msgHeader.length,
    msgId = '[iFrameSizer]', // Must match iframe msg ID
    msgIdLen = msgId.length,
    pagePosition = null,
    requestAnimationFrame = window.requestAnimationFrame,
    resetRequiredMethods = {
      max: 1,
      scroll: 1,
      bodyScroll: 1,
      documentElementScroll: 1
    },
    settings = {},
    timer = null,
    defaults = {
      autoResize: true,
      bodyBackground: null,
      bodyMargin: null,
      bodyMarginV1: 8,
      bodyPadding: null,
      checkOrigin: true,
      inPageLinks: false,
      enablePublicMethods: true,
      heightCalculationMethod: 'bodyOffset',
      id: 'iFrameResizer',
      interval: 32,
      log: false,
      maxHeight: Infinity,
      maxWidth: Infinity,
      minHeight: 0,
      minWidth: 0,
      mouseEvents: true,
      resizeFrom: 'parent',
      scrolling: false,
      sizeHeight: true,
      sizeWidth: false,
      warningTimeout: 5000,
      tolerance: 0,
      widthCalculationMethod: 'scroll',
      onClose: function () {
        return true
      },
      onClosed: function () {},
      onInit: function () {},
      onMessage: function () {
        warn('onMessage function not defined');
      },
      onMouseEnter: function () {},
      onMouseLeave: function () {},
      onResized: function () {},
      onScroll: function () {
        return true
      }
    };

  function getMutationObserver() {
    return (
      window.MutationObserver ||
      window.WebKitMutationObserver ||
      window.MozMutationObserver
    )
  }

  function addEventListener(el, evt, func) {
    el.addEventListener(evt, func, false);
  }

  function removeEventListener(el, evt, func) {
    el.removeEventListener(evt, func, false);
  }

  function setupRequestAnimationFrame() {
    var vendors = ['moz', 'webkit', 'o', 'ms'];
    var x;

    // Remove vendor prefixing if prefixed and break early if not
    for (x = 0; x < vendors.length && !requestAnimationFrame; x += 1) {
      requestAnimationFrame = window[vendors[x] + 'RequestAnimationFrame'];
    }

    if (!requestAnimationFrame) {
      log('setup', 'RequestAnimationFrame not supported');
    } else {
      // Firefox extension content-scripts have a globalThis object that is not the same as window.
      // Binding `requestAnimationFrame` to window allows the function to work and prevents errors
      // being thrown when run in that context, and should be a no-op in every other context.
      requestAnimationFrame = requestAnimationFrame.bind(window);
    }
  }

  function getMyID(iframeId) {
    var retStr = 'Host page: ' + iframeId;

    if (window.top !== window.self) {
      // eslint-disable-next-line unicorn/prefer-ternary
      if (window.parentIFrame && window.parentIFrame.getId) {
        retStr = window.parentIFrame.getId() + ': ' + iframeId;
      } else {
        retStr = 'Nested host page: ' + iframeId;
      }
    }

    return retStr
  }

  function formatLogHeader(iframeId) {
    return msgId + '[' + getMyID(iframeId) + ']'
  }

  function isLogEnabled(iframeId) {
    return settings[iframeId] ? settings[iframeId].log : logEnabled
  }

  function log(iframeId, msg) {
    output('log', iframeId, msg, isLogEnabled(iframeId));
  }

  function info(iframeId, msg) {
    output('info', iframeId, msg, isLogEnabled(iframeId));
  }

  function warn(iframeId, msg) {
    output('warn', iframeId, msg, true);
  }

  function output(type, iframeId, msg, enabled) {
    if (true === enabled && 'object' === typeof window.console) {
      // eslint-disable-next-line no-console
      console[type](formatLogHeader(iframeId), msg);
    }
  }

  function iFrameListener(event) {
    function resizeIFrame() {
      function resize() {
        setSize(messageData);
        setPagePosition(iframeId);
        on('onResized', messageData);
      }

      ensureInRange('Height');
      ensureInRange('Width');

      syncResize(resize, messageData, 'init');
    }

    function processMsg() {
      var data = msg.substr(msgIdLen).split(':');
      var height = data[1] ? parseInt(data[1], 10) : 0;
      var iframe = settings[data[0]] && settings[data[0]].iframe;
      var compStyle = getComputedStyle(iframe);

      return {
        iframe: iframe,
        id: data[0],
        height: height + getPaddingEnds(compStyle) + getBorderEnds(compStyle),
        width: data[2],
        type: data[3]
      }
    }

    function getPaddingEnds(compStyle) {
      if (compStyle.boxSizing !== 'border-box') {
        return 0
      }
      var top = compStyle.paddingTop ? parseInt(compStyle.paddingTop, 10) : 0;
      var bot = compStyle.paddingBottom
        ? parseInt(compStyle.paddingBottom, 10)
        : 0;
      return top + bot
    }

    function getBorderEnds(compStyle) {
      if (compStyle.boxSizing !== 'border-box') {
        return 0
      }
      var top = compStyle.borderTopWidth
        ? parseInt(compStyle.borderTopWidth, 10)
        : 0;
      var bot = compStyle.borderBottomWidth
        ? parseInt(compStyle.borderBottomWidth, 10)
        : 0;
      return top + bot
    }

    function ensureInRange(Dimension) {
      var max = Number(settings[iframeId]['max' + Dimension]),
        min = Number(settings[iframeId]['min' + Dimension]),
        dimension = Dimension.toLowerCase(),
        size = Number(messageData[dimension]);

      log(iframeId, 'Checking ' + dimension + ' is in range ' + min + '-' + max);

      if (size < min) {
        size = min;
        log(iframeId, 'Set ' + dimension + ' to min value');
      }

      if (size > max) {
        size = max;
        log(iframeId, 'Set ' + dimension + ' to max value');
      }

      messageData[dimension] = '' + size;
    }

    function isMessageFromIFrame() {
      function checkAllowedOrigin() {
        function checkList() {
          var i = 0,
            retCode = false;

          log(
            iframeId,
            'Checking connection is from allowed list of origins: ' +
              checkOrigin
          );

          for (; i < checkOrigin.length; i++) {
            if (checkOrigin[i] === origin) {
              retCode = true;
              break
            }
          }
          return retCode
        }

        function checkSingle() {
          var remoteHost = settings[iframeId] && settings[iframeId].remoteHost;
          log(iframeId, 'Checking connection is from: ' + remoteHost);
          return origin === remoteHost
        }

        return checkOrigin.constructor === Array ? checkList() : checkSingle()
      }

      var origin = event.origin,
        checkOrigin = settings[iframeId] && settings[iframeId].checkOrigin;

      if (checkOrigin && '' + origin !== 'null' && !checkAllowedOrigin()) {
        throw new Error(
          'Unexpected message received from: ' +
            origin +
            ' for ' +
            messageData.iframe.id +
            '. Message was: ' +
            event.data +
            '. This error can be disabled by setting the checkOrigin: false option or by providing of array of trusted domains.'
        )
      }

      return true
    }

    function isMessageForUs() {
      return (
        msgId === ('' + msg).substr(0, msgIdLen) &&
        msg.substr(msgIdLen).split(':')[0] in settings
      ) // ''+Protects against non-string msg
    }

    function isMessageFromMetaParent() {
      // Test if this message is from a parent above us. This is an ugly test, however, updating
      // the message format would break backwards compatibity.
      var retCode = messageData.type in { true: 1, false: 1, undefined: 1 };

      if (retCode) {
        log(iframeId, 'Ignoring init message from meta parent page');
      }

      return retCode
    }

    function getMsgBody(offset) {
      return msg.substr(msg.indexOf(':') + msgHeaderLen + offset)
    }

    function forwardMsgFromIFrame(msgBody) {
      log(
        iframeId,
        'onMessage passed: {iframe: ' +
          messageData.iframe.id +
          ', message: ' +
          msgBody +
          '}'
      );

      on('onMessage', {
        iframe: messageData.iframe,
        message: JSON.parse(msgBody)
      });

      log(iframeId, '--');
    }

    function getPageInfo() {
      var bodyPosition = document.body.getBoundingClientRect(),
        iFramePosition = messageData.iframe.getBoundingClientRect();

      return JSON.stringify({
        iframeHeight: iFramePosition.height,
        iframeWidth: iFramePosition.width,
        clientHeight: Math.max(
          document.documentElement.clientHeight,
          window.innerHeight || 0
        ),
        clientWidth: Math.max(
          document.documentElement.clientWidth,
          window.innerWidth || 0
        ),
        offsetTop: parseInt(iFramePosition.top - bodyPosition.top, 10),
        offsetLeft: parseInt(iFramePosition.left - bodyPosition.left, 10),
        scrollTop: window.pageYOffset,
        scrollLeft: window.pageXOffset,
        documentHeight: document.documentElement.clientHeight,
        documentWidth: document.documentElement.clientWidth,
        windowHeight: window.innerHeight,
        windowWidth: window.innerWidth
      })
    }

    function sendPageInfoToIframe(iframe, iframeId) {
      function debouncedTrigger() {
        trigger('Send Page Info', 'pageInfo:' + getPageInfo(), iframe, iframeId);
      }
      debounceFrameEvents(debouncedTrigger, 32, iframeId);
    }

    function startPageInfoMonitor() {
      function setListener(type, func) {
        function sendPageInfo() {
          if (settings[id]) {
            sendPageInfoToIframe(settings[id].iframe, id);
          } else {
            stop();
          }
        }
['scroll', 'resize'].forEach(function (evt) {
          log(id, type + evt + ' listener for sendPageInfo');
          func(window, evt, sendPageInfo);
        });
      }

      function stop() {
        setListener('Remove ', removeEventListener);
      }

      function start() {
        setListener('Add ', addEventListener);
      }

      var id = iframeId; // Create locally scoped copy of iFrame ID

      start();

      if (settings[id]) {
        settings[id].stopPageInfo = stop;
      }
    }

    function stopPageInfoMonitor() {
      if (settings[iframeId] && settings[iframeId].stopPageInfo) {
        settings[iframeId].stopPageInfo();
        delete settings[iframeId].stopPageInfo;
      }
    }

    function checkIFrameExists() {
      var retBool = true;

      if (null === messageData.iframe) {
        warn(iframeId, 'IFrame (' + messageData.id + ') not found');
        retBool = false;
      }
      return retBool
    }

    function getElementPosition(target) {
      var iFramePosition = target.getBoundingClientRect();

      getPagePosition(iframeId);

      return {
        x: Math.floor(Number(iFramePosition.left) + Number(pagePosition.x)),
        y: Math.floor(Number(iFramePosition.top) + Number(pagePosition.y))
      }
    }

    function scrollRequestFromChild(addOffset) {
      /* istanbul ignore next */ // Not testable in Karma
      function reposition() {
        pagePosition = newPosition;
        scrollTo();
        log(iframeId, '--');
      }

      function calcOffset() {
        return {
          x: Number(messageData.width) + offset.x,
          y: Number(messageData.height) + offset.y
        }
      }

      function scrollParent() {
        if (window.parentIFrame) {
          window.parentIFrame['scrollTo' + (addOffset ? 'Offset' : '')](
            newPosition.x,
            newPosition.y
          );
        } else {
          warn(
            iframeId,
            'Unable to scroll to requested position, window.parentIFrame not found'
          );
        }
      }

      var offset = addOffset
          ? getElementPosition(messageData.iframe)
          : { x: 0, y: 0 },
        newPosition = calcOffset();

      log(
        iframeId,
        'Reposition requested from iFrame (offset x:' +
          offset.x +
          ' y:' +
          offset.y +
          ')'
      );

      if (window.top !== window.self) {
        scrollParent();
      } else {
        reposition();
      }
    }

    function scrollTo() {
      if (false !== on('onScroll', pagePosition)) {
        setPagePosition(iframeId);
      } else {
        unsetPagePosition();
      }
    }

    function findTarget(location) {
      function jumpToTarget() {
        var jumpPosition = getElementPosition(target);

        log(
          iframeId,
          'Moving to in page link (#' +
            hash +
            ') at x: ' +
            jumpPosition.x +
            ' y: ' +
            jumpPosition.y
        );
        pagePosition = {
          x: jumpPosition.x,
          y: jumpPosition.y
        };

        scrollTo();
        log(iframeId, '--');
      }

      function jumpToParent() {
        if (window.parentIFrame) {
          window.parentIFrame.moveToAnchor(hash);
        } else {
          log(
            iframeId,
            'In page link #' +
              hash +
              ' not found and window.parentIFrame not found'
          );
        }
      }

      var hash = location.split('#')[1] || '',
        hashData = decodeURIComponent(hash),
        target =
          document.getElementById(hashData) ||
          document.getElementsByName(hashData)[0];

      if (target) {
        jumpToTarget();
      } else if (window.top !== window.self) {
        jumpToParent();
      } else {
        log(iframeId, 'In page link #' + hash + ' not found');
      }
    }

    function onMouse(event) {
      var mousePos = {};

      if (Number(messageData.width) === 0 && Number(messageData.height) === 0) {
        var data = getMsgBody(9).split(':');
        mousePos = {
          x: data[1],
          y: data[0]
        };
      } else {
        mousePos = {
          x: messageData.width,
          y: messageData.height
        };
      }

      on(event, {
        iframe: messageData.iframe,
        screenX: Number(mousePos.x),
        screenY: Number(mousePos.y),
        type: messageData.type
      });
    }

    function on(funcName, val) {
      return chkEvent(iframeId, funcName, val)
    }

    function actionMsg() {
      if (settings[iframeId] && settings[iframeId].firstRun) firstRun();

      switch (messageData.type) {
        case 'close':
          closeIFrame(messageData.iframe);
          break

        case 'message':
          forwardMsgFromIFrame(getMsgBody(6));
          break

        case 'mouseenter':
          onMouse('onMouseEnter');
          break

        case 'mouseleave':
          onMouse('onMouseLeave');
          break

        case 'autoResize':
          settings[iframeId].autoResize = JSON.parse(getMsgBody(9));
          break

        case 'scrollTo':
          scrollRequestFromChild(false);
          break

        case 'scrollToOffset':
          scrollRequestFromChild(true);
          break

        case 'pageInfo':
          sendPageInfoToIframe(
            settings[iframeId] && settings[iframeId].iframe,
            iframeId
          );
          startPageInfoMonitor();
          break

        case 'pageInfoStop':
          stopPageInfoMonitor();
          break

        case 'inPageLink':
          findTarget(getMsgBody(9));
          break

        case 'reset':
          resetIFrame(messageData);
          break

        case 'init':
          resizeIFrame();
          on('onInit', messageData.iframe);
          break

        default:
          if (
            Number(messageData.width) === 0 &&
            Number(messageData.height) === 0
          ) {
            warn(
              'Unsupported message received (' +
                messageData.type +
                '), this is likely due to the iframe containing a later ' +
                'version of iframe-resizer than the parent page'
            );
          } else {
            resizeIFrame();
          }
      }
    }

    function hasSettings(iframeId) {
      var retBool = true;

      if (!settings[iframeId]) {
        retBool = false;
        warn(
          messageData.type +
            ' No settings for ' +
            iframeId +
            '. Message was: ' +
            msg
        );
      }

      return retBool
    }

    function iFrameReadyMsgReceived() {
      // eslint-disable-next-line no-restricted-syntax, guard-for-in
      for (var iframeId in settings) {
        trigger(
          'iFrame requested init',
          createOutgoingMsg(iframeId),
          settings[iframeId].iframe,
          iframeId
        );
      }
    }

    function firstRun() {
      if (settings[iframeId]) {
        settings[iframeId].firstRun = false;
      }
    }

    var msg = event.data,
      messageData = {},
      iframeId = null;

    if ('[iFrameResizerChild]Ready' === msg) {
      iFrameReadyMsgReceived();
    } else if (isMessageForUs()) {
      messageData = processMsg();
      iframeId = messageData.id;
      if (settings[iframeId]) {
        settings[iframeId].loaded = true;
      }

      if (!isMessageFromMetaParent() && hasSettings(iframeId)) {
        log(iframeId, 'Received: ' + msg);

        if (checkIFrameExists() && isMessageFromIFrame()) {
          actionMsg();
        }
      }
    } else {
      info(iframeId, 'Ignored: ' + msg);
    }
  }

  function chkEvent(iframeId, funcName, val) {
    var func = null,
      retVal = null;

    if (settings[iframeId]) {
      func = settings[iframeId][funcName];

      if ('function' === typeof func) {
        retVal = func(val);
      } else {
        throw new TypeError(
          funcName + ' on iFrame[' + iframeId + '] is not a function'
        )
      }
    }

    return retVal
  }

  function removeIframeListeners(iframe) {
    var iframeId = iframe.id;
    delete settings[iframeId];
  }

  function closeIFrame(iframe) {
    var iframeId = iframe.id;
    if (chkEvent(iframeId, 'onClose', iframeId) === false) {
      log(iframeId, 'Close iframe cancelled by onClose event');
      return
    }
    log(iframeId, 'Removing iFrame: ' + iframeId);

    try {
      // Catch race condition error with React
      if (iframe.parentNode) {
        iframe.parentNode.removeChild(iframe);
      }
    } catch (error) {
      warn(error);
    }

    chkEvent(iframeId, 'onClosed', iframeId);
    log(iframeId, '--');
    removeIframeListeners(iframe);
  }

  function getPagePosition(iframeId) {
    if (null === pagePosition) {
      pagePosition = {
        x:
          window.pageXOffset !== undefined$1
            ? window.pageXOffset
            : document.documentElement.scrollLeft,
        y:
          window.pageYOffset !== undefined$1
            ? window.pageYOffset
            : document.documentElement.scrollTop
      };
      log(
        iframeId,
        'Get page position: ' + pagePosition.x + ',' + pagePosition.y
      );
    }
  }

  function setPagePosition(iframeId) {
    if (null !== pagePosition) {
      window.scrollTo(pagePosition.x, pagePosition.y);
      log(
        iframeId,
        'Set page position: ' + pagePosition.x + ',' + pagePosition.y
      );
      unsetPagePosition();
    }
  }

  function unsetPagePosition() {
    pagePosition = null;
  }

  function resetIFrame(messageData) {
    function reset() {
      setSize(messageData);
      trigger('reset', 'reset', messageData.iframe, messageData.id);
    }

    log(
      messageData.id,
      'Size reset requested by ' +
        ('init' === messageData.type ? 'host page' : 'iFrame')
    );
    getPagePosition(messageData.id);
    syncResize(reset, messageData, 'reset');
  }

  function setSize(messageData) {
    function setDimension(dimension) {
      if (!messageData.id) {
        log('undefined', 'messageData id not set');
        return
      }
      messageData.iframe.style[dimension] = messageData[dimension] + 'px';
      log(
        messageData.id,
        'IFrame (' +
          iframeId +
          ') ' +
          dimension +
          ' set to ' +
          messageData[dimension] +
          'px'
      );
    }

    function chkZero(dimension) {
      // FireFox sets dimension of hidden iFrames to zero.
      // So if we detect that set up an event to check for
      // when iFrame becomes visible.

      /* istanbul ignore next */ // Not testable in PhantomJS
      if (!hiddenCheckEnabled && '0' === messageData[dimension]) {
        hiddenCheckEnabled = true;
        log(iframeId, 'Hidden iFrame detected, creating visibility listener');
        fixHiddenIFrames();
      }
    }

    function processDimension(dimension) {
      setDimension(dimension);
      chkZero(dimension);
    }

    var iframeId = messageData.iframe.id;

    if (settings[iframeId]) {
      if (settings[iframeId].sizeHeight) {
        processDimension('height');
      }
      if (settings[iframeId].sizeWidth) {
        processDimension('width');
      }
    }
  }

  function syncResize(func, messageData, doNotSync) {
    /* istanbul ignore if */ // Not testable in PhantomJS
    if (
      doNotSync !== messageData.type &&
      requestAnimationFrame &&
      // including check for jasmine because had trouble getting spy to work in unit test using requestAnimationFrame
      !window.jasmine
    ) {
      log(messageData.id, 'Requesting animation frame');
      requestAnimationFrame(func);
    } else {
      func();
    }
  }

  function trigger(calleeMsg, msg, iframe, id, noResponseWarning) {
    function postMessageToIFrame() {
      var target = settings[id] && settings[id].targetOrigin;
      log(
        id,
        '[' +
          calleeMsg +
          '] Sending msg to iframe[' +
          id +
          '] (' +
          msg +
          ') targetOrigin: ' +
          target
      );
      iframe.contentWindow.postMessage(msgId + msg, target);
    }

    function iFrameNotFound() {
      warn(id, '[' + calleeMsg + '] IFrame(' + id + ') not found');
    }

    function chkAndSend() {
      if (
        iframe &&
        'contentWindow' in iframe &&
        null !== iframe.contentWindow
      ) {
        // Null test for PhantomJS
        postMessageToIFrame();
      } else {
        iFrameNotFound();
      }
    }

    function warnOnNoResponse() {
      function warning() {
        if (settings[id] && !settings[id].loaded && !errorShown) {
          errorShown = true;
          warn(
            id,
            'IFrame has not responded within ' +
              settings[id].warningTimeout / 1000 +
              ' seconds. Check iFrameResizer.contentWindow.js has been loaded in iFrame. This message can be ignored if everything is working, or you can set the warningTimeout option to a higher value or zero to suppress this warning.'
          );
        }
      }

      if (
        !!noResponseWarning &&
        settings[id] &&
        !!settings[id].warningTimeout
      ) {
        settings[id].msgTimeout = setTimeout(
          warning,
          settings[id].warningTimeout
        );
      }
    }

    var errorShown = false;

    id = id || iframe.id;

    if (settings[id]) {
      chkAndSend();
      warnOnNoResponse();
    }
  }

  function createOutgoingMsg(iframeId) {
    return (
      iframeId +
      ':' +
      settings[iframeId].bodyMarginV1 +
      ':' +
      settings[iframeId].sizeWidth +
      ':' +
      settings[iframeId].log +
      ':' +
      settings[iframeId].interval +
      ':' +
      settings[iframeId].enablePublicMethods +
      ':' +
      settings[iframeId].autoResize +
      ':' +
      settings[iframeId].bodyMargin +
      ':' +
      settings[iframeId].heightCalculationMethod +
      ':' +
      settings[iframeId].bodyBackground +
      ':' +
      settings[iframeId].bodyPadding +
      ':' +
      settings[iframeId].tolerance +
      ':' +
      settings[iframeId].inPageLinks +
      ':' +
      settings[iframeId].resizeFrom +
      ':' +
      settings[iframeId].widthCalculationMethod +
      ':' +
      settings[iframeId].mouseEvents
    )
  }

  function setupIFrame(iframe, options) {
    function setLimits() {
      function addStyle(style) {
        if (
          Infinity !== settings[iframeId][style] &&
          0 !== settings[iframeId][style]
        ) {
          iframe.style[style] = settings[iframeId][style] + 'px';
          log(
            iframeId,
            'Set ' + style + ' = ' + settings[iframeId][style] + 'px'
          );
        }
      }

      function chkMinMax(dimension) {
        if (
          settings[iframeId]['min' + dimension] >
          settings[iframeId]['max' + dimension]
        ) {
          throw new Error(
            'Value for min' +
              dimension +
              ' can not be greater than max' +
              dimension
          )
        }
      }

      chkMinMax('Height');
      chkMinMax('Width');

      addStyle('maxHeight');
      addStyle('minHeight');
      addStyle('maxWidth');
      addStyle('minWidth');
    }

    function newId() {
      var id = (options && options.id) || defaults.id + count++;
      if (null !== document.getElementById(id)) {
        id += count++;
      }
      return id
    }

    function ensureHasId(iframeId) {
      if ('' === iframeId) {
        // eslint-disable-next-line no-multi-assign
        iframe.id = iframeId = newId();
        logEnabled = (options || {}).log;
        log(
          iframeId,
          'Added missing iframe ID: ' + iframeId + ' (' + iframe.src + ')'
        );
      }

      return iframeId
    }

    function setScrolling() {
      log(
        iframeId,
        'IFrame scrolling ' +
          (settings[iframeId] && settings[iframeId].scrolling
            ? 'enabled'
            : 'disabled') +
          ' for ' +
          iframeId
      );
      iframe.style.overflow =
        false === (settings[iframeId] && settings[iframeId].scrolling)
          ? 'hidden'
          : 'auto';
      switch (settings[iframeId] && settings[iframeId].scrolling) {
        case 'omit':
          break

        case true:
          iframe.scrolling = 'yes';
          break

        case false:
          iframe.scrolling = 'no';
          break

        default:
          iframe.scrolling = settings[iframeId]
            ? settings[iframeId].scrolling
            : 'no';
      }
    }

    // The V1 iFrame script expects an int, where as in V2 expects a CSS
    // string value such as '1px 3em', so if we have an int for V2, set V1=V2
    // and then convert V2 to a string PX value.
    function setupBodyMarginValues() {
      if (
        'number' ===
          typeof (settings[iframeId] && settings[iframeId].bodyMargin) ||
        '0' === (settings[iframeId] && settings[iframeId].bodyMargin)
      ) {
        settings[iframeId].bodyMarginV1 = settings[iframeId].bodyMargin;
        settings[iframeId].bodyMargin =
          '' + settings[iframeId].bodyMargin + 'px';
      }
    }

    function checkReset() {
      // Reduce scope of firstRun to function, because IE8's JS execution
      // context stack is borked and this value gets externally
      // changed midway through running this function!!!
      var firstRun = settings[iframeId] && settings[iframeId].firstRun,
        resetRequertMethod =
          settings[iframeId] &&
          settings[iframeId].heightCalculationMethod in resetRequiredMethods;

      if (!firstRun && resetRequertMethod) {
        resetIFrame({ iframe: iframe, height: 0, width: 0, type: 'init' });
      }
    }

    function setupIFrameObject() {
      if (settings[iframeId]) {
        settings[iframeId].iframe.iFrameResizer = {
          close: closeIFrame.bind(null, settings[iframeId].iframe),

          removeListeners: removeIframeListeners.bind(
            null,
            settings[iframeId].iframe
          ),

          resize: trigger.bind(
            null,
            'Window resize',
            'resize',
            settings[iframeId].iframe
          ),

          moveToAnchor: function (anchor) {
            trigger(
              'Move to anchor',
              'moveToAnchor:' + anchor,
              settings[iframeId].iframe,
              iframeId
            );
          },

          sendMessage: function (message) {
            message = JSON.stringify(message);
            trigger(
              'Send Message',
              'message:' + message,
              settings[iframeId].iframe,
              iframeId
            );
          }
        };
      }
    }

    // We have to call trigger twice, as we can not be sure if all
    // iframes have completed loading when this code runs. The
    // event listener also catches the page changing in the iFrame.
    function init(msg) {
      function iFrameLoaded() {
        trigger('iFrame.onload', msg, iframe, undefined$1, true);
        checkReset();
      }

      function createDestroyObserver(MutationObserver) {
        if (!iframe.parentNode) {
          return
        }

        var destroyObserver = new MutationObserver(function (mutations) {
          mutations.forEach(function (mutation) {
            var removedNodes = Array.prototype.slice.call(mutation.removedNodes); // Transform NodeList into an Array
            removedNodes.forEach(function (removedNode) {
              if (removedNode === iframe) {
                closeIFrame(iframe);
              }
            });
          });
        });
        destroyObserver.observe(iframe.parentNode, {
          childList: true
        });
      }

      var MutationObserver = getMutationObserver();
      if (MutationObserver) {
        createDestroyObserver(MutationObserver);
      }

      addEventListener(iframe, 'load', iFrameLoaded);
      trigger('init', msg, iframe, undefined$1, true);
    }

    function checkOptions(options) {
      if ('object' !== typeof options) {
        throw new TypeError('Options is not an object')
      }
    }

    function copyOptions(options) {
      // eslint-disable-next-line no-restricted-syntax
      for (var option in defaults) {
        if (Object.prototype.hasOwnProperty.call(defaults, option)) {
          settings[iframeId][option] = Object.prototype.hasOwnProperty.call(
            options,
            option
          )
            ? options[option]
            : defaults[option];
        }
      }
    }

    function getTargetOrigin(remoteHost) {
      return '' === remoteHost ||
        null !== remoteHost.match(/^(about:blank|javascript:|file:\/\/)/)
        ? '*'
        : remoteHost
    }

    function depricate(key) {
      var splitName = key.split('Callback');

      if (splitName.length === 2) {
        var name =
          'on' + splitName[0].charAt(0).toUpperCase() + splitName[0].slice(1);
        this[name] = this[key];
        delete this[key];
        warn(
          iframeId,
          "Deprecated: '" +
            key +
            "' has been renamed '" +
            name +
            "'. The old method will be removed in the next major version."
        );
      }
    }

    function processOptions(options) {
      options = options || {};
      settings[iframeId] = {
        firstRun: true,
        iframe: iframe,
        remoteHost: iframe.src && iframe.src.split('/').slice(0, 3).join('/')
      };

      checkOptions(options);
      Object.keys(options).forEach(depricate, options);
      copyOptions(options);

      if (settings[iframeId]) {
        settings[iframeId].targetOrigin =
          true === settings[iframeId].checkOrigin
            ? getTargetOrigin(settings[iframeId].remoteHost)
            : '*';
      }
    }

    function beenHere() {
      return iframeId in settings && 'iFrameResizer' in iframe
    }

    var iframeId = ensureHasId(iframe.id);

    if (!beenHere()) {
      processOptions(options);
      setScrolling();
      setLimits();
      setupBodyMarginValues();
      init(createOutgoingMsg(iframeId));
      setupIFrameObject();
    } else {
      warn(iframeId, 'Ignored iFrame, already setup.');
    }
  }

  function debouce(fn, time) {
    if (null === timer) {
      timer = setTimeout(function () {
        timer = null;
        fn();
      }, time);
    }
  }

  var frameTimer = {};
  function debounceFrameEvents(fn, time, frameId) {
    if (!frameTimer[frameId]) {
      frameTimer[frameId] = setTimeout(function () {
        frameTimer[frameId] = null;
        fn();
      }, time);
    }
  }

  // Not testable in PhantomJS
  /* istanbul ignore next */

  function fixHiddenIFrames() {
    function checkIFrames() {
      function checkIFrame(settingId) {
        function chkDimension(dimension) {
          return (
            '0px' ===
            (settings[settingId] && settings[settingId].iframe.style[dimension])
          )
        }

        function isVisible(el) {
          return null !== el.offsetParent
        }

        if (
          settings[settingId] &&
          isVisible(settings[settingId].iframe) &&
          (chkDimension('height') || chkDimension('width'))
        ) {
          trigger(
            'Visibility change',
            'resize',
            settings[settingId].iframe,
            settingId
          );
        }
      }

      Object.keys(settings).forEach(function (key) {
        checkIFrame(key);
      });
    }

    function mutationObserved(mutations) {
      log(
        'window',
        'Mutation observed: ' + mutations[0].target + ' ' + mutations[0].type
      );
      debouce(checkIFrames, 16);
    }

    function createMutationObserver() {
      var target = document.querySelector('body'),
        config = {
          attributes: true,
          attributeOldValue: false,
          characterData: true,
          characterDataOldValue: false,
          childList: true,
          subtree: true
        },
        observer = new MutationObserver(mutationObserved);

      observer.observe(target, config);
    }

    var MutationObserver = getMutationObserver();
    if (MutationObserver) {
      createMutationObserver();
    }
  }

  function resizeIFrames(event) {
    function resize() {
      sendTriggerMsg('Window ' + event, 'resize');
    }

    log('window', 'Trigger event: ' + event);
    debouce(resize, 16);
  }

  // Not testable in PhantomJS
  /* istanbul ignore next */
  function tabVisible() {
    function resize() {
      sendTriggerMsg('Tab Visable', 'resize');
    }

    if ('hidden' !== document.visibilityState) {
      log('document', 'Trigger event: Visiblity change');
      debouce(resize, 16);
    }
  }

  function sendTriggerMsg(eventName, event) {
    function isIFrameResizeEnabled(iframeId) {
      return (
        settings[iframeId] &&
        'parent' === settings[iframeId].resizeFrom &&
        settings[iframeId].autoResize &&
        !settings[iframeId].firstRun
      )
    }

    Object.keys(settings).forEach(function (iframeId) {
      if (isIFrameResizeEnabled(iframeId)) {
        trigger(eventName, event, settings[iframeId].iframe, iframeId);
      }
    });
  }

  function setupEventListeners() {
    addEventListener(window, 'message', iFrameListener);

    addEventListener(window, 'resize', function () {
      resizeIFrames('resize');
    });

    addEventListener(document, 'visibilitychange', tabVisible);

    addEventListener(document, '-webkit-visibilitychange', tabVisible);
  }

  function factory() {
    function init(options, element) {
      function chkType() {
        if (!element.tagName) {
          throw new TypeError('Object is not a valid DOM element')
        } else if ('IFRAME' !== element.tagName.toUpperCase()) {
          throw new TypeError(
            'Expected <IFRAME> tag, found <' + element.tagName + '>'
          )
        }
      }

      if (element) {
        chkType();
        setupIFrame(element, options);
        iFrames.push(element);
      }
    }

    function warnDeprecatedOptions(options) {
      if (options && options.enablePublicMethods) {
        warn(
          'enablePublicMethods option has been removed, public methods are now always available in the iFrame'
        );
      }
    }

    var iFrames;

    setupRequestAnimationFrame();
    setupEventListeners();

    return function iFrameResizeF(options, target) {
      iFrames = []; // Only return iFrames past in on this call

      warnDeprecatedOptions(options);

      switch (typeof target) {
        case 'undefined':
        case 'string':
          Array.prototype.forEach.call(
            document.querySelectorAll(target || 'iframe'),
            init.bind(undefined$1, options)
          );
          break

        case 'object':
          init(options, target);
          break

        default:
          throw new TypeError('Unexpected data type (' + typeof target + ')')
      }

      return iFrames
    }
  }

  function createJQueryPublicMethod($) {
    if (!$.fn) {
      info('', 'Unable to bind to jQuery, it is not fully loaded.');
    } else if (!$.fn.iFrameResize) {
      $.fn.iFrameResize = function $iFrameResizeF(options) {
        function init(index, element) {
          setupIFrame(element, options);
        }

        return this.filter('iframe').each(init).end()
      };
    }
  }

  if (window.jQuery) {
    createJQueryPublicMethod(window.jQuery);
  }

  if (typeof undefined$1 === 'function' && undefined$1.amd) {
    undefined$1([], factory);
  } else {
    // Node for browserfy
    module.exports = factory();
  }
  window.iFrameResize = window.iFrameResize || factory();
})();
});

const iconX = `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M16.192 6.344L11.949 10.586 7.707 6.344 6.293 7.758 10.535 12 6.293 16.242 7.707 17.656 11.949 13.414 16.192 17.656 17.606 16.242 13.364 12 17.606 7.758z"/></svg>`;

const iconLoading = `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M12,22c5.421,0,10-4.579,10-10h-2c0,4.337-3.663,8-8,8s-8-3.663-8-8c0-4.336,3.663-8,8-8V2C6.579,2,2,6.58,2,12 C2,17.421,6.579,22,12,22z"/></svg>`;

const pdModalCss = ".pd-button{background:var(--poductivity-accent-alt, #61dcd2);outline:none;font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";color:var(--pd-text-color, #ffffff);font-size:0.9em;padding:4px 8px;display:inline-block;border-radius:3px;text-align:center;text-decoration:none;vertical-align:middle;cursor:pointer;border:none}.pd-button.pd-button-small{font-size:0.8em;padding:2px 4px}.pd-button.pd-button-large{padding:8px 12px}.pd-button-link{background:none;padding:0;border:none;display:inline-block;width:auto;color:var(--poductivity-accent, #112bd1);cursor:pointer;font-size:inherit}@keyframes spin{from{transform:rotate(0deg)}to{transform:rotate(360deg)}}poductivity-modal{display:none;z-index:999999;background:rgba(0, 0, 0, 0.2);position:fixed;top:0;left:0;height:100%;width:100%;align-items:center;justify-content:center;flex-direction:column}poductivity-modal.pd-modal-absolute{position:absolute}poductivity-modal.pd-modal-show{display:flex}.pd-modal{position:absolute;width:95%;max-width:600px;max-height:min(600px, 85vh);background:#fff;font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";border-radius:10px;box-shadow:0 0 25px rgba(0, 0, 0, 0.5);overflow:hidden;z-index:9999999}.pd-modal-fill .pd-modal{width:100%;height:100%;max-width:100%;border:none;border-radius:0}.pd-modal .pd-modal-close-button{position:absolute;top:4px;right:4px;background:none;font-size:1.5em;z-index:10}.pd-modal .pd-modal-close-button:hover svg,.pd-modal .pd-modal-close-button:focus svg{fill:var(--poductivity-accent-alt, #61dcd2)}.pd-modal .pd-modal-close-button svg{fill:var(--poductivity-accent, #112bd1);height:1.8em;width:auto}.pd-modal .pd-modal-loading{position:absolute;top:16px;left:16px;z-index:99}.pd-modal .pd-modal-loading svg{animation:1000ms linear infinite spin;height:1.5em;width:1.5em}.pd-modal .pd-modal-loading svg path{fill:var(--poductivity-muted, #c0c3d9)}.pd-modal iframe.pd-modal-frame{display:block;border:none;background:transparent;width:100%;max-height:min(600px, 85vh);overflow:auto;-webkit-overflow-scrolling:touch}.pd-modal .pd-modal-slot{max-height:100%;padding:16px;overflow:auto;color:var(--pd-text-color, #292c3d);-webkit-overflow-scrolling:touch}.pd-modal .pd-modal-slot:empty{display:none}";

const PdModal = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * Position the modal absolute rather than fixed
     */
    this.absolute = false;
    /**
     * Fill all available space - no visible backdrop
     */
    this.fill = false;
    /**
     * Debug mode
     */
    this.debug = false;
    this.isShown = false;
    this.isLoadingFrame = false;
    this.frameSrc = null;
    this.resolveI = null;
    this.rejectI = null;
    this.iframeResizeInitiated = false;
    this.instanceId = uuid();
    this.logger = new Logger('PdModal', this.debug);
    this.backdropClicked = this.backdropClicked.bind(this);
    this.forceClose = this.forceClose.bind(this);
    this.frameLoaded = this.frameLoaded.bind(this);
  }
  componentDidRender() {
    if (this.frameElement && !this.iframeResizeInitiated) {
      iframeResizer({
        checkOrigin: [LISTENERS_SERVER],
        log: this.debug,
        scrolling: true,
        minHeight: 150,
      }, this.frameElement);
      this.iframeResizeInitiated = true;
    }
  }
  /**
   * Show the modal
   * @param frameSrc Optional source for the iframe
   */
  async show(frameSrc = null) {
    // Dismiss any currently active modal
    this.hide(false);
    this.isLoadingFrame = !!frameSrc;
    this.isShown = true;
    this.frameSrc = frameSrc;
    return new Promise((resolve, reject) => {
      this.resolveI = resolve;
      this.rejectI = reject;
    });
  }
  /**
   * Hide the modal. Specify whether the action was successful
   * @param message
   */
  async hide(message) {
    if (this.resolveI && this.rejectI) {
      message ? this.resolveI(message) : this.rejectI(message);
    }
    this.isShown = false;
    this.iframeResizeInitiated = false;
  }
  /**
   * Hide the modal with a failed action result
   */
  async forceClose() {
    this.hide(false);
  }
  receiveMessage(e) {
    var _a;
    if (checkMessageOrigin(e.origin) &&
      ((_a = e.data) === null || _a === void 0 ? void 0 : _a.instanceId) === this.instanceId) {
      this.logger.log('Message received from frame', e.data);
      this.hide(e.data.success || false);
    }
  }
  get frameAllowPolicy() {
    return `payment ${API_SERVER};`;
  }
  get fullFrameSrc() {
    return addQueryParam(this.frameSrc, 'instanceId', this.instanceId);
  }
  get wrapperClasses() {
    return classList({
      'pd-modal-show': this.isShown,
      'pd-modal-absolute': this.absolute,
      'pd-modal-fill': this.fill,
    });
  }
  backdropClicked(event) {
    if (event.target.tagName.toLowerCase() ===
      'poductivity-modal') {
      this.hide(false);
    }
  }
  frameLoaded() {
    this.isLoadingFrame = false;
  }
  render() {
    return (h(Host, { class: this.wrapperClasses, onClick: this.backdropClicked }, h("div", { class: "pd-modal" }, h("button", { title: "Close", class: "pd-button pd-modal-close-button", onClick: this.forceClose, innerHTML: iconX }), this.frameSrc && this.isLoadingFrame && (h("div", { class: "pd-modal-loading", innerHTML: iconLoading })), this.frameSrc && this.isShown && (h("iframe", { ref: (el) => (this.frameElement = el), src: this.fullFrameSrc, class: "pd-modal-frame", onLoad: this.frameLoaded, allow: this.frameAllowPolicy })), h("div", { class: "pd-modal-slot" }, h("slot", null)))));
  }
};
PdModal.style = pdModalCss;

const pdOptOutControlCss = ".pd-button{background:var(--poductivity-accent-alt, #61dcd2);outline:none;font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";color:var(--pd-text-color, #ffffff);font-size:0.9em;padding:4px 8px;display:inline-block;border-radius:3px;text-align:center;text-decoration:none;vertical-align:middle;cursor:pointer;border:none}.pd-button.pd-button-small{font-size:0.8em;padding:2px 4px}.pd-button.pd-button-large{padding:8px 12px}.pd-button-link{background:none;padding:0;border:none;display:inline-block;width:auto;color:var(--poductivity-accent, #112bd1);cursor:pointer;font-size:inherit}.pd-dropdown-container{position:relative;display:inline-block}.pd-dropdown-container:focus-within .pd-dropdown-menu{display:block}.pd-dropdown-menu{display:none;position:absolute;top:100%;left:0;border-radius:3px;border:1px solid #4d4d4d;z-index:100;min-width:140px;background:#fff}.pd-dropdown-menu.up{top:initial;bottom:100%}.pd-dropdown-menu.right{left:initial;right:0}.pd-dropdown-menu.right .pd-dropdown-item{text-align:right}.pd-dropdown-item{padding:2px 4px;display:block;border:none;border-bottom:1px solid #4d4d4d;width:100%;transition:background-color 0.15s linear;text-align:left;border-radius:0}.pd-dropdown-item:hover{background:#eee}.pd-dropdown-item:last-child{border-bottom:none}poductivity-opt-out-control .pd-opt-out-reveal{color:#999999}poductivity-opt-out-control>.pd-dropdown-container[aria-expanded=true] .pd-opt-out-reveal{color:var(--poductivity-accent, #112bd1)}";

const PdOptOutControl = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.optOutChanged = createEvent(this, "optOutChanged", 7);
    /**
     * Make the dropdown appear above the trigger
     */
    this.up = false;
    /**
     * Make the dropdown's right edge align to the trigger
     */
    this.right = false;
    this.isOpen = false;
  }
  get dropdownClasses() {
    return classList({
      'pd-dropdown-menu': true,
      up: this.up,
      right: this.right,
    });
  }
  changed(optOut) {
    this.optOutChanged.emit({ optOut });
    this.isOpen = false;
  }
  open() {
    this.isOpen = true;
  }
  close() {
    this.isOpen = false;
  }
  getOptions() {
    const options = [];
    if (this.optOut === '') {
      options.push(h("button", { class: "pd-dropdown-item", onClick: this.changed.bind(this, 'show') }, "Opt out of analytics for show"), h("button", { class: "pd-dropdown-item", onClick: this.changed.bind(this, 'listener') }, "Opt out of analytics globally"));
    }
    else {
      options.push(h("button", { class: "pd-dropdown-item", onClick: this.changed.bind(this, '') }, "Disable ", this.optOut === 'show' ? 'for show' : ''));
    }
    return options;
  }
  render() {
    return (h("div", { class: "pd-dropdown-container", "aria-expanded": this.isOpen ? 'true' : 'false', "aria-haspopup": "true" }, h("button", { class: "pd-button-link pd-opt-out-reveal pd-dropdown-toggle", title: "Privacy", onFocus: this.open, onBlur: this.close }, "Privacy"), h("div", { class: this.dropdownClasses }, this.getOptions())));
  }
};
PdOptOutControl.style = pdOptOutControlCss;

const pdSubscribeCss = "poductivity-subscribe .pd-subscribe-waiting{font-size:1em;margin-bottom:-1em}poductivity-subscribe .pd-modal .pd-modal-slot{display:flex;padding:48px;flex-direction:column;justify-content:center;height:100%}poductivity-subscribe .pd-subscribe-button{display:block;width:100%;margin-bottom:16px}poductivity-subscribe .pd-subscribe-button:last-child{margin-bottom:0}";

const PdSubscribe = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * Required. Data for the show to subscribe to
     */
    this.showData = null;
    /**
     * Required. The browser/app's user agent. Used for feature detection
     */
    this.userAgent = '';
    /**
     * Position the modal absolute rather than fixed
     */
    this.absolute = false;
    /**
     * Fill all available space - no visible backdrop
     */
    this.fill = false;
    /**
     * Debug mode
     */
    this.debug = false;
    this.waiting = false;
    this.logger = new Logger('poductivity-subscribe', this.debug);
    this.linkClicked = this.linkClicked.bind(this);
  }
  /**
   * Attempt to open the default app
   */
  async attemptOpenDefaultApp() {
    try {
      this.waiting = true;
      if (!this.applePodcastsUrl) {
        throw 'Apple Podcasts ID not set';
      }
      if (this.isAndroid) {
        throw 'Android device';
      }
      await tryScheme(this.applePodcastsUrl);
      this.show();
      this.logger.log('Default app opened');
      this.waiting = false;
      return true;
    }
    catch (e) {
      this.waiting = false;
      this.logger.log('Failed to open default app', e);
      return this.show();
    }
  }
  /**
   * Show the modal
   */
  async show() {
    return this.modal.show();
  }
  linkClicked() {
    this.modal.hide(true);
  }
  get fullFeedUrl() {
    return `${this.showData.feedProtocol}://${this.showData.feedUrl}`;
  }
  get applePodcastsUrl() {
    const protocol = this.isAppleHandheld || this.isMac ? 'podcasts' : 'https';
    return this.showData.applePodcastsId
      ? `${protocol}://podcasts.apple.com/us/podcast/id${this.showData.applePodcastsId}?app=podcast`
      : '';
  }
  get spotifyUrl() {
    return this.showData.spotifyId
      ? `https://open.spotify.com/show/${this.showData.spotifyId}`
      : '';
  }
  get spotifyUri() {
    return this.showData.spotifyId
      ? `spotify://show:${this.showData.spotifyId}`
      : '';
  }
  get googlePodcastsUrl() {
    return `https://podcasts.google.com/feed/${btoa(this.fullFeedUrl)}`;
  }
  get pocketCastsUrl() {
    return this.showData.applePodcastsId
      ? `https://pca.st/itunes/${this.showData.applePodcastsId}`
      : '';
  }
  get overcastUrl() {
    return this.showData.applePodcastsId
      ? `https://overcast.fm/itunes${this.showData.applePodcastsId}`
      : '';
  }
  get isAppleHandheld() {
    return /(iPhone|iPad|iPod)/.test(this.userAgent);
  }
  get isMac() {
    return /Macintosh/i.test(this.userAgent);
  }
  get isAndroid() {
    return /android/i.test(this.userAgent);
  }
  getButton(url, content) {
    return (h("a", { href: url, target: "_blank", class: "pd-button pd-button-large pd-subscribe-button", onClick: this.linkClicked }, content));
  }
  render() {
    return (h(Host, null, this.waiting && (h("div", { class: "pd-subscribe-waiting" }, "Loading\u2026")), h("poductivity-modal", { ref: (el) => (this.modal = el), absolute: this.absolute, fill: this.fill, debug: this.debug }, !this.isAndroid &&
      this.applePodcastsUrl &&
      this.getButton(this.applePodcastsUrl, 'Apple Podcasts'), this.getButton(this.fullFeedUrl, 'Feed URL'), this.getButton(this.googlePodcastsUrl, 'Google Podcasts'), this.isAppleHandheld &&
      this.overcastUrl &&
      this.getButton(this.overcastUrl, 'Overcast'), this.pocketCastsUrl &&
      this.getButton(this.pocketCastsUrl, 'Pocket Casts'), this.spotifyUrl && this.getButton(this.spotifyUrl, 'Spotify'))));
  }
};
PdSubscribe.style = pdSubscribeCss;

export { PdClient as poductivity_client, PdMarquee as poductivity_marquee, PdMessages as poductivity_messages, PdModal as poductivity_modal, PdOptOutControl as poductivity_opt_out_control, PdSubscribe as poductivity_subscribe };
