// Poductivity SDK version: 0.11.0 - Built March 11, 2021 13:05:46
import { r as registerInstance, c as createEvent, h, g as getAssetPath } from './index-d5e03b38.js';
import { L as Logger, s as secsToHr, c as classList, g as guessImageMimeFromPath } from './Logger-d43ff94a.js';

const pdInputRangeCss = "poductivity-input-range{display:block;touch-action:none;overscroll-behavior-x:none}input.pd-input-range-control{--range:calc(var(--max) - var(--min));--ratio:calc((var(--val) - var(--min)) / var(--range));--sx:calc(.5 * 18px + var(--ratio) * (100% - 18px));--sx-rtl:calc(.5 * 18px + calc(1 - var(--ratio)) * (100% - 18px));-webkit-appearance:none;background:transparent;margin:9px 0;width:100%}input.pd-input-range-control::-moz-focus-outer{border:0}input.pd-input-range-control:focus{outline:0}input.pd-input-range-control::-webkit-slider-runnable-track{cursor:default;height:5px;transition:all 0.2s ease;width:100%;transition:none;background:linear-gradient(var(--poductivity-accent, #112bd1), var(--poductivity-accent, #112bd1)) 0/var(--sx) 100% no-repeat var(--poductivity-muted, #c0c3d9);border:0 solid #cfd8dc;border-radius:2.5px}input.pd-input-range-control::-moz-range-progress{height:5px;background:var(--poductivity-accent, #112bd1);border-radius:2.5px}input.pd-input-range-control::-ms-fill-lower{height:5px;background:var(--poductivity-accent, #112bd1)}input.pd-input-range-control::-webkit-slider-thumb{box-shadow:0 4px 7px rgba(0, 0, 0, 0.1);background:var(--poductivity-accent, #112bd1);border:2px solid #fff;border-radius:9px;box-sizing:border-box;cursor:default;height:18px;width:18px;-webkit-appearance:none;margin-top:-6.5px}input.pd-input-range-control::-moz-range-track{cursor:default;height:5px;transition:all 0.2s ease;width:100%;background:var(--poductivity-muted, #c0c3d9);border:0 solid #cfd8dc;border-radius:2.5px}input.pd-input-range-control::-moz-range-thumb{box-shadow:0 4px 7px rgba(0, 0, 0, 0.1);background:var(--poductivity-accent, #112bd1);border:2px solid #fff;border-radius:9px;box-sizing:border-box;cursor:default;height:18px;width:18px}input.pd-input-range-control::-ms-track{cursor:default;height:5px;transition:all 0.2s ease;width:100%;background:transparent;border-color:transparent;border-width:9px 0;color:transparent}input.pd-input-range-control::-ms-fill-lower{background:var(--poductivity-accent, #112bd1);border:0 solid #cfd8dc;border-radius:5px}input.pd-input-range-control::-ms-fill-upper{background:var(--poductivity-muted, #c0c3d9);border:0 solid #cfd8dc;border-radius:5px}input.pd-input-range-control::-ms-thumb{box-shadow:0 4px 7px rgba(0, 0, 0, 0.1);background:var(--poductivity-accent, #112bd1);border:2px solid #fff;border-radius:9px;box-sizing:border-box;cursor:default;height:18px;width:18px;margin-top:1.25px}input.pd-input-range-control:disabled::-webkit-slider-thumb,input.pd-input-range-control:disabled::-moz-range-thumb,input.pd-input-range-control:disabled::-ms-thumb,input.pd-input-range-control:disabled::-webkit-slider-runnable-track,input.pd-input-range-control:disabled::-ms-fill-lower,input.pd-input-range-control:disabled::-ms-fill-upper{cursor:not-allowed}";

const PdInputRange = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.inputRangeChanged = createEvent(this, "inputRangeChanged", 7);
    this.inputRangeInteracting = createEvent(this, "inputRangeInteracting", 7);
    /**
     * The minimum value
     */
    this.min = 0;
    /**
     * The maximum value
     */
    this.max = 100;
    /**
     * The step between values
     */
    this.step = 1;
    /**
     * Disable the input
     */
    this.disabled = false;
  }
  get style() {
    return {
      '--min': String(this.min),
      // Never allow the max to be 0 so there's always some bar rendered
      '--max': String(this.max || 1),
      '--val': String(this.value),
    };
  }
  changed(e) {
    const value = parseFloat(e.target.value) || 0;
    this.inputRangeChanged.emit({ value });
    this.inputRangeInteracting.emit({ isSeeking: false });
  }
  startedInteracting() {
    this.inputRangeInteracting.emit({ isSeeking: true });
  }
  render() {
    return (h("input", { type: "range", class: "pd-input-range-control", min: this.min, max: this.max, step: 0.01, value: this.value, style: this.style, disabled: this.disabled }));
  }
};
PdInputRange.style = pdInputRangeCss;

const icon0_5 = `<svg width="21" height="21" fill="none" xmlns="http://www.w3.org/2000/svg"><circle cx="10.5" cy="10.5" r="9.5" stroke="#112BD1" stroke-width="2"/><path d="M4.238 14.099a.9.9 0 01-.65-.242.815.815 0 01-.252-.616c0-.25.084-.458.253-.627a.879.879 0 01.649-.253c.264 0 .484.084.66.253a.834.834 0 01.264.627c0 .25-.088.455-.264.616a.94.94 0 01-.66.242zm3.606-4.444a6.899 6.899 0 011.133-.099c.74 0 1.331.18 1.771.539.44.352.66.862.66 1.529 0 .726-.246 1.324-.737 1.793-.52.491-1.177.737-1.969.737-.785 0-1.412-.198-1.88-.594-.492-.41-.745-.975-.76-1.694h1.331c0 .374.125.649.374.825.25.176.568.264.957.264s.708-.125.957-.374a1.3 1.3 0 00.385-.957c0-.726-.491-1.089-1.474-1.089H6.414l.33-3.674h4.28v1.188H7.997l-.154 1.606zm9.9 4.345H16.16l-1.32-1.716L13.608 14h-1.595l2.08-2.662-2.08-2.695h1.595l1.31 1.716 1.242-1.716h1.584l-2.068 2.695L17.744 14z" fill="#112BD1"/></svg>`;

const icon1_0 = `<svg width="21" height="21" fill="none" xmlns="http://www.w3.org/2000/svg"><circle cx="10.5" cy="10.5" r="9.5" stroke="#112BD1" stroke-width="2"/><path d="M5.95 7.4a3.185 3.185 0 001.453-.583h.979V14H7.084V8.368a2.551 2.551 0 01-1.133.275V7.4zm9.377 6.6h-1.584l-1.32-1.716L11.191 14H9.596l2.08-2.662-2.08-2.695h1.595l1.31 1.716 1.242-1.716h1.584l-2.068 2.695L15.327 14z" fill="#112BD1"/></svg>`;

const icon1_5 = `<svg width="21" height="21" fill="none" xmlns="http://www.w3.org/2000/svg"><circle cx="10.5" cy="10.5" r="9.5" stroke="#112BD1" stroke-width="2"/><path d="M4.613 7.4a3.185 3.185 0 001.452-.583h.98V14H5.745V8.368a2.551 2.551 0 01-1.133.275V7.4zm4.636 6.699a.9.9 0 01-.649-.242.815.815 0 01-.253-.616c0-.25.084-.458.253-.627a.879.879 0 01.649-.253c.264 0 .484.084.66.253a.834.834 0 01.264.627c0 .25-.088.455-.264.616a.94.94 0 01-.66.242zm3.606-4.444a6.899 6.899 0 011.133-.099c.741 0 1.331.18 1.771.539.44.352.66.862.66 1.529 0 .726-.245 1.324-.737 1.793-.52.491-1.177.737-1.969.737-.784 0-1.411-.198-1.88-.594-.492-.41-.745-.975-.76-1.694h1.331c0 .374.125.649.374.825.25.176.569.264.957.264.389 0 .708-.125.957-.374a1.3 1.3 0 00.385-.957c0-.726-.491-1.089-1.474-1.089h-2.178l.33-3.674h4.28v1.188h-3.026l-.154 1.606z" fill="#112BD1"/></svg>`;

const icon2_0 = `<svg width="21" height="21" fill="none" xmlns="http://www.w3.org/2000/svg"><circle cx="10.5" cy="10.5" r="9.5" stroke="#112BD1" stroke-width="2"/><path d="M6.083 12.812h4.08V14H4.786v-1.221c0-.858.304-1.481.913-1.87.205-.132.443-.25.715-.352l1.463-.561c.322-.125.557-.279.704-.462.154-.19.23-.425.23-.704a.904.904 0 00-.296-.693c-.198-.19-.48-.286-.847-.286s-.671.088-.913.264c-.235.176-.37.436-.407.781H4.906c.014-.697.282-1.243.803-1.639.476-.36 1.118-.539 1.925-.539.74 0 1.353.198 1.837.594.49.41.736.913.736 1.507 0 .594-.15 1.07-.45 1.43-.294.352-.752.649-1.375.891l-1.485.572c-.286.11-.495.235-.627.374a.722.722 0 00-.187.517v.209zM16.692 14h-1.584l-1.32-1.716L12.556 14H10.96l2.079-2.662-2.08-2.695h1.596l1.309 1.716 1.243-1.716h1.584l-2.068 2.695L16.692 14z" fill="#112BD1"/></svg>`;

const icon2_5 = `<svg width="21" height="21" fill="none" xmlns="http://www.w3.org/2000/svg"><circle cx="10.5" cy="10.5" r="9.5" stroke="#112BD1" stroke-width="2"/><path d="M4.245 12.812h4.081V14H2.947v-1.221c0-.858.304-1.481.913-1.87.205-.132.444-.25.715-.352l1.463-.561c.323-.125.557-.279.704-.462.154-.19.231-.425.231-.704a.904.904 0 00-.297-.693c-.198-.19-.48-.286-.847-.286s-.67.088-.913.264c-.235.176-.37.436-.407.781H3.07c.014-.697.281-1.243.802-1.639.477-.36 1.118-.539 1.925-.539.74 0 1.353.198 1.837.594.491.41.737.913.737 1.507 0 .594-.15 1.07-.45 1.43-.294.352-.753.649-1.376.891l-1.485.572c-.286.11-.495.235-.627.374a.722.722 0 00-.187.517v.209zm5.868 1.287a.9.9 0 01-.649-.242.815.815 0 01-.253-.616c0-.25.085-.458.253-.627a.879.879 0 01.65-.253c.263 0 .483.084.66.253a.834.834 0 01.263.627c0 .25-.088.455-.264.616a.94.94 0 01-.66.242zm3.607-4.444a6.899 6.899 0 011.132-.099c.741 0 1.331.18 1.771.539.44.352.66.862.66 1.529 0 .726-.245 1.324-.736 1.793-.521.491-1.178.737-1.97.737-.784 0-1.411-.198-1.88-.594-.492-.41-.745-.975-.76-1.694h1.332c0 .374.124.649.373.825.25.176.569.264.958.264.388 0 .707-.125.957-.374a1.3 1.3 0 00.384-.957c0-.726-.49-1.089-1.474-1.089H12.29l.33-3.674h4.28v1.188h-3.026l-.153 1.606z" fill="#112BD1"/></svg>`;

const icon3_0 = `<svg width="21" height="21" fill="none" xmlns="http://www.w3.org/2000/svg"><circle cx="10.5" cy="10.5" r="9.5" stroke="#112BD1" stroke-width="2"/><path d="M7.432 7.895c-.8 0-1.265.304-1.397.913H4.671c.074-.704.36-1.236.858-1.595.47-.33 1.086-.495 1.848-.495.785 0 1.423.183 1.914.55.514.374.77.854.77 1.441 0 .367-.099.686-.297.957-.198.264-.491.462-.88.594.433.147.778.367 1.034.66.264.286.396.616.396.99s-.07.693-.209.957a1.953 1.953 0 01-.572.682c-.528.403-1.213.605-2.057.605-.806 0-1.485-.205-2.035-.616-.616-.447-.924-1.041-.924-1.782h1.386c.022.374.173.667.451.88.286.213.657.319 1.111.319.455 0 .807-.099 1.056-.297.257-.205.385-.462.385-.77 0-.315-.117-.561-.352-.737-.227-.176-.59-.264-1.089-.264H6.52V9.864h.814c.418 0 .745-.088.98-.264a.866.866 0 00.351-.715.914.914 0 00-.308-.715c-.198-.183-.506-.275-.924-.275zM16.783 14h-1.584l-1.32-1.716L12.647 14h-1.595l2.079-2.662-2.079-2.695h1.595l1.309 1.716 1.243-1.716h1.584l-2.068 2.695L16.783 14z" fill="#112BD1"/></svg>`;

const pdPlaybackRateControlCss = "poductivity-playback-rate-control button{border:none;padding:4px}poductivity-playback-rate-control button svg{vertical-align:middle}";

const PdPlaybackRateControl = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.playbackRateChange = createEvent(this, "playbackRateChange", 7);
    this.toggleRate = this.toggleRate.bind(this);
  }
  changed(playbackRate) {
    this.playbackRateChange.emit({ playbackRate });
  }
  toggleRate() {
    const rate = this.playbackRate < 3 ? this.playbackRate + 0.5 : 0.5;
    this.changed(rate);
  }
  /* istanbul ignore next */
  getIcon() {
    switch (this.playbackRate) {
      case 0.5:
        return icon0_5;
      case 1:
        return icon1_0;
      case 1.5:
        return icon1_5;
      case 2:
        return icon2_0;
      case 2.5:
        return icon2_5;
      case 3:
        return icon3_0;
      default:
        return String(this.playbackRate);
    }
  }
  render() {
    return (h("button", { class: "btn-playback-rate", title: "Toggle playback rate", onClick: this.toggleRate, innerHTML: this.getIcon() }));
  }
};
PdPlaybackRateControl.style = pdPlaybackRateControlCss;

const logoSvg = `<svg height="56.86" viewBox="0 0 75.38 56.86" width="75.38" xmlns="http://www.w3.org/2000/svg"><path d="m51.59 0v25.83c-1.08-10.35-8.55-18.79-18.41-21.33v21.76c-1.06-8.18-8.04-14.5-16.51-14.5-9.21 0-16.67 7.46-16.67 16.67s7.46 16.67 16.67 16.67c8.47 0 15.45-6.32 16.51-14.5v21.76c9.85-2.54 17.32-10.98 18.41-21.33v25.83c13.52-2.41 23.79-14.22 23.79-28.43s-10.27-26.02-23.79-28.43z" fill="#000000"/></svg>
`;

const iconPause = `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M8 7H11V17H8zM13 7H16V17H13z"/></svg>`;

const iconPlay = `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M7 6L7 18 17 12z"/></svg>`;

const iconSeekBack = `<svg width="21" height="24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M1.268 9.72c.257-.606.575-1.193.947-1.746l1.988 1.342a7.921 7.921 0 00-1.184 2.812 7.985 7.985 0 000 3.176c.104.501.257.996.46 1.476.195.463.439.913.724 1.334a7.978 7.978 0 002.13 2.128c.422.286.87.53 1.336.725.474.2.97.355 1.474.46 1.036.209 2.142.209 3.173 0a7.866 7.866 0 003.978-2.145 7.916 7.916 0 001.689-2.504c.2-.475.353-.97.458-1.475a7.95 7.95 0 00-.001-3.175 7.853 7.853 0 00-2.144-3.977 7.948 7.948 0 00-2.502-1.687 7.841 7.841 0 00-1.476-.458c-.128-.026-.258-.038-.388-.059v3.647L5.934 4.797 11.93 0v3.523c.291.035.583.075.87.134a10.208 10.208 0 013.673 1.547 10.275 10.275 0 014.318 6.443 10.357 10.357 0 010 4.139 10.23 10.23 0 01-1.546 3.673c-.365.54-.786 1.05-1.253 1.521a10.335 10.335 0 01-3.263 2.198 10.254 10.254 0 01-6.07.599 10.287 10.287 0 01-5.191-2.8 10.212 10.212 0 01-2.198-3.263 10.14 10.14 0 01-.6-1.93 10.364 10.364 0 010-4.137c.134-.655.337-1.303.598-1.927z" fill="#112BD1"/><path d="M5.795 10.956a3.185 3.185 0 001.452-.583h.979v7.183H6.928v-5.632a2.552 2.552 0 01-1.133.275v-1.243zm5.879 2.255a6.899 6.899 0 011.133-.1c.74 0 1.33.18 1.77.54.44.352.66.861.66 1.529 0 .726-.245 1.323-.736 1.793-.521.49-1.177.737-1.97.737-.784 0-1.411-.198-1.88-.594-.492-.411-.745-.976-.76-1.694h1.332c0 .374.124.649.374.825.249.176.568.264.957.264.388 0 .707-.125.957-.374a1.3 1.3 0 00.385-.957c0-.726-.492-1.09-1.474-1.09h-2.178l.33-3.673h4.279v1.188h-3.025l-.154 1.606z" fill="#112BD1"/></svg>`;

const iconSeekForward = `<svg width="22" height="24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M20.194 9.722a10.433 10.433 0 00-.946-1.747l-1.989 1.343a7.922 7.922 0 011.184 2.813 7.985 7.985 0 010 3.176 7.787 7.787 0 01-.46 1.476 7.684 7.684 0 01-.724 1.334 7.952 7.952 0 01-3.466 2.854c-.473.2-.97.355-1.474.46a8.075 8.075 0 01-3.173 0 7.86 7.86 0 01-3.978-2.145 7.922 7.922 0 01-1.689-2.505c-.2-.475-.354-.97-.458-1.475a7.95 7.95 0 01.458-4.65 7.859 7.859 0 011.687-2.503 7.95 7.95 0 012.502-1.688c.474-.2.972-.355 1.475-.458.129-.026.26-.038.39-.059v3.648l5.996-4.798L9.532 0v3.524a10.56 10.56 0 00-.87.133A10.207 10.207 0 004.99 5.205a10.274 10.274 0 00-3.72 12.512c.257.61.576 1.198.947 1.746.366.54.787 1.05 1.253 1.521a10.328 10.328 0 003.264 2.199c.619.261 1.268.464 1.93.6a10.418 10.418 0 004.14-.001 10.288 10.288 0 006.445-4.32c.374-.554.69-1.142.945-1.745.264-.624.467-1.273.6-1.93a10.362 10.362 0 000-4.138 10.42 10.42 0 00-.599-1.927z" fill="#112BD1"/><path d="M5.795 10.956a3.185 3.185 0 001.452-.583h.979v7.183H6.928v-5.632a2.552 2.552 0 01-1.133.275v-1.243zm5.879 2.255a6.899 6.899 0 011.133-.1c.74 0 1.33.18 1.77.54.44.352.66.861.66 1.529 0 .726-.245 1.323-.736 1.793-.521.49-1.177.737-1.97.737-.784 0-1.411-.198-1.88-.594-.492-.411-.745-.976-.76-1.694h1.332c0 .374.124.649.374.825.249.176.568.264.957.264.388 0 .707-.125.957-.374a1.3 1.3 0 00.385-.957c0-.726-.492-1.09-1.474-1.09h-2.178l.33-3.673h4.279v1.188h-3.025l-.154 1.606z" fill="#112BD1"/></svg>`;

class FontLoader {
  constructor() {
    this.loaded = [];
  }
  isLoaded(font) {
    return (this.loaded.findIndex((comp) => comp.family === font.family &&
      comp.weight === font.weight &&
      comp.style === font.style) > -1);
  }
  load(font) {
    if (!this.isLoaded(font)) {
      const style = this.getStyleElement(font);
      document.head.append(style);
      this.markLoaded(font);
    }
  }
  getStyleElement(font) {
    const el = document.createElement('style');
    el.innerHTML = this.getFontFaceRule(font);
    return el;
  }
  getFontFaceRule(font) {
    return `
    @font-face {
      font-family: '${font.family}';
      font-style: '${font.style}';
      font-weight: ${font.weight};
      font-display: swap;
      src: url('${font.woff2Url}') format('woff2');
      ${font.unicodeRange ? `unicode-range: ${font.unicodeRange};` : ''}
    }
    `;
  }
  markLoaded(font) {
    if (!this.isLoaded(font)) {
      this.loaded.push(font);
    }
  }
}
const FontLoader$1 = new FontLoader();

const karlaUnicodeRange = 'U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD';

const pdPlayerCss = ".pd-button{background:var(--poductivity-accent-alt, #61dcd2);outline:none;font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";color:var(--pd-text-color, #ffffff);font-size:0.9em;padding:4px 8px;display:inline-block;border-radius:3px;text-align:center;text-decoration:none;vertical-align:middle;cursor:pointer;border:none}.pd-button.pd-button-small{font-size:0.8em;padding:2px 4px}.pd-button.pd-button-large{padding:8px 12px}.pd-button-link{background:none;padding:0;border:none;display:inline-block;width:auto;color:var(--poductivity-accent, #112bd1);cursor:pointer;font-size:inherit}*{box-sizing:border-box}h1,h2,h3,h4,h5,h6{font-weight:400;margin:0 0 8px 0}input,button,textarea{font-family:inherit;color:inherit}input[disabled],button[disabled],textarea[disabled]{opacity:0.4}a{color:var(--poductivity-accent, #112bd1);text-decoration:none}a:hover,a:focus{text-decoration:underline}button{border:3px solid var(--poductivity-accent, #112bd1);color:var(--poductivity-accent, #112bd1);padding:0.25em 0.5em;background:transparent;border-radius:5px;outline:none}button:focus{background:#eee}.theme-dark.poductivity-player-wrapper{color:#fff}.theme-dark.poductivity-player-wrapper a{color:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper button{color:var(--poductivity-accent, #4fddd3);border-color:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper button:focus{background:#999}.theme-dark.poductivity-player-wrapper .main{border:none;background:#292C3D}.theme-dark.poductivity-player-wrapper .subscribe-link{border-right-color:#fff}.theme-dark.poductivity-player-wrapper .btn-controls{background:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper poductivity-playback-rate-control svg circle{stroke:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper poductivity-playback-rate-control svg path{fill:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper input.pd-input-range-control::-webkit-slider-thumb{background:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper input.pd-input-range-control::-moz-range-thumb{background:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper input.pd-input-range-control::-ms-thumb{background:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper input.pd-input-range-control::-moz-range-progress{background:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper input.pd-input-range-control::-ms-fill-lower{background:var(--poductivity-accent, #4fddd3)}.theme-dark.poductivity-player-wrapper input.pd-input-range-control::-webkit-slider-runnable-track{background:linear-gradient(var(--poductivity-accent, #4fddd3), var(--poductivity-accent, #4fddd3)) 0/var(--sx) 100% no-repeat var(--poductivity-muted, #c0c3d9)}.theme-dark.poductivity-player-wrapper poductivity-modal .pd-modal{background:#292C3D;color:#fff}.theme-dark.poductivity-player-wrapper poductivity-modal .pd-modal-slot{color:#fff}.theme-poductivity.poductivity-player-wrapper .main{border:none;background:#EFEFF0}.theme-poductivity.poductivity-player-wrapper .btn-controls{background:linear-gradient(to right, #112bd1 0%, #5569F1 100%)}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-ms-fill-lower{background:none}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-ms-thumb{background:var(--poductivity-accent-alt, #61dcd2)}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-moz-range-thumb{background:var(--poductivity-accent-alt, #61dcd2)}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-moz-range-progress{background:linear-gradient(to right, var(--poductivity-accent-alt, #61dcd2) 0%, var(--poductivity-accent, #112bd1) 100%)}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-webkit-slider-container{background:linear-gradient(to right, var(--poductivity-accent-alt, #61dcd2) 0%, var(--poductivity-accent, #112bd1) 100%);border-radius:2.5px}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-webkit-slider-thumb{background:var(--poductivity-accent-alt, #61dcd2)}.theme-poductivity.poductivity-player-wrapper input.pd-input-range-control::-webkit-slider-runnable-track{background:linear-gradient(var(--poductivity-muted, #c0c3d9), var(--poductivity-muted, #c0c3d9)) 0/var(--sx-rtl) 100% no-repeat rgba(0, 0, 0, 0);background-position:right}.theme-poductivity.poductivity-player-wrapper poductivity-modal .pd-modal{background:#EFEFF0}.hide-text{text-indent:100%;white-space:nowrap;overflow:hidden}@keyframes play-buffering{from{border-color:var(--poductivity-accent, #112bd1)}to{border-color:#cccccc}}:host{display:block;position:relative;min-width:250px;font-size:16px;font-family:\"Karla\", -apple-system, BlinkMacSystemFont, \"Segoe UI\", Roboto, Helvetica, Arial, sans-serif, \"Apple Color Emoji\", \"Segoe UI Emoji\", \"Segoe UI Symbol\";color:#292c3d}:host *{box-sizing:border-box}.poductivity-player-wrapper{display:flex;flex-direction:column}@media screen and (min-width: 520px){.poductivity-player-wrapper.horizontal{flex-direction:row-reverse;align-items:stretch;justify-content:center}}.art-wrapper{background-color:var(--poductivity-muted, #c0c3d9);background-size:cover;position:relative;overflow:hidden;border-radius:10px 10px 0 0}.art-wrapper:after{content:\"\";display:block;padding-bottom:100%}@media screen and (min-width: 520px){.horizontal .art-wrapper{flex:0 0 45%;margin-bottom:0;align-self:center;max-height:220px;max-width:220px;border-radius:0 10px 10px 0}}.main{position:relative;display:flex;flex-direction:column;padding:20px;overflow:hidden;background:#fff;border:1px solid var(--poductivity-muted, #c0c3d9);border-top:none;border-radius:0 0 10px 10px}@media screen and (min-width: 520px){.horizontal .main{flex:1 1 auto;border:1px solid var(--poductivity-muted, #c0c3d9);border-right:none;border-radius:10px 0 0 10px}}poductivity-client{position:absolute;top:0;left:0;height:100%;width:100%}.title{font-size:0.8em;margin-right:30px;text-transform:uppercase}.subtitle{font-size:1.2em;font-weight:700}.loading .subtitle{color:var(--poductivity-muted, #c0c3d9)}.subtext{font-size:0.8em;margin-bottom:8px}.subscribe-title{margin-right:4px}.subscribe-link{border-right:1px solid #292c3d;margin-right:4px;padding-right:4px}.subscribe-link:last-child{border-right:none}.notes-button{margin-right:4px}.subscribe-reveal{margin-right:4px}.controls{display:flex;align-items:center;justify-content:center;flex-wrap:wrap;margin-top:32px;margin-left:-8px}.controls>*{margin-left:8px}@media screen and (min-width: 520px){.horizontal .controls{margin-top:auto}}.btn-controls{width:40px;height:40px;border-radius:50%;flex:0 0 auto;background:var(--poductivity-accent, #112bd1);color:#fff;border:none;padding:0}.btn-controls:focus{background:var(--poductivity-accent, #112bd1);color:var(--poductivity-accent-alt, #61dcd2)}.btn-controls svg{vertical-align:middle;fill:#fff !important}.btn-controls svg path{fill:#ffffff !important}.btn-seek svg{margin-top:-2px}.btn-seek-back{margin-left:3em}@media screen and (min-width: 520px){.horizontal .btn-seek-back{margin-left:8px}}.btn-play{width:50px;height:50px}.buffering .btn-play{animation:play-buffering linear 0.6s infinite alternate}.btn-play svg{margin-left:2px;height:100%;width:auto}.btn-play.btn-pause svg{margin-left:0}.controls-main{flex:1 0 100%;margin-top:16px}@media screen and (min-width: 520px){.horizontal .controls-main{flex:1 1 auto;margin-top:12px}}.time{display:flex;font-size:0.8em;font-weight:700;justify-content:space-between;align-items:center;flex-wrap:wrap}@media screen and (min-width: 520px){.horizontal .time{flex-wrap:nowrap}}.time .time-current,.time .time-duration{flex:0 0 60px}.time .time-duration{text-align:right}.time poductivity-input-range{flex:1 0 auto;width:100%;order:999}@media screen and (min-width: 520px){.horizontal .time poductivity-input-range{width:auto;order:initial;margin:0 8px}}.buffering-message{opacity:0;transition:opacity 0.3s linear}.buffering .buffering-message{opacity:0.8}.pd-logo{position:absolute;width:1.5em;height:1.5em;opacity:0.8;transition:opacity 0.3s linear;top:16px;right:16px}.pd-logo svg{display:block;width:100%;height:100%}.pd-logo svg path{fill:var(--poductivity-accent-alt, #61dcd2)}.pd-logo:hover{opacity:1}";

const PdPlayer = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.pdPlayback = createEvent(this, "pdPlayback", 7);
    /**
     * Should the player be horizontal?
     */
    this.horizontal = false;
    /**
     * Specify the theme to use. Defaults to 'light'.
     */
    this.theme = 'light';
    /**
     * Use the Operating System's default UI font instead of
     * the Poductivity brand font
     */
    this.useOsFont = false;
    /**
     * Enable Debug mode. For internal use only
     */
    this.debug = false;
    this.episode = null;
    this.isPlaying = false;
    this.isBuffering = false;
    this.currentTime = 0;
    this.playbackRate = 1;
    this.isLoading = true;
    this.loadingFailed = false;
    this.playError = false;
    this.isSeeking = false;
    this.audio = null;
    this.client = null;
    this.notesModal = null;
    this.messagesElement = null;
    this.optOutControl = null;
    this.enablePrivacyControl = false;
    this.logger = new Logger('PdPlayer', this.debug);
    this.onDataLoading = this.onDataLoading.bind(this);
    this.onDataLoaded = this.onDataLoaded.bind(this);
    this.onDataLoadingFailed = this.onDataLoadingFailed.bind(this);
    this.onTimeUpdate = this.onTimeUpdate.bind(this);
    this.onPlaybackRateChanged = this.onPlaybackRateChanged.bind(this);
    this.onSeeked = this.onSeeked.bind(this);
    this.onSeeking = this.onSeeking.bind(this);
    this.togglePlay = this.togglePlay.bind(this);
    this.seekBack = this.seekBack.bind(this);
    this.seekForward = this.seekForward.bind(this);
    this.onPlay = this.onPlay.bind(this);
    this.onPlaying = this.onPlaying.bind(this);
    this.onBuffering = this.onBuffering.bind(this);
    this.onPause = this.onPause.bind(this);
    this.showEpisodeNotes = this.showEpisodeNotes.bind(this);
    this.hideEpisodeNotes = this.hideEpisodeNotes.bind(this);
  }
  componentWillLoad() {
    if (!this.useOsFont) {
      FontLoader$1.load({
        family: 'Karla',
        style: 'normal',
        weight: '200 800',
        unicodeRange: karlaUnicodeRange,
        woff2Url: getAssetPath('./assets/karla-latin-variable-wghtOnly-normal.woff2'),
      });
    }
  }
  componentDidLoad() {
    this.setEpisodeParams();
  }
  componentDidRender() {
    this.client.audioElement = this.audio;
    this.client.debug = this.debug;
    if (this.optOutControl) {
      this.client.optOutControl = this.optOutControl;
    }
    this.client.messagesElement = this.messagesElement;
  }
  get duration() {
    var _a;
    if ((_a = this.audio) === null || _a === void 0 ? void 0 : _a.duration) {
      return this.audio.duration;
    }
    if (this.episode) {
      return this.episode.duration;
    }
    return 0;
  }
  get durationFormatted() {
    return secsToHr(this.duration);
  }
  get currentTimeFormatted() {
    return secsToHr(this.currentTime);
  }
  get subtitle() {
    var _a;
    if (this.isLoading || this.loadingFailed) {
      return String.fromCharCode(160);
    }
    if (this.playError) {
      return 'Unable to play episode';
    }
    return ((_a = this.episode) === null || _a === void 0 ? void 0 : _a.title) || 'Untitled Episode';
  }
  get title() {
    var _a, _b;
    if (this.loadingFailed) {
      return 'Unable to load episode';
    }
    if (this.isLoading) {
      return 'Loading episode' + String.fromCharCode(8230);
    }
    return ((_b = (_a = this.episode) === null || _a === void 0 ? void 0 : _a.show) === null || _b === void 0 ? void 0 : _b.title) || 'Untitled Show';
  }
  get ready() {
    return !!this.episode && !this.isLoading && !this.loadingFailed;
  }
  get wrapperClasses() {
    const classes = {
      'poductivity-player-wrapper': true,
      loading: this.isLoading,
      buffering: this.isBuffering,
      horizontal: this.horizontal,
    };
    if (this.theme) {
      classes[`theme-${this.theme}`] = true;
    }
    return classList(classes);
  }
  /**
   * Begin or resume playback
   */
  async play() {
    var _a;
    this.registerMediaSession();
    try {
      // NETWORK_NO_SOURCE === 3
      if (this.episode && ((_a = this.audio) === null || _a === void 0 ? void 0 : _a.networkState) === 3) {
        this.logger.log('Initialising audio element');
        this.audio.load();
      }
      await this.audio.play();
      this.playError = false;
    }
    catch (e) {
      if ('call to pause'.indexOf(e.message) === -1) {
        this.logger.log('Unable to play', e);
        this.playError = true;
        this.isBuffering = false;
      }
    }
  }
  /**
   * Pause playback
   */
  async pause() {
    return this.audio.pause();
  }
  /**
   * Either play or pause playback, depending on the current playback state
   */
  async togglePlay() {
    return this.isPlaying ? this.pause() : this.play();
  }
  /**
   * Seek to a specific timestamp in seconds
   */
  async seekTo(timestamp, fastSeek = false) {
    const sanitized = (typeof timestamp === 'number' ? timestamp : parseFloat(timestamp)) || 0;
    // Explicitly set current time in case audio el isn't yet initialised
    this.currentTime = sanitized;
    if (fastSeek && 'fastSeek' in this.audio) {
      this.audio.fastSeek(sanitized);
    }
    else {
      this.audio.currentTime = sanitized;
    }
  }
  /**
   * Seek to a timestamp relative to the current playback position in seconds
   */
  async seekRelative(seconds) {
    const parsed = (typeof seconds === 'number' ? seconds : parseFloat(seconds)) || 0;
    this.seekTo(this.currentTime + parsed);
  }
  /**
   * Show the episode notes panel
   */
  async setPlaybackRate(playbackRate) {
    if (typeof playbackRate === 'number') {
      this.playbackRate = playbackRate;
      this.audio.playbackRate = this.playbackRate;
    }
  }
  /**
   * Show the episode notes panel
   */
  async showEpisodeNotes() {
    this.notesModal.show().catch(() => { });
  }
  /**
   * Hide the episode notes panel
   */
  async hideEpisodeNotes() {
    this.notesModal.hide(true);
  }
  async setEpisodeParams() {
    if (this.episodeId) {
      await this.client.setEpisodeById(this.episodeId);
    }
    else if (this.episodeGuid && this.feedUrl) {
      await this.client.setEpisodeByFeed(this.feedUrl, this.episodeGuid);
    }
  }
  onDataLoading() {
    this.isLoading = true;
    this.loadingFailed = false;
  }
  onDataLoaded(event) {
    this.episode = event.detail.episode;
    this.isLoading = false;
    this.loadingFailed = !event.detail.episode;
  }
  onDataLoadingFailed() {
    this.loadingFailed = true;
    this.isLoading = false;
  }
  async seekForward() {
    await this.seekRelative(15);
  }
  async seekBack() {
    await this.seekRelative(-15);
  }
  onTimeUpdate(e) {
    if (!this.isSeeking) {
      this.currentTime = e.target.currentTime || 0;
    }
    this.updateMediaSessionState();
  }
  onPlaybackRateChanged(e) {
    this.setPlaybackRate(e.detail.playbackRate);
    this.updateMediaSessionState();
  }
  onSeeking(e) {
    this.isSeeking = e.detail.isSeeking;
  }
  onSeeked(e) {
    this.seekTo(e.detail.value, true);
  }
  onPlay() {
    this.isPlaying = true;
    this.pdPlayback.emit({
      playing: this.isPlaying,
      currentTime: this.currentTime,
    });
    this.updateMediaSessionState();
  }
  onPlaying() {
    this.isBuffering = false;
  }
  onBuffering() {
    this.logger.log('Buffering');
    this.isBuffering = true;
  }
  onPause() {
    this.isPlaying = false;
    this.pdPlayback.emit({
      playing: this.isPlaying,
      currentTime: this.currentTime,
    });
    this.updateMediaSessionState();
  }
  onPlaybackError(error) {
    this.logger.log('Playback error', error);
    this.playError = true;
    this.isPlaying = false;
    this.updateMediaSessionState();
  }
  getMediaMetadata() {
    const data = {
      title: this.episode.title,
      artist: this.episode.show.title,
      album: 'Poductivity',
    };
    const image = this.episode.imageUrl || this.episode.show.imageUrl;
    const mime = guessImageMimeFromPath(image);
    if (image && mime) {
      data.artwork = [
        {
          src: image,
          sizes: '500x500',
          type: mime,
        },
      ];
    }
    return new MediaMetadata(data);
  }
  updateMediaSessionState() {
    if (navigator && 'mediaSession' in navigator) {
      navigator.mediaSession.metadata = this.getMediaMetadata();
      navigator.mediaSession.playbackState = this.isPlaying
        ? 'playing'
        : 'paused';
      navigator.mediaSession.setPositionState({
        duration: this.duration,
        playbackRate: this.playbackRate,
        position: this.currentTime,
      });
    }
  }
  mediaSessionSeekBackward(details) {
    this.seekRelative(-details.seekOffset || -10);
  }
  mediaSessionSeekForward(details) {
    this.seekRelative(details.seekOffset || 10);
  }
  mediaSessionSeekTo(details) {
    this.seekTo(details.seekTime || 0, !!details.fastSeek);
  }
  registerMediaSession() {
    this.addMediaSessionHandler('play', this.play);
    this.addMediaSessionHandler('pause', this.pause);
    this.addMediaSessionHandler('seekbackward', this.mediaSessionSeekBackward);
    this.addMediaSessionHandler('seekforward', this.mediaSessionSeekForward);
    this.addMediaSessionHandler('seekto', this.mediaSessionSeekTo);
  }
  addMediaSessionHandler(action, handler) {
    try {
      navigator.mediaSession.setActionHandler(action, handler.bind(this));
    }
    catch (e) {
      this.logger.log(`The media session action, ${action}, is not supported`);
    }
  }
  cssVars() {
    return `
    :host {
      ${this.accentColor ? `--poductivity-accent: ${this.accentColor};` : ''}
    }
    `;
  }
  render() {
    var _a, _b, _c, _d, _e, _f, _g;
    const nodes = [];
    nodes.push(h("style", null, this.cssVars()));
    nodes.push(h("div", { class: this.wrapperClasses }, h("div", { class: "art-wrapper", style: this.episode &&
        (this.episode.imageUrl || this.episode.show.imageUrl)
        ? {
          backgroundImage: `url(${this.episode.imageUrl || this.episode.show.imageUrl})`,
        }
        : {} }, h("poductivity-client", { ref: (el) => {
        this.client = el;
      }, timestamp: this.currentTime, onLoadingEpisodeData: this.onDataLoading, onLoadedEpisodeData: this.onDataLoaded, onLoadingEpisodeDataFailed: this.onDataLoadingFailed, debug: this.debug })), h("div", { class: "main" }, h("audio", { ref: (el) => {
        this.audio = el;
      }, preload: "none", controls: false, onPlay: this.onPlay, onPlaying: this.onPlaying, onPause: this.onPause, onWaiting: this.onBuffering, onTimeUpdate: this.onTimeUpdate, onError: this.onPlaybackError }, h("source", { type: "audio/mpeg", src: (_a = this.episode) === null || _a === void 0 ? void 0 : _a.mediaUrl })), h("h1", { class: "title" }, ((_c = (_b = this.episode) === null || _b === void 0 ? void 0 : _b.show) === null || _c === void 0 ? void 0 : _c.websiteUrl) ? (h("a", { href: (_e = (_d = this.episode) === null || _d === void 0 ? void 0 : _d.show) === null || _e === void 0 ? void 0 : _e.websiteUrl, target: "_blank" }, this.title)) : (this.title)), h("h2", { class: "subtitle" }, h("poductivity-marquee", { active: this.isPlaying }, this.subtitle)), h("div", { class: "subtext" }, h("button", { class: "pd-button-link notes-button", onClick: this.showEpisodeNotes }, "Episode description"), this.enablePrivacyControl && (h("poductivity-opt-out-control", { ref: (el) => {
        this.optOutControl = el;
      }, up: true }))), h("poductivity-modal", { ref: (el) => {
        this.notesModal = el;
      }, absolute: true, fill: true, debug: this.debug }, h("p", { class: "subtitle" }, (_f = this.episode) === null || _f === void 0 ? void 0 : _f.title), h("div", { innerHTML: ((_g = this.episode) === null || _g === void 0 ? void 0 : _g.description) || 'This episode has no notes.' })), h("div", { class: "controls" }, h("button", { class: "btn-controls btn-seek btn-seek-back", title: "Seek back", disabled: !this.ready, onClick: this.seekBack, innerHTML: iconSeekBack }), h("button", { class: 'btn-controls btn-play ' + (this.isPlaying ? 'btn-pause' : ''), title: this.isPlaying ? 'Pause' : 'Play', disabled: !this.ready, onClick: this.togglePlay, innerHTML: this.isPlaying ? iconPause : iconPlay }), h("button", { class: "btn-controls btn-seek btn-seek-forward", title: "Seek forward", disabled: !this.ready, onClick: this.seekForward, innerHTML: iconSeekForward }), h("poductivity-playback-rate-control", { "playback-rate": this.playbackRate, onPlaybackRateChange: this.onPlaybackRateChanged }), h("div", { class: "controls-main" }, h("poductivity-input-range", { min: 0, max: this.duration, step: 0.01, value: this.currentTime, disabled: !this.ready, onInputRangeChanged: this.onSeeked, onInputRangeInteracting: this.onSeeking }), h("div", { class: "time" }, h("span", { class: "time-current" }, this.currentTimeFormatted), h("span", { class: "buffering-message", "aria-live": "off" }, "Buffering\u2026"), h("span", { class: "time-duration" }, this.durationFormatted)))), h("a", { href: "https://poductivity.com", target: "_blank", class: "pd-logo hide-text", innerHTML: logoSvg }))));
    // eslint-disable-next-line @stencil/render-returns-host
    return nodes;
  }
  static get assetsDirs() { return ["assets"]; }
  static get watchers() { return {
    "episodeGuid": ["setEpisodeParams"],
    "episodeId": ["setEpisodeParams"],
    "feedUrl": ["setEpisodeParams"]
  }; }
};
PdPlayer.style = pdPlayerCss;

export { PdInputRange as poductivity_input_range, PdPlaybackRateControl as poductivity_playback_rate_control, PdPlayer as poductivity_player };
